#!/usr/bin/perl -w

# This script reads soas_main.f and soas_sodrfits.f,
# extracts names for potential commands and writes a compat/commands.c
# file containing this information.

=pod
   read_commands.pl: turns comments from fortran files into a command list
   Copyright (C) 2007, 2008 Vincent Fourmond

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
=cut


my %commands;

open MAIN, "soas_main.f";
my $comments = "";
while(<MAIN>) {
    if(/^\s*
(?:else)?if\s*\(\(ileng....\d+\).and.#biniou
    \(ch\(1:\d+\).eq.'([^']+)'/x) {
	my $c = $1;
	$c =~ s/\s+$//;
	$commands{$c} ||= $comments unless ($c =~ /fit2?_|dl_/);
    }
    elsif(/^\s*
(?::|(?:else)?if\s*\s*\(+)\s*
(?:\(+ileng.eq.\d+\)+.and.)?\s*
\(+ch\(1:\d+\)+\s*.eq.\s*'([^']+)'\s*\)+\s*
(?:.or.|\)\)\s*then)\s*(?:!\s*(.+))?/x
) {
	my $c = $1;
	$comments = $2 if($2);
	$c =~ s/\s+$//;
	$commands{$c} ||= $comments;
    }
    elsif(/^C\s*(.*)/i) {
	$comments.= $1." ";
    }
    else {
	$comments = "";
    }
}
close MAIN;

open FITS, "soas_sodrfits.f";
$comments = "";
while(<FITS>) {
    if(/^\s*(?:else)?if\s*\(fitfcn\(1:lfitfcn\)\s*\.eq\.\s*'([^']+)'/) {
	my $c = $1;
	$c =~ s/\s+$//;
	$commands{"fit_$c"} ||= $comments;
    }
    elsif(/^C\s*(.*)/i) {
	$comments.= $1." ";
    }
    else {
	$comments = "";
    }
    # Strip out useless comments
    if($comments =~ /^\s*\$\$\$/) {
	$comments = "";
    }
}
close FITS;

open FITS, "soas_double_layer.f";
$comments = "";
while(<FITS>) {
    if(/^\s*(?:else)?if\s*\(fitfcn\(1:lfitfcn\)\s*\.eq\.\s*'([^']+)'/) {
	my $c = $1;
	$c =~ s/\s+$//;
	$commands{"dl_$c"} ||= $comments;
    }
    elsif(/^C\s*(.*)/i) {
	$comments.= $1." ";
    }
    else {
	$comments = "";
    }
    # Strip out useless comments
    if($comments =~ /^\s*\$\$\$/) {
	$comments = "";
    }
}
close FITS;

if ( -f "soas_sodrfits2.f" ) {
    open FITS, "soas_sodrfits2.f";
    $comments = "";
    while(<FITS>) {
	if(/^\s*(?:else)?if\s*\(fitfcn\(1:lfitfcn\)\s*\.eq\.\s*'([^']+)'/) {
	    my $c = $1;
	    $c =~ s/\s+$//;
	    $commands{"fit2_$c"} ||= $comments;
	}
	elsif(/^C\s*(.*)/i) {
	    $comments.= $1." ";
	}
	else {
	    $comments = "";
	}
	# Strip out useless comments
	if($comments =~ /^\s*\$\$\$/) {
	    $comments = "";
	}
    }
    close FITS;
}

open COMMANDS, "> compat/commands.c";

print COMMANDS "#include \"commands.h\"\n".
    "command_t commands[] = {\n";
# Now sorting case-insensively.
for my $c (sort {uc($a) cmp uc($b)} (keys(%commands))) {
    $commands{$c} =~ s/\s+$//;
    print COMMANDS "  { \"".$c."\",\"".
	$commands{$c}."\"},\n";
}
print COMMANDS "  {0,0}};\n";

# print COMMANDS "\n\ncommand_t fits[] = {\n";
# for my $c (sort(keys(%fits))) {
#     $fits{$c} =~ s/\s+$//;
#     print COMMANDS "  { \"".$c."\",\"".
# 	$fits{$c}."\"},\n";
# }
# print COMMANDS "  {0,0}};\n";


