C    soas_file.f: advanced functions for film loss compensation
C    Copyright (C) 2008 Vincent Fourmond
C
C    This program is free software; you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation; either version 2 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License along
C    with this program; if not, write to the Free Software Foundation, Inc.,
C    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


C     A routine to interactively estimate the film loss during
C     a chronoamperometric experiment.
C
C     The experiment is divided into chunks of the same size (for now)
C     in which the user is prompted for a way to choose film loss:
C     * none
C     * a given time constant
C     * a linear regression around the end of the element.
      subroutine estimate_film_loss(np,x,y, datachanged)
      parameter (N=2**14)
      real*4 x(*),y(*)
      integer np, ibeg, iend, i
      logical datachanged
      real*4 loss(N)            ! the film loss, since the beginning
      real*8 xu,yu
      real*4 x4,y4
      real kfilm
      real x_begin, x_end,a,b   ! linear regression variables
      integer i_begin, i_end, i_temp    ! idem
      real*4 yt(N)            ! the film loss, since the beginning

C     Some few external functions
      real*8 promptforfloat
      character*128 promptfor, dummy
      character code
      logical notdone
      logical partition_cancelled

      integer nb_parts, i_parts(N)
      integer cur_part
      

      datachanged = .false.

      call interactive_partition_data(np, x, y, nb_parts, i_parts,
     :     partition_cancelled)
      if(partition_cancelled) then
         return
      endif


      ibeg = 1
      loss(1) = 1                  ! loss starts at 1 (no initial loss)
      cur_part = 1

C     Don't use a 0-length last interval...
      if(i_parts(nb_parts).eq.nb) then
         nb_parts = nb_parts - 1
      endif

      do cur_part = 1,nb_parts
C     We scan until the end of the current segment
         ibeg = i_parts(cur_part)
         if (cur_part .eq. nb_parts) then
            iend = np
         else
            iend = i_parts(cur_part+1)
         endif
         do while((iend.le.np).and.(x(iend).lt.(x(ibeg) + delta_x)))
            iend = iend + 1
         enddo


         call plot(iend - ibeg, x(ibeg), y(ibeg), 10)

C     Draw some label
 200     format('dr text 0 1 "Interval ', I2,' out of ', I2,
     :        '" 2 /box 8')
         write(dummy, 200) cur_part, nb_parts
         call gr_exec1(dummy)
         call gr_out            ! GTVL\flush 

 300     format("Film loss estimation on interval", G12.4,
     :        " -> ", G12.4, "(", I2, "/", I2, ")")
         write(6, 300) x(ibeg), x(iend), cur_part, nb_parts
         write(6, *) "(0, mid) : no film loss"
         write(6, *) "(k) : enter manually k"
         write(6, *) "(l) : linear regression estimation"

         notdone = .true.
         do while(notdone)
            call gr_curs(xu,yu,x4,y4,code) ! get mouse
            if(code.eq.'Q') then
               write(6, *) "INFO: cancelling film loss estimation"
               return
            endif
            if((code.eq.'0').or.
     :           (code.eq.'&')) then ! no film loss on this interval
               kfilm = 0
               notdone = .false.
               write(6, *) "Choosing no film loss"
C     Updating the film loss array
               do i=ibeg,iend
                  loss(i) = loss(ibeg)
               enddo
            endif
            if(code.eq.'K') then ! manual setting of the film loss
               kfilm = promptforfloat('Film loss rate constant', 'f')
               notdone = .false.
               do i=ibeg,iend
                  loss(i) = loss(ibeg) * exp( - kfilm *(x(i) - x(ibeg)))
               enddo
            endif
            if(code.eq.'L') then ! linear regression fitting
C     We start the regression in the middle, more likely to be suitable.
               i_begin = (ibeg + iend)/2
               i_end = iend

               code = ' '
C     The following code comes from soas_log.f/reglin
               do while((code(1:1).ne.'&').and.(code(1:1).ne.'Q')) ! mid button or q
                  if ((code(1:1).eq.'^').or.(code(1:1).eq.'B')) then
                     x_begin=real(xu) ! left button
                     call locate(x,np,x_begin,i_begin)
                     if(i_begin.le.ibeg) i_begin = ibeg
                  endif
                  if ((code(1:1).eq.'*').or.(code(1:1).eq.'E')) then
                     x_end=real(xu) ! right button
                     call locate(x,np,x_end,i_end)
                     if(i_end.ge.iend) i_end = iend
                  endif
                  if (i_begin.gt.i_end) then ! Swap left and right
                     i_temp=i_begin
                     i_begin=i_end
                     i_end=i_temp
                  endif
                  x_begin = x(i_begin)
                  x_end = x(y_end)
                  a=reglin2(x,y,i_begin,i_end)
                  b=ord2(x,y,i_begin,i_end)
C     The film loss rate constant is the inverse of the time
C     necessary to reach 0 starting from the beginning of the
C     regression.
                  kfilm = -a/(a* x(i_begin) + b)
                  if ((code(1:1).ne.'&').and.(code(1:1).ne.'Q')) then
 301                 format("Current rate constant: ", G12.4)
                     write(6, 301) kfilm
                  endif
                  do i=ibeg,iend
                     yt(i)=b+a*x(i)
                  enddo
                  call plot(iend - ibeg,x(ibeg),
     :                 y(ibeg),1) ! plot data
                  call plot(iend - ibeg,x(ibeg),
     :                 yt(ibeg),0) ! linear regression
                  call plotdiff(x(ibeg),yt(ibeg),
     :                 y(ibeg),1,iend - ibeg,.false.) ! plot residue
                  call reclim(iend - ibeg,x(ibeg),
     :                 y(ibeg)) ! recover limits
                  call plotline(x(i_begin),-10.,x(i_begin),10.) ! plot lim. as...
                  call plotline(x(i_end),-10.,x(i_end),10.) ! ... vertical lines
c added CL
                  write(dummy, 200) cur_part, nb_parts
                  call gr_exec1(dummy)
                  call gr_out   ! GTVL\flush 
                  call gr_curs(xu,yu,x4,y4,code)
               enddo
               notdone = .false.
               code = ' '
               do i=ibeg,iend
                  loss(i) = loss(ibeg) * (b+a*x(i))/(b+a*x(ibeg))
               enddo
            endif

         enddo

         write(6,*) iend, ibeg, np, delta_x
      enddo
C     Now, y becomes the loss.
      do i=1,np
         y(i) = loss(i)
      enddo
      datachanged = .true.
      end
     

C     Called by estimate_film_loss to more-or-less interactively partition
C     the data into segments, whose beginning index is given by the array
C     i_parts, of size nb_parts (return values)
      subroutine interactive_partition_data(np,x,y, nb_parts,
     :     i_parts, cancelled)
      parameter (N=2**14)
      real*4 x(*),y(*)
      integer np
      integer nb_parts
      integer i_parts(N)
      logical cancelled

      logical notdone, auto_partition, tmp
      integer i
      character*128 promptfor
      character*128 filename
      real*8 promptforfloat, delta_x
      integer promptforint
      

      character*128 greg_command

C     For gr_curs:
      real*8 xu,yu
      real*4 x4,y4
      character code

C     Parameters for 'jump partitioning'
      integer i_min ! the minimal size of a partition
      integer i_avg ! the number of elements over which to average
      integer i_ignore ! the number of elements to ignore
                       ! after a jump
      real*4  jump_factor ! the factor for which a jump is
                          ! considered significant
      real*4  min_delta   ! the minimum magnitude of a significant delta

      i_min = 50
      i_avg = 10
      i_ignore = 10
      jump_factor = 2
      min_delta = abs(y(np) - y(1)) * 20/(np) ! reasonable default value...

      cancelled = .false.
      auto_partition = .false.

C     One part starting at index 1:
      nb_parts = 1
      i_parts(1) = 1


      notdone = .true.
      do while(notdone)

         call plot(np, x, y, 10)

C     Now, we draw the segments:
         call gr_exec1('lim = = 0 1') ! First, we reset Y limits to 0-1
         call gr_exec1('pen 12 /col 6') ! use another color:
         do i= 1, nb_parts
 200        format('dr re ', G12.4, '0 /user')
            write(greg_command, 200) x(i_parts(i))
            call gr_exec1(greg_command)
 201        format('dr line ', G12.4, '1 /user')
            write(greg_command, 201) x(i_parts(i))
            call gr_exec1(greg_command)
         enddo
C     switch back to usual pen ???
         call gr_exec1('pen 0') 
         call gr_out            ! GTVL\flush 

 300     format("INFO: currently ", I2, " partitions")
         write(6, 300) nb_parts
         
         if(auto_partition) then
            write(6, *) "Auto-partition data"
            write(6, *) "(q,o,y, mid) to accept"
            write(6, *) "Jump partition parameters"
 400        format("(i) minimum interval size: ", I2)
            write(6, 400) i_min
 401        format("(a) averaging over ", I2, " elements")
            write(6, 401) i_avg
 402        format("(j) jump factor: ", G12.4)
            write(6, 402) jump_factor
 403        format("(m) minimum delta: ", G12.4)
            write(6, 403) min_delta
         else
            write(6, *) "Partition data"
            write(6, *) "(f): fixed size division"
            write(6, *) "(j): jump-based estimation"
            write(6, *) "(s): save partition"
            write(6, *) "(l): load previously saved partition"
            write(6, *) "(q): cancel"
            write(6, *) "(y,o,mid): accept"
         endif
            
         call gr_curs(xu,yu,x4,y4,code) ! get mouse + keyboard events
         if(auto_partition) then
            if((code .eq. 'Q').or.(code .eq. 'O')
     :           .or.(code .eq. 'Y').or.(code .eq. '&')) then
               auto_partition = .false.
            elseif(code .eq. 'J') then
               jump_factor = promptforfloat("New jump factor: ", "n")
            elseif(code .eq. 'M') then
               min_delta = promptforfloat("New minimum delta: ", "n")
            elseif(code .eq. 'I') then
               i_min = promptforint("New minimum size "//
     :              "for an interval: ", "n")
            elseif(code .eq. 'A') then
               i_avg = promptforint("New size "//
     :              "for averaging: ", "n")
            endif
            call jump_partition_data(np,x,y,nb_parts,i_parts,
     :           i_min, i_avg, i_ignore, jump_factor, min_delta)
         else
            if(code.eq.'Q') then
               write(6, *) "INFO: cancelling segment partition"
               cancelled = .true.
               notdone = .false.
            elseif((code.eq.'Y').or.(code.eq.'O')
     :              .or.(code .eq. '&')) then
               notdone = .false.
            elseif(code.eq.'S') then
               filename = promptfor("Enter file name for saving:", "f")
               call save_partition_data(i_parts, nb_parts,filename)
            elseif(code.eq.'L') then
               filename = promptfor("Enter file name for loading:", "f")
               call load_partition_data(i_parts, nb_parts,filename, tmp)
               if(nb_parts.le.0) then
                  nb_parts = 1
                  i_parts(1) = 1
               endif
            elseif(code.eq.'F') then
               delta_x = promptforfloat("Delta X for each step: ", "n")

 301           format("INFO: partitioning with delta_x = ", G12.4)
               write(6, 301) delta_x
               nb_parts = 1
               i_parts(1) = 1
               do i=1,n
                  if (x(i).ge.(x(i_parts(nb_parts)) + delta_x)) then
                     nb_parts = nb_parts + 1
                     i_parts(nb_parts) = i
                  endif
               enddo
            elseif(code.eq.'J') then
               write(6, *) "INFO: Starting auto partitioning "
               auto_partition = .true.
               call jump_partition_data(np,x,y,nb_parts,i_parts,
     :              i_min, i_avg, i_ignore, jump_factor, min_delta)
            endif
         endif
         
      enddo

      end

C     Partitions a given dataset into segments according to the
C     given parameters
      subroutine jump_partition_data(np,x,y, nb_parts, i_parts,
     :     i_min, i_avg, i_ignore, jump_factor, min_delta)
      parameter (N=2**14)
      real*4 x(*),y(*)
      integer np
      integer nb_parts
      integer i_parts(N)

C     Parameters for the partitioning scheme
      integer i_min ! the minimal size of a partition
      integer i_avg ! the number of elements over which to average
      integer i_ignore ! the number of elements to ignore
                       ! after a jump
      real*4  jump_factor ! the factor for which a jump is
                          ! considered significant
      real*4  min_delta   ! the minimum magnitude of a significant delta


C     Temporary variables
      real*4 delta_running ! running average of deltas on the
                           ! current interval
      real*4 delta, avg_fact
      integer i
      logical starting_interval

C     Parts start at the beginning, of course ;-)...
      nb_parts = 1
      i_parts(1) = 1
      starting_interval = .true.
      avg_fact = 1/i_avg

      i = 1
      do while(i.le.(np-1))
         delta = y(i+1) - y(i)
         if(starting_interval) then
            delta_running = delta
            starting_interval = .false.
         else
            delta_running = delta_running * (1 - avg_fact)
     :           + delta * avg_fact
         endif

C     We create a new interval if the jump is delta is big compared
C     to the rest and compared to the minimum delta
         if((abs(delta) .ge. max(abs(jump_factor * delta_running),
     :        abs(min_delta))) .and.
     :        ((i - i_parts(nb_parts)) .ge. i_min))  then
            nb_parts = nb_parts + 1
            i_parts(nb_parts) = i
            starting_interval = .true.
            i = i + i_ignore
         endif
         i = i + 1
      enddo

      end


C     A routine to load partition data from a file
      subroutine load_partition_data(i_parts, nb_parts, filename,
     :     failed)
      parameter (N=2**14)
      integer nb_parts, ileng, tmp
      integer i_parts(N)
      logical failed, ex
      character*128  filename    ! of length ileng


 12   format('ERROR : file ',A,' not found')

      failed = .true.
C     From now on, the code is copied verbatim from load
      ileng=lnblnk(filename)
      if(ileng.ne.0) then
c     remove first and last blanks in filename
         tmp = 1
         do while ((filename(tmp:tmp).eq.' ').and.(tmp.lt.ileng)) 
            tmp = tmp + 1
         enddo
         filename(1:ileng-tmp+1)=filename(tmp:ileng)
         ileng = ileng-tmp+1
         
c     does the file exist ?
         inquire(file=filename(1:ileng),exist=ex) 
         if(ex) then           ! file found
            nb_parts = 0
            open(unit=1,file=filename(1:ileng),
     :           status='old',err=931) 
            do i=1,N
 100           format(I10)
               read(1,100,end=931,err=101) i_parts(nb_parts)
               nb_parts = nb_parts + 1
 101           continue
            enddo
 931        continue
         else 
            write(6,12) filename(1:ileng)
            nb_parts = 0
         endif
      else
         nb_parts = 0
      endif
      if(nb_parts.gt.0) failed=.false.
      
      return
      end

C     Save partition data in a format in which it can be loaded
C     later on. Overwrites the file without questions.
      subroutine save_partition_data(i_parts, nb_parts, filename)
      parameter (N=2**14)
      integer nb_parts, ileng, tmp
      integer i_parts(N)
      logical conf, ex
      character*128  filename    ! of length ileng


 12   format('ERROR : file ',A,' not found')

C     From now on, the code is copied verbatim from load
      ileng=lnblnk(filename)

      if(ileng.ne.0) then
c     remove first and last blanks in filename
         tmp = 1
         do while ((filename(tmp:tmp).eq.' ').and.(tmp.lt.ileng)) 
            tmp = tmp + 1
         enddo
         filename(1:ileng-tmp+1)=filename(tmp:ileng)
         ileng = ileng-tmp+1

         inquire(file=filename(1:ileng),exist=ex) 
         if (ex) then           ! file exists
 305        format('WARNING : file ',A,' already exists.')
            write(6,305) filename(1:ileng)
            call confirm('overwrite',9,conf,.false.)
         endif
         if ((conf).or.(.not.ex)) then 
            if(ex) then
 306           format('INFO : removing existing file ',A)
               write(6, 306) filename
               call system('rm '//filename)
            endif
            open(unit=1,file=filename(1:ileng),status='NEW',err=951) !
            do tmp=1,nb_parts
 100           format(I10)
               write(1, 100) i_parts(tmp)
            enddo
         else
            write(6,*) 'WARNING : partition data NOT saved.' 
         endif
      endif
      close(unit=1)

      return 

 951  continue
 952  format ('ERROR while opening ',A,
     :     '; file not saved!')
      write(6,952) filename(1:ileng)
      close(unit=1)

      end
