C    soas_filters.f: filters
C    Copyright (C) 1999-2008  Christophe Leger
C                  2007, 2008 Vincent Fourmond
C
C    This program is free software; you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation; either version 2 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License along
C    with this program; if not, write to the Free Software Foundation, Inc.,
C    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

************************************************************************

      subroutine filtering_netlib(np,x,y,sigma,order,power,gaussian) 
c     like filteringR but using netlib's real fft, see below

      parameter (N=2**14,tiny=0.)
c     main
      real*4 x(*),y(*),yold(N)  ! old to save data (to compute redidue)
      real*4 wsave(2*N+15)      ! for use by fftpack
      integer np,order          ! order=0/1/2:smooth differentiate one or twice
c     power spectrum
      real*4 f(N/2),w1(N/2),w2(N/2) ! f: freq, w1: raw data, w2: convoluted
c     baseline 
      real*4 sl1,sl2,y1,y2,x1,x2,delta,r(N) ! to compute the baseline
      real*4 a,b,c,d,yb(N)      ! to put the baseline
c     fft
      integer Nmin              ! the smallest power of two greater than np
      integer sign1,sign2       ! for bandpass filter
      real*4 sigma              ! lowpass filter
      real*4 pref,pi,wave,xx    ! for convolution
      real*4 freq(N),ysauv(N)   ! sauv=ffted+1 bls raw data
      logical gaussian,         ! if T, gaussian instead of box.
     :     band,                ! if T, band filter on
     :     recompute,           ! if T, need to redo filtering?
     :     onlyffted,           ! if T, do not display raw data
     :     showbaseline,        ! if T, show the baseline
     :     autocut,             ! if T, try and estimate value of filter size
     :     replot,              ! if T, the plot is erased and redrawn
     :     power,               ! if T, display power instead of residue
     :     expert,              ! expert/challenged mode
     :     exportfiles          ! for development only
c     mouse handling
      character*10 code
      real*8 xu,yu
      real*4 x4,y4
c     display
      character*3 fsz
c     
      common /expert/ expert
      save autocut
c
      exportfiles=.false.
c     

      if (np.le.16) return
c
 303  format(A)
 3    format(I3.3)
 309  format('ERROR : Unknown command ''',A,'''')
      sign1=1.
      sign2=2.
c     look for Nmin = smallest power of two greater than np
      j=1
      Nmin=2**j  
      do while ((Nmin.lt.np))
         j=j+1
         Nmin=2**j
      enddo
      if (sigma.eq.0) sigma=10.

c     submenu
      if (.not.expert) then
         write(6,303) 'use left/right buttons or d/i to'//
     :        ' decrease/increase filter size,' !
         write(6,303) 'use 1/2 to decrease/increase'//
     :        ' quickly filter size,' !
         write(6,303) 'use r to turn off/on'//
     :        ' the display of the Raw data,' !
         write(6,303) 'use p to toggle the display'//
     :        ' of the Power spectrum,' !
         write(6,303) '...  then left button to select'//
     :        ' frequency (lowpass),' !
         write(6,303) '...  or b to Bandpass,' !
         write(6,303) 'use c to switch between gaussian'//
     :        ' and heavyside Cutoff filters,' !
         write(6,303) 'use a for Automatic selection of cutoff..'
c        write(6,303) 'use z/f/s to switch to 0th-, 1st- or 2nd-order derivative' ! expert mode only.
c        write(6,303) 'use l to show the baseline' ! expert mode only.
         write(6,303) 'mid button or q to quit.' !
         if(.not.gaussian) write(6,303)
     :        'INFO : cutoff set to heavyside (hard)' !
      endif                     ! .not.expert

c     first check that points are evenly spaced
      call checkdx(np,x,y,dxmean) ! hereafter
c
c     before ffting the data, y must be put in a array of length 2**Nmin
c     padded with zeros between np+1 and Nmin.
c     to avoid discontinuity at 1 & np, a baseline yb is removed so that y-yb
c     and the derivative of y-yb are very small at 1 and np. (before padding)
c
c     the baseline can be seen as blue dashed line typing l in expert mode.
c     otherwise, it is hidden to the user.
c
c     only y-yb is fftted; once convoluted, the baseline or its derivative
c     is added again to get the result
c
c     NB: therefore, the power spectra shown is not that of the data,
c     but that of the data minus the baseline (which should contribute only at 
c     low frequency since this is a smooth baseline.
c
c     USES Netlib realfft routines, in soas_fftpack.f
c
c     remove baseline
      do i=1,np                 ! not so clever...
         r(i)=real(i)           ! to use reglin2()
         yold(i)=y(i)           ! to plot residue
      enddo
      x1=r(1)                   ! x_i values
      x2=r(np)
      xp=.08                    ! slopes estimated from first and last 8% data
      np01=int(np*xp)
      np02=int(np*(1.-xp))
      sl1=reglin2(r,y,1,np01)   ! sl_i values
      sl2=reglin2(r,y,np02,np)
      y1=sl1*x1+ord2(r,y,1,np01) ! y_i values
      y2=sl2*x2+ord2(r,y,np02,np)
c     let s find a, b, c & d such as yb=a+b*x+c*x**2+d*x**3 satisfies
c     yb(x1)=y1, ybPrime(x1)=sl1 (slope @ x1), etc...
c     x1, y1, sl1, x2, y2, sl2 have just been determined.
c     (y is the polynomial function such as current-y is both 0
c     and flat in x(1) and x(np) )
c     a b c and d are given by :
c     [              2        3  ] [   ]   [     ]
c     [ 1    x1    x1       x1   ] [ a ]   [ y1  ]
c     [                          ] [   ]   [     ]
c     [                        2 ] [   ]   [     ]
c     [ 0    1     2 x1    3 x1  ] [ b ]   [ sl1 ]
c     [                          ] [   ] = [     ]
c     [              2        3  ] [   ]   [     ]
c     [ 1    x2    x2       x2   ] [ c ]   [ y2  ]
c     [                          ] [   ]   [     ]
c     [                        2 ] [   ]   [     ]
c     [ 0    1     2 x2    3 x2  ] [ d ]   [ sl2 ]
c     and maple gives:
      delta=(-x1**3+3*x2*x1**2+x2**3-3*x1*x2**2)
      if (delta.ne.0) then      
c     baseline = third order polyn
         a=-(-y1*x2**3+3*y1*x1*x2**2+x1**2*x2**2*sl2-x1**2*x2**2*sl1-3*x
     :        1**2*x2*y2-x1**3*x2*sl2+x1**3*y2+x1*sl1*x2**3)/delta
         b=(2*x2**2*x1*sl2+x2**2*x1*sl1+6*x1*x2*y1-6*x1*x2*y2-2*x1**2*x2
     :        *sl1-x1**2*x2*sl2-x1**3*sl2+x2**3*sl1)/delta
         c=-(x2**2*sl2+2*x2**2*sl1+3*x2*y1-3*x2*y2-x2*x1*sl1+x2*x1*sl2-2
     :        *x1**2*sl2+3*x1*y1-3*x1*y2-x1**2*sl1)/delta
         d=(2*y1-x1*sl2-x1*sl1+x2*sl2+x2*sl1-2*y2)/delta
      else                      ! (just in case)
c     baseline = linear trend instead...
         if(expert) write(6,303)
     :        'WARNING in Filtering_NetLib(singular matrix)' !
         call ab(y,np,a,b)
         c=0.
         d=0.
      endif
      do i=1,np
c     compute the baseline
         yb(i)=(a+b*r(i)+c*r(i)**2+d*r(i)**3)
c     remove the  baseline
         y(i)=y(i)-yb(i)        !(a+b*r(i)+c*r(i)**2+d*r(i)**3)
      enddo
c     padding (np+1 to Nmin) with zeros
      do i=np+1,Nmin
         y(i)=0
      enddo
c     init wsave, only once, do no longer modify wsave
      call rffti(Nmin,wsave)
c     fft+1
      call rfftf(Nmin,y,wsave)  ! y now contains fft+1 of bl-subtracted data
c     the wrapping order, slightly different from NR's, mind the signs too.
c     is delta the time interval between 2 pts, N=Nmin>=np
c
c                b4 fft           after realft
c     
c     i=1        t=0              real, f=0; NOT STORED: imag(f=0)=0
c     i=2        t=delta          real, f=1/(N delta)
c     i=3        t=2delta         -imag, f=1/(N delta)
c     i=4        t=3delta         
c     (...)
c     j          t=(j-1)delta
c     (...)
c     i=2*k-2                     real, f=(k-1)/N                    
c     i=2*k-1                     -imag, f=(k-1)/N
c     (...)
c     i=N-1      t=(N-2)delta     imag, f=(N/2-1)/delta
c     i=N,       t=(N-1)delta     real, f=1/2; NOT STORED: imag(f=1/2)=0 

c     ysauv = baseline sub fft+1
      do i=1,Nmin               
         ysauv(i)=y(i)          
      enddo
c     compute frequencies in freq 
      freq(1)=0                 ! freq(1-Nmin), y, ysauv
      do k=2,Nmin/2
         do l=2*k-2,2*k-1
            freq(l)=real(k-1)/real(Nmin)
         enddo
      enddo
      freq(Nmin)=.5
c     power spectrum in w1(Nmin/2) (old data)
      pi=3.1415926535
      f(1)=0.   
      w1(1)=(2.*pi*freq(1))**(order)*y(1)**2 ! Im(F_0)=0
      do i=2,Nmin/2-1
         f(i)=freq(2*i-2)       ! f(1-Nmin/2), w1, w2
         pref=(2.*pi*f(i))**(order) 
         w1(i)=pref*(y(2*i-2)**2+y(2*i-1)**2)
      enddo
      f(Nmin/2)=.5
      w1(Nmin/2)=(2.*pi*0.5)**(order)*y(Nmin)**2 ! Im(F_N/2)=0
c     the above part is done only once

      if (autocut) then
         call autosigma(Nmin/2,f,w1,sigma,gaussian)
         write(6,303) 'INFO : filter guessed;'//
     :        ' type a again to disable this option' !
      endif

      code=' '                  ! mouse
      band=.false.              ! badpass
      recompute=.true.          ! fft-1 or just redisplay?
      replot=.true.             ! redisplay?
      onlyffted=.false.         ! do not shaw raw data
      showbaseline=.false.

      do while ((code(1:1).ne.'&').and.
     :     (code(1:1).ne.'Q'))  ! mid or q to quit
c     recompute
         if (recompute) then    ! convolution and fft-1
c     convolution
            if ((order.eq.0).or.(order.eq.2)) then
c     order = 0 (smooth) or 2 (second order derivative)
c     deal first with F_0 
               if (order.eq.2) then
                  y(1)=0.
               else             ! order=0
                  y(1)=ysauv(1)
               endif
c     and F_N/2
               if (order.eq.0) then
                  pref=1.
               else
                  pref=-1.*pi**2
               endif
c     lowpass F_N/2
               if (gaussian) then ! smooth cutoff (gauss)
                  wave=exp(-1.*.25*sigma*sigma/2.)
               else             ! hard cutoff (heavyside)
                  wave=tiny
                  if ((.5*sigma).lt.1.) wave=1.
               endif
c     bandpass F_N/2
               if (((abs(freq(i))*sign2).lt.1.)
     :              .and.((abs(freq(i))*sign1).gt.1.)) wave=tiny
c     convolve F_N/2
               y(Nmin)=ysauv(Nmin)*wave*pref
c     remaining coefficients
               if (order.eq.0) pref=1. ! zeroth order derivative
               do i=2,Nmin-1,2    ! 1 and 2 already done        
                  xx=(freq(i)*freq(i)) 
                  if (order.eq.2) pref=-1.*xx*4.*pi**2 ! second order deriv.
c     lowpass
                  if (gaussian) then ! smooth cutoff (gauss) 
                     wave=exp(-1.*xx*sigma*sigma/2.)
                  else          ! hard cutoff (heavyside)
                     wave=tiny
                     if ((abs(freq(i))*sigma).lt.1.) wave=1.
                  endif
c     bandpass
                  if (((abs(freq(i))*sign2).lt.1.)
     :                 .and.((abs(freq(i))*sign1).gt.1.)) wave=tiny
c     convolve
                  y(i)=ysauv(i)*wave*pref
                  y(i+1)=ysauv(i+1)*wave*pref ! 0???
               enddo
            elseif (order.eq.1) then 
c     deal first with F_0 and F_N/2
               y(1)=0.          ! swap imag and reel, but imag=0
               y(Nmin)=0.
c     remaining frequencies i=3,Nmin,2
c     order=1, 1st derivative
               pref=2.*pi       ! -2*pi?
               do i=2,Nmin-1,2        
                  xx=(freq(i)*freq(i))
c     lowpass
                  if (gaussian) then ! smooth cutoff (gauss)
                     wave=exp(-1.*xx*sigma*sigma/2.) ! first order deriv
                  else          ! hard cutoff (heavyside)
                     wave=tiny
                     if ((abs(freq(i))*sigma).lt.1.) wave=1.
                  endif
c     bandpass
                  if (((abs(freq(i))*sign2).lt.1.)
     :                 .and.((abs(freq(i))*sign1).gt.1.)) wave=tiny
c     convolve, \times -1 cause swap Real and -Imag.
c                 y(i)=freq(i)*ysauv(i+1)*wave*pref[*-1]
                  y(i)=-1.*freq(i)*ysauv(i+1)*wave*pref ! 0???
c                 y(i+1)=-1.*freq(i)*ysauv(i)*wave*pref[*-1]
                  y(i+1)=freq(i)*ysauv(i)*wave*pref
               enddo
            endif               ! order=
c     new power spectrum in w2 
            w2(1)=(2.*pi*freq(1))**(order)*y(1)**2 ! Im(F_0)=0
            do i=2,Nmin/2-1
               pref=(2.*pi*f(i))**(order)                
               w2(i)=pref*(y(2*i-2)**2+y(2*i-1)**2)
            enddo
            w2(Nmin/2)=(2.*pi*0.5)**(order)*y(Nmin)**2 ! Im(F_N/2)=0
c     fft(-1)
            call rfftb(Nmin,y,wsave)
c     normalization
            do i=1,Nmin
               y(i)=y(i)/real(Nmin)
            enddo 
c     put back baseline (or order^th derivative of the baseline)
            if (order.eq.0) then ! put back baseline
               do i=1,np
                  y(i)=y(i)+(a+b*r(i)+c*r(i)**2+d*r(i)**3) 
               enddo   
            elseif (order.eq.1) then ! add derivative of bl
               do i=1,np
                  y(i)=y(i)+(b+2.*c*r(i)+3.*d*r(i)**2) 
               enddo   
            elseif (order.eq.2) then ! add 2nd deri of bl
               do i=1,np
                  y(i)=y(i)+(2.*c+6.*d*r(i)) 
               enddo   
            endif               ! (order=?)
c     put unit
            if ((order.eq.1).and.(dxmean.ne.0)) then
               do i=1,np
                  y(i)=y(i)/dxmean
               enddo
            elseif ((order.eq.2).and.(dxmean.ne.0)) then
               do i=1,np
                  y(i)=y(i)/dxmean**2
               enddo
            endif
         endif                  ! (recompute)
c     display
         if (replot) then
            if (order.eq.0) then ! smoothing
               if (exportfiles) then
c     write out all files for making figure
                  call soas_system('rm exportfiles.dat')
                  open (unit=3,file='exportfiles.dat',status='new')
                  write(3,*)'!data'
                  do i=1,np
                     write(3,*) x(i),y(i),yold(i)
                  enddo
                  write(3,*)'!power'
                  do i=1,Nmin/2
                     write(3,*) f(i),w1(i),w2(i)
                  enddo
                  close(3)
               endif
c     plots
c     main box
               if(onlyffted) then
                  call plot(np,x,y,2)
                  call showfiltersize(sigma,x,np)
               else 
                  call plot(np,x,yold,1) ! old, yellow  
                  call showfiltersize(sigma,x,np)
                  call plot(np,x,y,0) ! new, blue
                  if (showbaseline) then
c     draw baseline (dashed), just to check, only in expert mode
                     call gr_exec1('pen /da 2')
                     call plot(np,x,yb,0)
                     call penreset
                  endif
               endif            ! onlyfft
c     lower box
               if ((power)) then
                  call plot_power(Nmin/2,f,w1,w2) ! power spectrum
               else
                  call plotdiff(x,y,yold,sigma,np,.false.) ! plot residue
               endif
            else                ! derivation
c     main box
               call plot(np,x,y,1)  
               call showfiltersize(sigma,x,np)
c     lower box
               if ((power)) then
                  call plot_power(Nmin/2,f,w1,w2) !
               else
                  call gr_exec('dr text 0 -1 "type p to see'//
     :                 'power spectra" 3 /box 1') 
               endif
            endif
c     text
            write(fsz,3) int(sigma) 
            call gr_exec('dr text 0 1 "Filter size  '//fsz(1:3)//
     :           '" 2 /box 8')   !
            if (gaussian) then       
               call gr_exec1('dr text 0 1 "Windowing function :'//
     :              ' Gaussian" 3 /box 7') !
            else
               call gr_exec1('dr text 0 1 "Windowing function :'//
     :              ' Box" 3 /box 7') !
            endif
         endif                  ! replot

c     mouse
         if (band) then
            call gr_exec1('dr text 0 -5. "   BAND SELECTION: '//
     :           'Left-click twice to select'//
     :           ' band or right to resume" 3 /box 1') !
         elseif (order.eq.0) then
            call gr_exec1('dr text 0 1 "Smoothing." 1 /box 9') !
         elseif (order.eq.1) then
            call gr_exec1('dr text 0 1 "First derivative." 1 /box 9') !
         elseif (order.eq.2) then
            call gr_exec1('dr text 0 1 "Second Derivative." 1 /box 9') !
         endif 
         recompute=.true.       ! will it be necessary to redo filtering?
         replot=.true.
         call gr_curs(xu,yu,x4,y4,code) ! get mouse
c     band selection
         if (band) then
            if (code(1:1).eq.'*') then ! right button, reset.
               sign1=1.
               sign2=2.
               if(.not.expert) write(6,303)
     :              'INFO : bandpass filter removed.' !
            elseif (code(1:1).eq.'^') then ! left button, band selection
               sign1=real(int(1./real(xu)))
               call gr_curs(xu,yu,x4,y4,code) ! second time
               if (code(1:1).eq.'^') then
                  sign2=real(int(1./real(xu)))
                  if (sign1.lt.sign2) then
                     xtmp=sign1
                     sign1=sign2
                     sign2=xtmp
                  endif
                  if (.not.expert) write(6,303)
     :                 'INFO : bandpass filter set' !
               else
                  write(6,309) code(1:1) ! unknown command
                  write(6,303) 'ERROR : bandpass filter removed.'
                  sign1=1.
                  sign2=2.               
                  code=' '
               endif
            else
               write(6,309) code(1:1) ! unknown command
               write(6,303) 'ERROR : bandpass filter removed'
               sign1=1.
               sign2=2.               
               code=' '
            endif
            band=.false.
         elseif (((code(1:1).eq.'^').and.(.not.power))
     :           .or.(code(1:1).eq.'D')) then ! decrease filter size
            sigma=sigma-1
         elseif ((code(1:1).eq.'*').or.(code(1:1).eq.'I')) then ! increase
            sigma=sigma+1
         elseif (code(1:1).eq.'1') then ! decrease filter size
            sigma=sigma-10
         elseif (code(1:1).eq.'2') then ! increase
            sigma=sigma+10
         elseif (code(1:1).eq.'P') then ! toggle power spectrum display
            power=.not.power
            recompute=.false.
         elseif (code(1:1).eq.'R') then ! toggle display of raw data
            onlyffted=.not.onlyffted
            recompute=.false.
         elseif (expert.and.(code(1:1).eq.'L')) then ! expert mode only
            showbaseline=.not.showbaseline
            recompute=.false.
         elseif (expert.and.(code(1:1).eq.'N')) then ! expert mode only, export files
            exportfiles=.not.exportfiles
            recompute=.false.
         elseif (power.and.((code(1:1).eq.'^'))) then ! 
            sigma=real(int(1./real(xu))) ! get sigma from mouse
         elseif (code(1:1).eq.'C') then ! change cutoff
            gaussian=.not.gaussian
         elseif (expert.and.(code(1:1).eq.'Z')) then ! expert mode only
            order=0             ! switch to zeroth derivative
         elseif (expert.and.(code(1:1).eq.'F')) then ! expert mode only
            order=1             ! switch to first derivative
         elseif (expert.and.(code(1:1).eq.'S')) then ! expert mode only
            order=2             ! switch to second rivative
         elseif ((code(1:1).eq.'A')) then ! auto cutoff
            autocut=.not.autocut
            if (autocut) then
               call autosigma(Nmin/2,f,w1,sigma,gaussian)
               if (.not.expert) write(6,303) 'INFO : filter guessed'!
            else
               recompute=.false. ! no action
               if (.not.expert) write(6,303)
     :              'INFO : automatic filter set to off' !
            endif
         elseif (power.and.(code(1:1).eq.'B')) then ! power mode only
            band=.not.band
            replot=.false.
            recompute=.false.
         elseif ((code(1:1).eq.'&').or.(code(1:1).eq.'Q')) then 
            continue            ! about to quit
         else
            if (.not.expert) write(6,309) code(1:1) ! unknown command
            recompute=.false.
            replot=.true.
         endif 
         if (sigma.lt.0) sigma=0 
         if (recompute) call wait 
      enddo                     ! exit if q or mid button.
      return
      end

c     *********************************************************************

      subroutine autosigma(np,f,w1,sigma,gaussian)
      parameter (N=2**(14-1))
      integer np
      real*4 f(N),w1(N),sigma,w2(N)
      logical gaussian,found
      w2(1)=w1(1)
      w2(np)=w1(np)
      do i=2,np-1
         w2(i)=(w1(i-1)+w1(i)+w1(i+1))/3. ! small filter
      enddo
      call SSORT (f,w2,np,2)
      found=.false.
      if (n.le.20) return
      j=10.
      do while ((found.eqv..false.).and.(j.lt.np-2))
         if (w2(j+1).gt.(w2(j))) then
            found=.true.
            sigma=real(int(1./f(j)))
         endif
         j=j+1
      enddo
      gaussian=.false.
      return
      end

c     **********************************************************************

      subroutine removespikes(np,x,y,datachanged,npr) 
      parameter (N=2**14,nvar=200,nprmax=5)
      logical datachanged,expert,conf
      integer np,npold
      real*4 x(*),y(*),increment(N),ave,var,yold(N),xold(N),sigma
      common /expert/ expert

 303  format(A)
      datachanged=.false.
      sigma=5.
      npr=min(npr,nprmax)
 304  format('INFO : looking for up to ',I1,'-points spikes.')
      if (.not.expert) then 
         write(6,303) 'WARNING: avoid using this'//
     :        ' if there is regular noise.' !
         write(6,304) npr
      endif

      npold=np
      do i=1,np
         yold(i)=y(i)
         xold(i)=x(i)
      enddo

c     Y spike

      do i=1,np-1
         increment(i)=abs(y(i+1)-y(i))
      enddo
      call aver(increment,np-1,ave,var)
      do i=npr,np-npr
         do nn=1,npr
            dylim=real(nn)*abs(ave)+real(nn)*real(nvar)*var
            if((abs(yold(i+nn)-yold(i)).gt.real(dylim))
     :           .and.(abs(yold(i+nn+1)-yold(i))
     :           .lt.real(dylim))) then 
c     nn pts spike
               datachanged=.true.
               do l=1,nn
                  y(i+l)= (real(nn-l+1)*yold(i)
     :                 + real(l)*yold(i+nn+1))/real(nn+1)
               enddo            ! l
            endif
         enddo                  ! nn
      enddo                     ! i

c     X spikes

      do i=1,np-1
         increment(i)=abs(x(i+1)-x(i))
      enddo
      call aver(increment,np-1,ave,var)
      do i=1,np-2
         dxlim=abs(ave)+real(nvar)*var
         if ((abs(xold(i+1)-xold(i)).gt.real(dxlim))) then 
c     nn pts spike
            datachanged=.true.
            x(i+1)= (x(i)+x(i+2))/2.
         endif
      enddo                    

      if (datachanged) then
         call plot(npold,xold,yold,1) ! yellow
         call plot(np,x,y,0)    ! blue
         call plotdiff(x,y,yold,sigma,np,.false.) ! difference
         call reclim(np,x,yold)
         call confirm('accept the changes',18,conf,.true.)
         if (.not.(conf)) then
            np=npold
            do i=1,np
               x(i)=xold(i)
               y(i)=yold(i)
            enddo
            write(6,303) 'INFO : data NOT changed'
         endif
      else
         write(6,303) 'INFO : data NOT changed (no spikes found)'
      endif
      return
      end

c     **********************************************************************

      subroutine remove_points_off(np,x,y) 
      parameter(N=2**14,n_max_pt_off=1)
      integer np,n_pt_off_found,imax
      real*4 x(*),y(*),dist(N),ave,adev,sdev,var,skew,curt,skmax,distm
      logical expert
      common /expert/ expert

 300  format ('INFO: ',I1,' point(s) removed.')
 303  format(A)

      if (np.le.2) return

      skmax=10.
      skew=skmax+1.
      n_pt_off_found=0

      do while((skew.gt.skmax).and.(n_pt_off_found.lt.n_max_pt_off))
         distm=0.
         do i=1,np-1
            dist(i)=sqrt((x(i+1)-x(i))**2+(y(i+1)-y(i))**2)
            if (dist(i).gt.distm) then
               distm=dist(i)
               imax=i
            endif
         enddo
         call moment(dist,np-1,ave,adev,sdev,var,skew,curt) ! in soas_numerical

         if (skew.gt.skmax) then
            n_pt_off_found=n_pt_off_found+1
            if (imax.eq.np-1) then
c     remove last point
               np=np-1
            elseif (dist(imax).gt.dist(imax+1)) then
c     remove point imax
               do i=imax,np-1
                  x(i)=x(i+1)
                  y(i)=y(i+1)
               enddo
               np=np-1
            else
c     remove point imax+1
               do i=imax+1,np-1
                  x(i)=x(i+1)
                  y(i)=y(i+1)
               enddo
               np=np-1
            endif
         endif
      enddo

      if (expert.and.(n_pt_off_found.ne.0)) then 
         write(6,300) n_pt_off_found
      endif

      return
      end

c     **********************************************************************

      subroutine checkdx(np,x,y,dxmean)
      parameter(N=2**14,dxmin=1.e-4)
      real x(*),y(*),y1(N),ave,adev,sdev,var,skew,curt,dxmean
      logical conf,expert

      common /expert/ expert
 303  format(A)
      do i=1,np-1
         y1(i)=x(i+1)-x(i)
      enddo
      call moment(y1,np-1,ave,adev,sdev,var,skew,curt) ! in soas_numerical
c     write(6,*) ave,adev,sdev,var,skew,curt
      if (sdev.gt.abs(ave/5.)) then
         if (.not.expert) write(6,*)
         write(6,303) 'WARNING: there''s something wrong'//
     :        ' with these data' !
         if (abs(skew).gt.2.) then
            write(6,303) 'Is there a point completely off?'
         else
            write(6,303) 'The data points seem not to be'//
     :           ' evenly spaced against time.' !
         endif
         if (abs(ave).lt.dxmin) write(6,303) 'That seems to'//
     :        ' be because you are fft-ing an entire scan.' !
         if (abs(ave).gt.dxmin) then
            conf=.false.
            call confirm('interpolate the data',20,conf,.true.) !
            if (conf) then
               call tri(np,x,y)
               call interpol(np,x,y) 
               write(6,303) '... Done.'
            else
               write(6,303) 'Alright then. I was just asking.'
            endif
         endif
      endif
      dxmean=0.
      do i=1,np-1
         dxmean=dxmean+x(i+1)-x(i)
      enddo
      dxmean=dxmean/real(np-1)
      return
      end


      subroutine deletedatapoints(np,x,y,datachanged)
      parameter (N=2**14)
      character*1 code
      real*8 xu,yu,xnorm,ynorm
      real*4 x4,y4,dist,distmin,xnear,ynear
      real*4 x(*),y(*)
      real*4 xdel(N),ydel(N)
      integer np,idel(N),ndel
      logical datachanged,expert
      common /expert/ expert
 303  format (A)
 309  format('ERROR : Unknown command ''',A,'''')
 310  format(G12.4)
      code(1:1)=' '
      call plot(np,x,y,1)
      call gr_exec1('poi')
      if (.not.expert) then
         write(6,303) 'use left button or d to delete a data point' !
         write(6,303) 'use right button or b to put it back' !
         write(6,303) 'use midle or q to Quit' !
      endif

      xnorm=0
      ynorm=0
      do i=1,np
         if (abs(x(i)).gt.xnorm) xnorm=abs(x(i))
         if (abs(y(i)).gt.ynorm) ynorm=abs(y(i))
      enddo
      ndel=0

      do while((code(1:1).ne.'&').and.(code(1:1).ne.'Q')
     :     .and.(np.gt.1))      ! right button

         call gr_curs(xu,yu,x4,y4,code)

         if ((code(1:1).eq.'^').or.(code(1:1).eq.'D')) then ! left=delete
            xnear=real(xu)
            ynear=real(yu)

            distmin=1e9
            do i=1,np
               dist=(  (x(i)/xnorm-xnear/xnorm)**2 
     :              + (y(i)/ynorm-ynear/ynorm)**2 ) ! if nor normalization, there may be a pb with the fact that x and Y numbers may be orders of magnitude appart.
               if (dist.lt.distmin) then
                  i_near=i
                  distmin=dist
               endif
            enddo
            
         elseif ((code(1:1).eq.'*').or.(code(1:1).eq.'B')) then ! put back
            continue

         elseif((code(1:1).eq.'&').or.(code(1:1).eq.'Q')) then ! about to quit
            continue            ! about to quit
         else
            if (.not.expert) write(6,309) code(1:1)
         endif


         if ((code(1:1).eq.'^').or.(code(1:1).eq.'D')) then

c     store
            ndel=ndel+1
            idel(ndel)=i_near
            xdel(ndel)=x(i_near)
            ydel(ndel)=y(i_near)
c     remove
            do i=i_near,np-1
               x(i)=x(i+1)
               y(i)=y(i+1)
            enddo
            np=np-1
            datachanged=.true.

         elseif ((code(1:1).eq.'*').or.(code(1:1).eq.'B')) then !putting it back
            if (ndel.ge.1) then
               do i=np,idel(ndel)+1,-1
                  x(i)=x(i-1)
                  y(i)=y(i-1)
               enddo
               x(idel(ndel))=xdel(ndel)
               y(idel(ndel))=ydel(ndel)
               ndel=ndel-1
            else
               write(6,*) 'No more deleted data points'
            endif

         endif

         call plot(np,x,y,1)
         call gr_exec1('poi')
      enddo

      return
      end

