C    soas_main.f: the main SOAS program.
C    Copyright (C) 1999-2008  Christophe Leger
C                  2007, 2008 Vincent Fourmond
C
C    This program is free software; you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation; either version 2 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License along
C    with this program; if not, write to the Free Software Foundation, Inc.,
C    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

c     to add format
c     -ff option in soas_main
c     loadformatfromfile() in soas_file
c     soas_file: nothing
c     document doc/soas.tex

c     S(ex) O(n) A S(tick) 

      parameter (Npmax=2**14,Nstack=10,nc=2,nlstack=9)
c     2*Nstack-1 data (2**14 pairs of points each)
c     and their filenames ares stored in the stack
c     Npmax could be increased to 2**15 (changing header of MOST of the subs)
c     Take care that max number of pts for GreG is 50000<2**16,
c     special plot procedures would be required to plot longer data
c     The over-dimension of arrays is a waste of memory, but not of CPU
      real*4 data(Npmax,2*Nstack-1,nc) ! data(,,1)=x, data(,,2)=y
      integer nps(2*Nstack-1)   ! number of point in each buffer, < Npmax
      character*128 filename(2*Nstack-1) ! filenames
      integer ilengname(2*Nstack-1) ! lengths of filenames

c     equivalence : the displayed data is the Nstack^th
c     x(1:np), y(1:np) are equivalent to data(1:nps(Nstack),Nstack,1) and ,2)
      equivalence (np,nps(Nstack))
      equivalence (x,data(1,Nstack,1))
      equivalence (y,data(1,Nstack,2))
c     name(1:leng) is equivalent to filename(1:ilengname(Nstack))(Nstack)
      integer leng
      character*128 name
      logical marked(2*Nstack-1)
      equivalence (name,filename(Nstack))
      equivalence (leng, ilengname(Nstack))

      character*128 arg          ! arguments when starting.
      character*128 ch           ! what to do next ?
      character*128 promptfor   ! The external function to do user interaction

      character*128 file
      character*128 logstack(nlstack) ! history list
      integer*4 logstackl(nlstack) ! logstack(i)(1:logstackl(i))
      character*4 r1,r2         ! used in subroutine baseline (soas_base)
      real*4 sigmaold,areaold,ylimold,mdx !
      logical conf              ! ``do you really wish to do...'' ?
      logical datachanged       ! exiting somes procedures
      logical ExpertModeOn      ! expert/challenged user mode
      logical gaussian          ! it=T gaussian windowing function if fft routines
      logical power             ! if=T display power spectra in fft routine
      integer order             ! used in fft/sg: 0=smooth, 1=1st deriv., etc
      character*128 editor       ! e.g. editor(1:leditor)='EditorName'
      integer leditor           !
      integer npr               ! spikes removal options
      integer iload             ! fit, load options
      character*128 directory    ! name of wd = directory(1:l1)
      integer l1,l2             ! 
      integer ich               ! function char*1 -> int
      integer iZoomOut          ! to expand Y axis during base line sub.
      integer n                 ! ???
      real ctemp                ! temperature
      logical l_stack           ! if=T, load stack when starting
      logical s_stack           ! if=T, save stack after compretion of every command
      logical check_pt_off      ! if=T, remove data pts off scale after loading
      character*128 listpattern  ! in load, save, browse
      integer llistpattern
      character*10 colformat
      logical textonly          ! no graphic window
      logical lastplotstack     ! true if last plot is stack
      logical colorson
      logical alternatecolors
      logical showstack
      logical backspacerequired
      logical rungv             ! Whether to run directly lp or gv on the
                                ! resulting postscript files.

c     Fortran, sweet fortran...
      common /stack/ data, nps, filename, ilengname
      common /mark/ marked
      common /ratio/ r1,r2
      common /expert/ ExpertModeOn
      common /edit/ editor, leditor
      common /zoom/ iZoomOut
      common /temperature/ ctemp
      common /lastplot/ lastplotstack
      common /showstack/ showstack
      common /colors/ colorson
      common /alternatecolors/ alternatecolors 
      common /backspacerequired/ backspacerequired
      common /rungv/ rungv

C     Whether or not to turn on debugging informations:
      logical debugon
      common /debug/ debugon

C     Whether or not to write fit results to the fits files.
      logical autoout, askout
      common /autooutput/ autoout, askout

C     String constants defined as functions in soas_strings.c
      character *128 datafile    ! Use of the datafile function
      character *128 homefile    ! Use of the homefile function
      character *128 banner      ! For displaying the banner
      character *128 browser     ! The browser to display online help
      external datafile, homefile, banner, browser


c     "soas help" returns possible options
      do i=1,iargc() 
         call getarg(i,arg)  
         if (arg.eq.'help') then  
            write(6,303) 'Options:'
            write(6,303) 'expert, -Eeditorname, save, load, -ffFORMAT'
            write(6,303) 'FORMAT can be one of:'//
     :           ' GPES, CH, EGGsave, EGGexport.'
            write(6,303) '-listpatternEXPRESSION, checkptoff'
            stop 
         endif  
      enddo    

      call soas_system('clear')      ! clear
                                                                ! text
                                                                ! window
      write(6,*) 

c     get home directory, and source directories
C      call GetHomeSoasd(home,lhome,SoasD,lSoasD)  ! in soas_misc
C     Not used anymore, see README.devel
c     test existence of ~/gregtmp and ~/.gag.dico
      call TestHome             ! in soas_misc

c     init
c     ... to use of macros in GAG_INIT
c     GAG_INIT is defined in ~/.gag.dico as SoasD(1:)

      call sic_opt('GreG',homefile('gregtmp/LOG'),.true.) !
      call load_GreG('interactive') ! to allow calls to GreG routines
c     symbols in GREG's interactive mode:
      call exec_greg('symbol q "exit"')
c     in interactive greg, h will start Gildas' help, not soas'
      call exec_greg('symbol h "help"')
c     using the default printer defined in 
c     root/gildas/_system_/bin/GreGPLOT
c     NON! las is the queue name
      call exec_greg('symbol p "hard /dev ps g /plot las exact"') 
      call initpageA4 ! default
c     look for textonly command line argument
      if (iargc().ge.1) write(6,303)
     :     'INFO : reading command line options...'
      textonly=.false.
      alternatecolors=.false.

      do i=1,iargc() 
         call getarg(i,arg)  
         if (arg.eq.'textonly') then 
            textonly=.true. 
            write(6,303) 'INFO : text only.'
         endif
         if (arg.eq.'alternatecolors') then 
            alternatecolors=.true. 
            write(6,303) 'INFO : using alternate colors.'
         endif
      enddo
c     call exec_greg('dev xl b'), device Xlandscape black
      if (.not.(textonly)) then
         if (.not.alternatecolors) then
            call gr_devi('xl b') 
         else !alternatecolors
            call gr_devi('xl w') 
         endif
      endif
         
c      
      call InitGreg             ! in soas_plot. default parameters for boxes.
      do j=1,2*Nstack-1         ! stack of data
         nps(j)=0
         ilengname(j)=0
         filename(j)=''
         marked(j)=.false.
      enddo
      do j=1,nlstack            ! stack of commands
         logstackl(j)=0
         do i=1,50
            logstack(j)(i:i)=' '
         enddo
      enddo

c     default settings
      call OpenPeakDefault      ! set default peak file. in soas_peak
      ylimold=1.                ! limiting current
      areaold=0.                ! area after base line subtraction
      sigmaold=10.              ! filter size
      gaussian=.true.           ! smooth cutoff is default, changed in common
      power=.false.             ! display of residual instead of spectra = def
      r1(1:4)='*0  '            ! default ratio lines displayed during b-l-sub
      r2(1:4)='*1  '            ! 
      ExpertModeOn=.false.      ! default set to crap user mode
      iZoomOut=0                ! def = normal limits
      leditor=0                 ! default editor is emacs
      editor=''                 !
      ctemp=293                 ! default temperature, to compute napp in reg()
      l_stack=.false.           ! changed with option loadstack
      s_stack=.false.           ! changed with option savestack
      listpattern='*'           ! default set as for the Free format
      llistpattern=1            ! but -ffFree must be used for 
      colformat=' '
      check_pt_off=.false.
      lastplotstack=.false.
      colorson=.false.
      showstack=.false.
      backspacerequired=.false. ! true for g77?
      autoout = .false.         ! Autooutput is necessarily false by default
      askout = .false.          ! the same applies for askout
C     No debugging information by default
      debugon = .false.

c     formats
 2    format(A)                 !format(q,a)
 303  format(A)
 306  format('Working directory is ',A,'. Type G to change it.')
 304  format('INFO : buffer is named ',A,' (',I4,' data points).')
 3042 format('INFO : buffer is unnamed (',I4,' points).')
 305  format('INFO : ',A,' set to ',A,'.')
 308  format('ERROR : Load a file first !')
 309  format('ERROR : Unknown command ''',A,'''')
 311  format('INFO : last limiting current was ',G12.4,'.')
 312  format('INFO : temperature set to ',G12.4)
C 313  format('WARNING : unrecognized option ''',A,'''.')
C 314  format('ERROR : cannot find macro ',A,'.')
C 315  format('INFO : executing macro ',A,'.') 
C 316  format('INFO : length of header set to ',I2,'.') 
 317  format('INFO : list pattern set to ',A,'.') 
 318  format('INFO : file format set to ',A,'.') 
C 321  format('INFO : column format set to ',A,'.')
 323  format('ERROR : unrecognized file format ',A,'; fatal.') 
 324  format('I am sorry, but you must tell me',
     :     ' what type of data files to expect.',
     :     '  For example, try soas -ffGPES if you',
     :     ' are an Autolab user, else see the documentation') 
 325  format('INFO: current buffer marked')
 326  format('INFO: current buffer unmarked')
 327  format('INFO: buffer ',I2, ' unmarked')
 328  format('INFO: buffer ',I2, ' marked')
 329  format('ERROR: no action taken')
 330  format('INFO: colors are now ON')
 331  format('INFO: colors are now OFF')
 340  format('INFO: now ON')
 341  format('INFO: now OFF')
c     main 

      if (.not.alternatecolors) call soas_system('clear') ! clear text window  
      write(6,*) 
      write(6,303) banner() !
      write(6,303) 'For Help, type h or see file: '// 
c 23/01/08
c     :     datafile('soas.html') ! 
     :'http://bip.cnrs-mrs.fr/bip06/'
c     read conf files, if any
c     'temperature.info'
      call readtemp(ctemp)       
c     'soas.ff'
      call loadformatfromfile(listpattern, llistpattern,
     :     colformat,check_pt_off) 

c     read argument(s) (if any)  
      if (iargc().ge.1) write(6,303)
     :     'INFO : reading command line options...'
      do i=1,iargc() 
         call getarg(i,arg)  
c     expert
         if (arg.eq.'expert') then   
            ExpertModeOn=.true.  
            write(6,305) 'user mode', 'expert' 
c     -EeditorName  
         elseif (arg(1:2).eq.'-E') then 
            editor=arg(3:lnblnk(arg)) 
            leditor=lnblnk(arg)-2  
            write(6,305) 'editor', editor(1:leditor) 
c     save
         elseif (arg.eq.'save') then 
            s_stack=.true.  
            write(6,303) 'INFO : stack will be saved.'   
c     load
         elseif (arg.eq.'load') then 
            l_stack=.true. 
            write(6,303) 'INFO : load stack.'
c     textonly
         elseif (arg.eq.'textonly') then 
c     textonly=.true. 
            write(6,303) 'INFO : text only.'
c     -ff
         elseif (arg(1:3).eq.'-ff') then
c     GPES
            if ((arg(4:lnblnk(arg)).eq.'GPES')
     :           .or.(arg(4:lnblnk(arg)).eq.'Gpes')) then
               listpattern='*.[oO]?? *.[Dd][Aa][Tt]'
               llistpattern=23
               colformat(1:4)='GPES'
               check_pt_off=.true.
               write(6,318) arg(4:lnblnk(arg))
c     CH
            elseif (arg(4:lnblnk(arg)).eq.'CH') then
               listpattern='*.[Tt][Xx][Tt] *.[Dd][Aa][Tt]'
               llistpattern=29
               colformat(1:2)='CH'
               check_pt_off=.false.
               write(6,318) arg(4:lnblnk(arg))
c     EGGsave
            elseif (arg(4:lnblnk(arg)).eq.'EGGsave') then
               listpattern='*.[Dd][Aa][Tt]'
               llistpattern=14
               colformat(1:7)='EGGsave'
               check_pt_off=.true.
               write(6,318) arg(4:lnblnk(arg))
c     EGGexport 
            elseif (arg(4:lnblnk(arg)).eq.'EGGexport') then
               listpattern='*.[Tt][Xx][Tt] *.[Dd][Aa][Tt]'
               llistpattern=29
               colformat='EGGexport'
               check_pt_off=.false.
               write(6,318) arg(4:lnblnk(arg))
c     unknown format, fatal
            else
               write(6,323) arg(4:lnblnk(arg))
               stop
            endif
c     -listpattern 
         elseif (arg(1:12).eq.'-listpattern') then
            if (llistpattern.eq.0) then 
               listpattern=arg(13:lnblnk(arg)) 
               llistpattern=lnblnk(arg)-12 
            else
               listpattern(1:llistpattern+1+lnblnk(arg)-12)
     :              =listpattern(1:llistpattern)//
     :              ' '//arg(13:lnblnk(arg))
               llistpattern=llistpattern+1+lnblnk(arg)-12
            endif
            write(6,317) listpattern(1:llistpattern)
c     checkptoff
         elseif (arg.eq.'checkptoff') then 
            check_pt_off=.true.
            write(6,303) 'INFO : argument checkptoff.'
c     paper size
         elseif ((arg.eq.'Letter').or.(arg.eq.'letter')) then 
            call initpageLetter
            call initgreg
            write(6,303) 'INFO : paper format = Letter.'
         endif
      enddo
c     
c     no file format specified, fatal
      if (colformat(1:1).eq.' ') then
         write(6,303) 'ERROR : no file format specified; fatal.'
         write(6,324)
         stop
      endif 
c
      if (l_stack) then
         call load_stack
         call plot(np,x,y,1)
      endif

      call GetCwdShort(directory,l1,l2) ! in soas_misc
      if(.not.expertModeOn) write(6,306) directory(1:l1) ! working dir is...
      call TestDirectory        ! in soas_misc
      call CheckBirthdayfromfile ! in soas_misc

      do while (.not.((ileng.eq.1).and.(ch(1:1).eq.'q')))
         
c     print menu if not expert
         if (.not.(ExpertModeOn)) then 
c            write(6,*)          ! A blank line 
            if  (np.ne.0) then
               if (leng.ne.0) then
                  write(6,304) name(1:leng),np ! buffer name is...
               else
                  write(6,3042) np
               endif
            endif
            write(6, 303) "Load (l) Save (s) Cut (c) "//
     :           "Filter (f) Base line non-cat/cat data (b/B),"
            write(6, 303) "peak search (1) Undo (u) Redo"//
     :           "(r) Help (h) Quit (q), show all commands (m)."
         endif
         
c     put promt and get choice in ch(1:1)
         continue
C     call prompt('Soas>')   ! in soas_misc
C     read(5,2,err=3) ch     !ileng,ch
         ch = promptfor('Soas>','c') ! Prompt for a command 
         ileng=lnblnk(ch)
 380     format('Input: ',A,' -- length: ',I2)
C        write(6, 380) ch(1:ileng), ileng

         if ((ileng.ne.0).and.  ! handling of history list in soas_buff
     :        (.not.((ileng.eq.4).and.(ch(1:4).eq.'hist')))) then
            if ((ileng.le.2).and.(ch(1:1).eq.'!')) then
               call retrieve_from_history(ch,ileng,logstack,logstackl)
            else
               call add_to_history(ch,ileng,logstack,logstackl)
            endif
         endif

         if (ileng.eq.0) then
            call gr_out  

C     FILES
C     Vincent Fourmond: modified so that we can now specify the 
C     File name on the command-line rather than having to select
C     it later on. You can therefore use now
C     
C     l biniou.dat
C     
C     for directly loading the biniou.dat file.

c     Load a file
         elseif ((ch(1:1).eq.'l').and.((ileng.le.2)
     :           .or.(ch(2:2).eq.' ').or.(ch(3:3).eq.' '))) then
            call un_re_do(0)    ! save, in soas_buff
            if (ileng.eq.2) then
               iload=ich(ch(2:2),1,1,3)
            else
               iload=1
            endif
            if (ileng.le.2) then
               call load(np,x,y,name,leng,datachanged,iload,
     :              listpattern, llistpattern,
     :              colformat)  ! in soas_file
            else                ! we grab the filename on the command-line
               if (ch(2:2).eq.' ') then
                  name = ch(3:ileng)
               else
                  name = ch(4:ileng)
               endif            ! yes, that isn't beautiful
               call load_only(np,x,y,name,leng,datachanged,iload,
     :              colformat)
            endif
            if (datachanged) then
               if (check_pt_off) call remove_points_off(np,x,y) 
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! new plot
               endif
            else                ! data not changed
               call un_re_do(2) ! ``unsave'', in soas_buff
            endif
            iload=1             ! reset default
c     Save
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'s')) then
            if (np.ne.0) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
               call save(np,x,y,name,leng,1,colformat) ! in soas_file
            else
               write(6,308)     ! load a file first
            endif
c     broWse ocw files 

C     broWse files in the current directory
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'W')) then 
            call browse(check_pt_off,
     :           listpattern, llistpattern,colformat) ! in soas_plot
C     Change by Vincent Fourmond: trying now to add a supplementary
C     argument to W to read arbitrary patterns
         elseif ((ileng.gt.2).and.(ch(1:2).eq.'W ')) then 
C     It seems to be safe to use the file variable for that
            file = ch(3:ileng)
            call browse(check_pt_off,
     :           file, lnblnk(file),colformat) ! in soas_plot
c     oVerlay file

C     load a file oVerlaying current output
         elseif ((ileng.le.2).and.(ch(1:1).eq.'v')) then ! overlay buffer
            if (np.ne.0) then
               if (ileng.eq.2) then
                  iload=ich(ch(2:2),1,1,3) ! (,def,max)
               else
                  iload=1
               endif
               call un_re_do(0) ! save, in soas_buff
               call load(np,x,y,name,leng,datachanged,iload,
     :              listpattern, llistpattern,colformat)
               if (datachanged) then 
                  if (check_pt_off) call remove_points_off(np,x,y) 
                  marked(nstack+1)=.true.
                  call plot(np,x,y,10) ! overlay new data
                  marked(nstack+1)=.false.
               else             ! data not changed
                  call un_re_do(2) ! unsave
               endif
            else
               write(6,308)     ! load a file first
            endif
c     oVerlay file, old version, does not recompute limits.
         elseif ((ileng.le.3).and.(ch(1:2).eq.'ov')) then ! overlay buffer
            if (np.ne.0) then
               if (ileng.eq.3) then
                  iload=ich(ch(3:3),1,1,3) ! (,def,max)
               else
                  iload=1
               endif
               call un_re_do(0) ! save, in soas_buff
               call load(np,x,y,name,leng,datachanged,iload,
     :              listpattern, llistpattern,colformat)
               if (datachanged) then 
                  call plot(np,x,y,0) ! overlay new data
               else             ! data not changed
                  call un_re_do(2) ! unsave
               endif
            else
               write(6,308)     ! load a file first
            endif
            
C     BUFFERS
            
c     Undo

C     Undo once. (un) (with n=1--9) undo n times.
         elseif ((ch(1:1).eq.'u').and.(ileng.le.2)) then
            if (ileng.eq.1) then
               call un_re_do(1) ! undo
            elseif (ileng.eq.2) then
               j=ich(ch(2:2),1,1,9)
               do k=1,j
                  call un_re_do(1) ! undo, in soas_buff
               enddo
            endif
            if (np.ne.0) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else                ! np=0 
               call gr_exec('clear') ! clear graphic window
            endif
c     Redo

C     Redo once. (rn) (with n=1--9) redo n times.
         elseif ((ch(1:1).eq.'r').and.(ileng.le.2)) then
            if (ileng.eq.1) then
               call un_re_do(-1) ! undo
            elseif (ileng.eq.2) then
               j=ich(ch(2:2),1,1,9)
               do k=1,j
                  call un_re_do(-1) ! redo, in soas_buff
               enddo
            endif
            if (np.ne.0) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else                ! np=0
               call gr_exec('clear') ! clear graphic window
            endif
c     oVerlay buffer
         elseif ((ileng.le.2).and.(ch(1:1).eq.'V')) then ! overlay buffer
            if (np.ne.0) then
               if (ileng.eq.2) then
                  iload=ich(ch(2:2),1,1,3) ! (,def,max)
               else
                  iload=1
               endif
               call OverlayBuffer ! in soas_buff
            else
               write(6,308)     ! load a file first
            endif
c     show buffer stacK
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'k')) then 
            n=0
            do i=1,2*Nstack-1
               n=n+nps(i)
            enddo
            if (n.eq.0) then    ! all buffers empty
               write(6,308)
            else
               call plotbuffers(.true.) ! in soas_buff
            endif
c     sWitch to any buffer
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'w')) then 
            call switch         ! in soas_misc
            if (np.ne.0) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            endif
c     chAnge buffer name
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'a')) then
            if (np.ne.0) then
               call ChangeBufferName(name,leng) ! in soas_buff
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
            endif
c     Save the entire stack into file soas.stack
         elseif ((ileng.eq.9).and.(ch(1:9).eq.'savestack')) then
            call save_stack
c     Load a stack from file soas.stack
         elseif ((ileng.eq.9).and.(ch(1:9).eq.'loadstack')) then
            write(6,303) 'WARNING : the current stack will be lost'
            call confirm('load stack',10,conf,.true.)
            if (conf) call load_stack
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1)
            endif

c     Prompts for film loss correction for chronoamperometry
         elseif ((ileng.eq.10).and.(ch(1:10).eq.'film_decay')) then
            if (np.ne.0) then
               call un_re_do(0)
               call estimate_film_loss(np,x,y, datachanged) ! in soas_film
               if (.not.datachanged) call un_re_do(2)
               call plot(np,x,y,1)
            else
               write(6,308)     ! load a file first
            endif

C     FITS, new, using sodr
         elseif ((ileng.gt.4).and.(ch(1:4).eq.'fit_')) then
            if (np.ne.0) then
               call un_re_do(0)
               call fit_y(np,x,y,name,leng,ch(5:ileng),
     :              ileng-4,datachanged) !
               if (.not.datachanged) call un_re_do(2)
               call plot(np,x,y,1)
            else
               write(6,308)     ! load a file first
            endif
c     soas_1 only
         elseif ((ileng.gt.5).and.(ch(1:5).eq.'fit2_')) then
            if (np.ne.0) then
               call un_re_do(0)
               call fit2_y(np,x,y,name,leng,ch(6:ileng),
     :              ileng-5,datachanged) !
               if (.not.datachanged) call un_re_do(2)
               call plot(np,x,y,1)
            else
               write(6,308)     ! load a file first
            endif
C     Double-layer-related stuff
         elseif ((ileng.gt.3).and.(ch(1:3).eq.'dl_')) then
            if (np.ne.0) then
               call un_re_do(0)
               call dl_do(np,x,y,name,leng,ch(4:ileng),ileng - 3,
     :              datachanged)
               if (.not.datachanged) call un_re_do(2)
               call plot(np,x,y,1)
            else
               write(6,308)     ! load a file first
            endif
            
C     SIMPLE HANDLING
            
c     Cut (remove) parts of the buffer
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'c')) then
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call cuts(np,x,y,datachanged) ! in soas_cut
               if (datachanged) then
                  call gpes2dat(name,leng)
               else 
                  call un_re_do(2) ! unsave, in soas_buff
               endif
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif
c     Insert buffer into the current one
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'i')) then
            n=0
            do i=1,2*Nstack-1
               n=n+nps(i)
            enddo
            if (n.eq.0) then    ! all buffers empty
               write(6,308)
            else
               call un_re_do(0) ! save, in soas_buff
               call InsertBuffer(np,x,y,datachanged) ! in soas_cut
               if (.not.datachanged) call un_re_do(2) ! unsave, in soas_buff
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! whichever new or old data
               endif
            endif
            
C     FILTERING

c     Remove spikes 
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'deldp')) then
            if (np.ne.0) then
               call plot(np,x,y,1) ! new plot
               call un_re_do(0) ! save, in soas_buff
               call deletedatapoints(np,x,y,datachanged) ! in soas_filters
               if (datachanged) then
                  call gpes2dat(name,leng)
                  if (showstack) then
                     call plotbuffers(.true.) ! in soas_buff
                  else
                     call plot(np,x,y,1)
                  endif
               else 
                  call un_re_do(2) ! unsave, in soas_buff
               endif
            else
               write(6,308)     ! load a file first
            endif
         elseif ((ileng.le.2).and.(ch(1:1).eq.'R')) then
            if (np.ne.0) then
               if (ileng.eq.2) then
                  npr=ich(ch(2:2),5,1,4)
               else
                  npr=5
               endif
               call plot(np,x,y,1) ! new plot
               call un_re_do(0) ! save, in soas_buff
               call RemoveSpikes(np,x,y,datachanged,npr) ! in soas_filter
               if (datachanged) then
                  call gpes2dat(name,leng)
                  if (showstack) then
                     call plotbuffers(.true.) ! in soas_buff
                  else
                     call plot(np,x,y,1)
                  endif
               else 
                  call un_re_do(2) ! unsave, in soas_buff
               endif
            else
               write(6,308)     ! load a file first
            endif
c$$$c     savitzky golay, version 1 only
c$$$         elseif ((ileng.le.3).and.((ch(1:3).eq.'sgf')
c$$$     :           .or.(ch(1:3).eq.'sgd').or.(ch(1:3).eq.'sgD'))) then
c$$$            if (np.ne.0) then
c$$$               if (ch(3:3).eq.'f') order=0 ! smooth
c$$$               if (ch(3:3).eq.'d') order=1 ! first deriv
c$$$               if (ch(3:3).eq.'D') order=2 ! second deriv
c$$$               call un_re_do(0) ! save, in soas_buff
c$$$               call savgolfilter(np,x,y,sigmaold,order)
c$$$               call gpes2dat(name,leng) ! data always changed
c$$$               if (showstack) then
c$$$                  call plotbuffers(.true.) ! in soas_buff
c$$$               else
c$$$                  call plot(np,x,y,1) ! in soas_plot
c$$$               endif
c$$$            else
c$$$               write(6,308)     ! load a file first
c$$$            endif
c$$$c     savitzky golay, version 1 only
c$$$         elseif ((ileng.le.4).and.((ch(1:4).eq.'osgf')
c$$$     :           .or.(ch(1:4).eq.'osgd').or.(ch(1:4).eq.'osgD'))) then
c$$$            if (np.ne.0) then
c$$$               if (ch(4:4).eq.'f') order=0 ! smooth
c$$$               if (ch(4:4).eq.'d') order=1 ! first deriv
c$$$               if (ch(4:4).eq.'D') order=2 ! second deriv
c$$$               call un_re_do(0) ! save, in soas_buff
c$$$               call savgolilternobaseline(np,x,y,sigmaold,order)
c$$$               call gpes2dat(name,leng) ! data always changed
c$$$               if (showstack) then
c$$$                  call plotbuffers(.true.) ! in soas_buff
c$$$               else
c$$$                  call plot(np,x,y,1) ! in soas_plot
c$$$               endif
c$$$            else
c$$$               write(6,308)     ! load a file first
c$$$            endif
            
c     FFT : f,d,D = smooth or derive one or twice, realft, Nmin
c     using Netlib's libraries, rfft

c     Fourier transform Filter
         elseif ((ileng.eq.1).and.((ch(1:1).eq.'f').or. ! Filter using FFT
     :           (ch(1:1).eq.'d').or.    ! Derive using FFT
     :           (ch(1:1).eq.'D'))) then ! Derive twice using FFT
            if (np.ne.0) then
               if (ch(1:1).eq.'f') order=0 ! smooth
               if (ch(1:1).eq.'d') order=1 ! first deriv
               if (ch(1:1).eq.'D') order=2 ! second deriv
               call un_re_do(0) ! save, in soas_buff
               call filtering_netlib(np,x,y,sigmaold,order,
     :              power,gaussian) ! Netlib 
               call gpes2dat(name,leng) ! data always changed
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif
            
C     BASELINES
            
c     subtract Base line, marker smoothed or exact, cat or non cat
         elseif ((ileng.eq.1).and.((ch(1:1).eq.'b')
     :           .or.(ch(1:1).eq.'B'))) then !
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               if (ch(1:1).eq.'b') then
                  call baseline(np,x,y,areaold,datachanged) ! in soas_base
               elseif (ch(1:1).eq.'B') then
                  call baselineCat(np,x,y,ylimold,datachanged,name,leng) ! 
               endif
               if (datachanged) then
                  call gpes2dat(name,leng)
                  if (showstack) then
                     call plotbuffers(.true.) ! in soas_buff
                  else
                     call plot(np,x,y,1) ! new plot
                  endif
               else             ! data not changed
                  call un_re_do(2) ! ``unsave'', in soas_buff
               endif
            else
               write(6,308)     ! load a file first
            endif
c     Subtract buffers 
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'S')) then
            call un_re_do(0)    ! save, in soas_buff
            call subtract(np,x,y,datachanged) ! in soas_cut
            if (datachanged) then
               call gpes2dat(name,leng)
            else 
               call un_re_do(2) ! unsave, in soas_buff
            endif
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! whichever new or old data            
            endif
c     Divide buffers
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'div')) then
            call un_re_do(0)    ! save, in soas_buff
            call divide(np,x,y,datachanged) ! in soas_cut
            if (datachanged) then
               call gpes2dat(name,leng)
            else 
               call un_re_do(2) ! unsave, in soas_buff
            endif
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! whichever new or old data            
            endif
c     AVErage buffers 
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'ave')) then
            call un_re_do(0)    ! save, in soas_buff
            call average(np,x,y,datachanged) ! in soas_cut
            if (datachanged) then
               call gpes2dat(name,leng)
            else 
               call un_re_do(2) ! unsave, in soas_buff
            endif
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! whichever new or old data            
            endif

c     Collapse a buffer (average points on steps)
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'coll')) then
            call un_re_do(0)    ! save, in soas_buff
            call collapse(np,x,y) ! in soas_buff
            call gpes2dat(name,leng)
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! whichever new or old data            
            endif

c     choose raTio displayed during base line sub
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'t')) then
            call ChooseRatio(r1,r2) ! in soas_base

C     PEAK SEARCH
            
c     search peak(s)
         elseif ((ileng.eq.1).and.((ch(1:1).eq.'1').or. ! 1-peak search, max or min
     :           (ch(1:1).eq.'2').or. ! 2-peak search : 2 extrema same sign (one on each side)
     :           (ch(1:1).eq.'3').or. ! 2-peak search : 2 extrema (one on each side)

     :           (ch(1:1).eq.'4').or. ! 2-peak search : 1 max & 1 min [areas meaningless]
     :           (ch(1:1).eq.'5'))) then !  3-peak search, undocumented, ask Lars
            if (np.ne.0) then
               call searchpeaks(np,x,y,name,leng,ich(ch(1:1),1,1,5))
            else
               write(6,308)     ! load a file first
            endif
c     Peak file handling:

c     Set output file name
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'o')) then
            call SetPeakfile    ! in soas_peak
C     Write a comment to the output file
         elseif ((ileng.eq.7).and.(ch(1:7).eq.'comment')) then
            
            call write_comment_to_ouput_file
C     When doing a fit, soas will automatically write fit data to
C     the output file
         elseif ((ileng.eq.7).and.(ch(1:7).eq.'autoout')) then
            autoout = .true.
 234        format('Automatically writing fit results to output file')
            write(6,234)
C     Cancel autoout
         elseif ((ileng.eq.9).and.(ch(1:9).eq.'noautoout')) then
            autoout = .false.
 235        format('No automatic writing of fit results to output file')
            write(6,235)
C     When doing a fit, soas will ask whether to write fit data to
C     the output file
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'askout')) then
            askout = .true.
 236        format('Will ask for writing fit results to output file')
            write(6,236)
C     Cancel askout
         elseif ((ileng.eq.8).and.(ch(1:8).eq.'noaskout')) then
            askout = .false.
 237        format('Will not ask for writing fit results '//
     :           'to output file')
            write(6,237)
c     See contents of the output file
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'C')) then
            call cat            ! in soas_peak
c     Remove last line of the output file
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'E')) then
            call erase          ! in soas_peak

C     MISCELLANEOUS

c     Get help (use links to browse online documentation)
         elseif (((ileng.eq.4).and.(ch(1:4).eq.'help')).or.
     :           ((ileng.eq.1).and.(ch(1:1).eq.'h'))) then
c 23/01/08
            call soas_system(browser() //
     :           ' http://bip.cnrs-mrs.fr/bip06/soas/') !

C     Toogle debugging information
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'debug')) then 
            if(debugon) then
               write(6, *) "INFO: Turning debugging output off"
               debugon = .false.
            else
               write(6, *) "INFO: Turning debugging output on"
               debugon = .true.
            endif
C     List available commands
         elseif ((ileng.eq.8).and.(ch(1:8).eq.'commands')
     :           .or.(ileng.eq.1).and.(ch(1:1).eq.'m')) then 
            call displaycommands ! in compat/display_commands.c
C     List available fits
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'fits')) then 
            call displayfits    ! in compat/display_commands.c
C     Display soas configuration option
         elseif ((ileng.eq.13).and.(ch(1:13).eq.'configuration')) then 
            call displayconfiguration ! in soas_strings.c
c     Zoom
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'z')) then 
            call zoomScreen     ! in soas_plot
c     chanGe working directory
         elseif (((ileng.eq.1).and.(ch(1:1).eq.'G')).or.
     :           ((ileng.eq.2).and.(ch(1:2).eq.'cd'))) then ! Change working directory
            call ChangeDirectory(directory,l1,l2) ! in soas_misc
            call readtemp(ctemp)
            call loadformatfromfile(listpattern, llistpattern,
     :           colformat,check_pt_off)
c     refresh/flush screeN
         elseif ((ileng.eq.1).and.(ch(1:1).eq.'n')) then ! refresh
            if (np.ne.0) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else                  
                  call plot(np,x,y,1) ! in soas_plot
               endif
            endif
            if (np.eq.0) call gr_exec('clear') ! clear graphic window
c     Say current fileformat
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'ff')) then
            call sayfileformat(listpattern, llistpattern,
     :           colformat,check_pt_off)
c     Read fileformat from file
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'rff')) then
            call loadformatfromfile(listpattern, llistpattern,
     :           colformat,check_pt_off) 
c     flush screeN: redraw all (alternatively, press the return key)
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'flush').or.
     :           ((ileng.eq.1).and.(ch(1:1).eq.'N'))) then
            call gr_out         ! GTVL\flush 
            Write(6,303) 'INFO: You might as well just type ''return''.'
            Write(6,303) 'INFO: Screen flushed.'
c     Print
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'print').or.
     :           ((ileng.eq.1).and.(ch(1:1).eq.'p'))) then
            call confirm('print screen',12,conf,.true.)
            if (conf) call PrintScreen    ! soas_plot
c     Print
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'saveps')) then
            call saveps         ! in soas_plot
c     Quit soas
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'exit').or.
     :           ((ileng.eq.1).and.(ch(1:1).eq.'q'))) then
            call OnSEnVa(ch,ileng) ! in soas_misc
c     ... go on
            if ((ch(1:1).eq.' ').and.(np.ne.0)) then
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) 
               endif
            endif
c     Enters interactive GreG
         elseif (((ileng.eq.1).and.(ch(1:1).eq.'g')).or.
     :           ((ileng.eq.4).and.(ch(1:1).eq.'greg'))) then
            if(.not.expertmodeon) then
               write(6,303) 'INFO : Entering now interactive GreG.' !
               write(6,303) '       Type q to go back to soas.' !
            endif
            call gr4_give('X',np,x) ! transfer data from xy to XY
            call gr4_give('Y',np,y)
c     if (zarr) call gr4_give('Z',np,z)
            call enter_GreG
c     ... and coming back
            call initGreG       ! in soas_plot, reset graphic parameters
 4          call prompt('Update buffer ([y]/z/n) ?') ! return=y
            read(5,2,err=4) ch  !ileng,ch
            ileng=lnblnk(ch)

            if ((ch(1:1).eq.'y').or.(ch(1:1).eq.'z').or.(ileng.eq.0)) then !
               call un_re_do(0) ! save old data
               np=Npmax         ! updating of data
               call gr4_get('X',np,x) ! returns Greg's NXY into np,
                                ! and Greg's X into x
               if (ch(1:1).eq.'z') then
                  call gr4_get('Z',np,y) ! and Z into y (y(es) or default)
               else
                  call gr4_get('Y',np,y) ! and Y into y (y(es) or default)
               endif
               if (np.ne.0) then
                  if (showstack) then
                     call plotbuffers(.true.) ! in soas_buff
                  else
                     call plot(np,x,y,1) ! in soas_plot
                  endif
                  write(6,303) 'INFO : Buffer updated.'
               else
                  write(6,303) 'INFO : GreG''s buffers are empty.'
                  call un_re_do(2) ! unsave 
               endif
            else
               write(6,303) 'INFO : Buffer NOT changed.'
            endif

C     remaining stuff, in alphabetical order.

c     Toggle colors
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'colors')) then 
            if (colorson) then
               write(6,331)
            else
               write(6,330)
            endif
            colorson=.not.colorson
         elseif ((ileng.eq.15).and.(ch(1:15).eq.'alternatecolors')) then 
            alternatecolors=.not.alternatecolors
c     Toggle the use of gv for displaying/printing
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'rungv')) then 
            if (rungv) then
               write(6,*) "Not running gv anymore"
               rungv = .false.
            else
               write(6,*) "Running gv instead of lp for printing"
               rungv = .true.
            endif
c     delete the entire stack
         elseif((ileng.eq.8).and.(ch(1:8).eq.'delstack')) then
            call confirm('delete the current stack',24,conf,.true.)
            if (conf) then
               do j=1,2*Nstack-1 ! stack of data
                  nps(j)=0
                  ilengname(j)=0
                  filename(j)=''
                  marked(j)=.false.
               enddo
               call gr_exec('clear') ! clear graphic window
            endif
c     Clears useless GPES files
         elseif((ileng.eq.5).and.(ch(1:5).eq.'clear')) then
            call confirm('remove files ''*.[iI]??''',29,conf,.true.)
            if (conf) call soas_system('rm *.[iI]??')!
c     Check whether there is a point off scale, yes or no
         elseif((ileng.eq.5).and.(ch(1:5).eq.'check')) then
            check_pt_off=.not.check_pt_off
            if(check_pt_off) then 
               write(6,340) ! 'INFO: now ON'
            else
               write(6,341) ! 'INFO: now OFF'
            endif
c     Open the cursor
         elseif(((ileng.eq.6).and.(ch(1:6).eq.'cursor')).or.
     :           ((ileng.eq.2).and.(ch(1:6).eq.'cu'))) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call cursor(np,x,y,name,leng,xe,ye,datachanged) ! in soas_peaks
               if (datachanged) then
                  call gpes2dat(name,leng)
                  if (showstack) then
                     call plotbuffers(.true.) ! in soas_buff
                  else
                     call plot(np,x,y,1) ! new plot
                  endif
               else 
                  call un_re_do(2) ! unsave, in soas_buff
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     return Delta X for each point
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'dx')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save
               call dx(np,x,y,mdx) ! in soas_base
               call gpes2dat(name,leng)
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     calculate xi^2  = sum_N y(i)^2
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'xi2')) then 
            if (np.ne.0) then
               call calcxi2(np,x,y,xi2) ! in soas_base
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     Shift all X values so that the first element has X = 0
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'shiftx')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save
               xx=data(1,Nstack,1)
               do i=1,nps(Nstack)
                  data(i,Nstack,1)=
     :                    (data(i,Nstack,1))-xx
               enddo
               call gpes2dat(name,leng)
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     Shift all Y values so that the first element has Y = 0
         elseif ((ileng.eq.7).and.(ch(1:7).eq.'shifty1')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save
               yy=data(1,Nstack,2)
               do i=1,nps(Nstack)
                  data(i,Nstack,2)=
     :                    (data(i,Nstack,2))-yy
               enddo
               call gpes2dat(name,leng)
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     scale derivative (times 2.3RT/F) to get napp from d/dE (log transform) 
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'scale')) then
            if (np.ne.0) then
               call un_re_do(0) ! save
               if ((ctemp).ne.0.) then
                  call un_re_do(0) ! save, in soas_buff
                  do i=1,nps(Nstack)
                     data(i,Nstack,2)=
     :                    (data(i,Nstack,2))
     :                    *(8.3144*ctemp*2.3/96500)
                  enddo
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
               else
                  write(6,303) 'ERROR; set non zero temperature first'
               endif
            else
               write(6,308)     ! load a file first
            endif
c     return dy
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'dy')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save
               call dy(np,x,y)  ! in soas_base
               call gpes2dat(name,leng)
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     Select a buffer to be use for baseline subtraction
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'cb')) then 
            if (np.ne.0) then
               call choose_cv_baseline
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     Subtracts previously selected baseline
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'sb')) then 
            if (np.ne.0) then
               call un_re_do(0) ! Save old buffer
               call subtract_baseline(np, x, y)
               call gpes2dat(name,leng)
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     set the editor
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'editor')) then
            write(6,303) 'Enter editor name,'//
     :           ' e.g. ''emacs -name Soas -geometry 60x20 -fn 7x14''' !
            read(5,2) editor    !leditor,editor
            leditor=lnblnk(editor)
            write(6,305) 'editor', editor(1:leditor)
            ch(1:1)=' '
            ileng=1
c     expert ! ExpertModeOn/Off typing expert

C     Toogles expert mode
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'expert')) then 
            ExpertModeOn=.not.(ExpertModeOn)
            ileng=1
            ch(1:1)=' '
         elseif ((ileng.eq.10).and.(ch(1:10).eq.'challenged')) then 
            if (ExpertModeOn) then
               ExpertModeOn=.false.
            else
               write(6,303) 'You''re already using the challenged mode' !
               write(6,303) 'You may  try ''dummy''... '
            endif
c     history list, in soas_buff
         elseif((ileng.eq.4).and.(ch(1:4).eq.'hist')) then 
            call history(logstack,logstackl)
c     Say ilim
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'ilim')) then
            write(6,311) ylimold
c     change ilim
         elseif ((ileng.eq.10).and.(ch(1:10).eq.'changeilim')) then
            write(6,311) ylimold
            call prompt('New value: ')
            read(5,*) ylimold
c     INTerpolation
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'int')) then
            if (np.ne.0) then
               call un_re_do(0) ! save old data
               call interpol(np,x,y) ! in soas_cut
               write(6,303) 'INFO : data interpolated.'
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
               ileng=1
               ch(1:1)=' '
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'intb')) then
            if (np.ne.0) then
               call un_re_do(0) ! save old data
               call interpolb(np,x,y) ! in soas_cut
               write(6,303) 'INFO : data interpolated & blanked.'
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1)
               endif
               ileng=1
               ch(1:1)=' '
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     koutecky-levich transform + extrapolation.
         elseif ((ileng.eq.2).and.(ch(1:2).eq.'kl')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call klt(np,x,y) ! in soas_log
            else
               write(6,308)     ! load a file first
            endif           
c     change x(i) so that equal dx on a log X-axis
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'logsp')) then
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call logspacing(np,x,y,datachanged) ! in soas_cut
               if (datachanged) then
                  call gpes2dat(name,leng)
               else 
                  call un_re_do(2) ! unsave, in soas_buff
               endif
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif
C     let y (ilim-y)/y + log transforms 
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'logt')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               if (ylimold.ne.0) then
                  do i=1,nps(Nstack)
                     data(i,Nstack,2)=
     :                    (ylimold-data(i,Nstack,2))/data(i,Nstack,2)
                  enddo
               endif
               call cuts(np,x,y,datachanged) ! in soas_cut
               call tlog(np,x,y,1) ! in soas_log
c               
               call cuts(np,x,y,datachanged) ! in soas_cut
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif            
c     let y y/ilim
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'iilim')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               if (ylimold.ne.0) then
                  do i=1,nps(Nstack)
                     data(i,Nstack,2)=
     :                    (data(i,Nstack,2))/ylimold
                  enddo
               endif
               call cuts(np,x,y,datachanged) ! in soas_cut
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif            
C     chopping...
C     It looks like we can safely use mdx - it will be used to store
C     the offset in case it is specified on the command-line.

C     Cut the current buffer into two parts based on X values.
         elseif ((ileng.ge.4).and.(ch(1:4).eq.'chop')) then 
            if (ileng.ge.6) then
               read(ch(6:ileng), *) mdx
            else
               mdx = 100
            endif
            call chop_buffer(mdx)
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! in soas_plot
            endif
c     splita/b

C     Get the part of the buffer before the first change of direction
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'splita')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call splita(nps(Nstack), x, y) ! in soas_cut
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif            

C     Get the second half of a buffer
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'splitb')) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call splitb(nps(Nstack), x, y) ! in soas_cut
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif            
c     log Y, log10, loge
         elseif (((ileng.eq.5).and.(ch(1:5).eq.'log10'))
     :           .or.((ileng.eq.4).and.(ch(1:4).eq.'loge'))) then 
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               if (ch(4:4).eq.'1') then
                  call tlog(np,x,y,1) ! in soas_log
               else
                  call tlog(np,x,y,2) ! in soas_log
               endif
               call cuts(np,x,y,datachanged) ! in soas_cut
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif

c      (un/v)mark(N/all)

c     mark a buffer
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'mark')) then
            if ((np.ne.0) .and.(marked(Nstack).eqv..false.)) then
               marked(Nstack)=.true.
               write(6,325)     ! marked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif
         elseif ((ileng.eq.5).and.(ch(1:4).eq.'mark')) then !markN
            ib=ich(ch(5:5),1,1,9) ! (,def,min,max)
            if ((ib.ge.1).and.(ib.le.19).and.
     :           (nps(ib).ne.0) .and.(marked(ib).eqv..false.)) then
               marked(ib)=.true.
               write(6,328) ib  ! marked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif
         elseif ((ileng.eq.6).and.(ch(1:4).eq.'mark')) then !markNN
            ib=ich(ch(5:5),1,0,9) ! (,def,min,max)
            ib=ib*10+ich(ch(6:6),1,0,9) ! (,def,min,max)
            if ((ib.ge.1).and.(ib.le.19).and.
     :           (nps(ib).ne.0) .and.(marked(ib).eqv..false.)) then
               marked(ib)=.true.
               write(6,328) ib  ! marked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else 
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif

c     Unmark a buffer
         elseif ((ileng.eq.6).and.(ch(1:6).eq.'unmark')) then
            if (marked(Nstack).eqv..true.) then
               marked(Nstack)=.false.
               write(6,326)     ! unmarked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif 
         elseif ((ileng.eq.7).and.(ch(1:6).eq.'unmark')) then !unmarkN
            ib=ich(ch(7:7),1,1,9) ! (,def,min,max)
            if ((ib.ge.1).and.(ib.le.19).and.
     :           (nps(ib).ne.0) .and.(marked(ib).eqv..true.)) then
               marked(ib)=.false.
               write(6,327) ib  ! unmarked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif
         elseif ((ileng.eq.8).and.(ch(1:6).eq.'unmark')) then !markNN
            ib=ich(ch(7:7),1,0,9) ! (,def,min,max)
            ib=ib*10+ich(ch(8:8),1,0,9) ! (,def,min,max)
            if ((ib.ge.1).and.(ib.le.19).and.
     :           (nps(ib).ne.0) .and.(marked(ib).eqv..true.)) then
               marked(ib)=.false.
               write(6,327) ib  ! marked
               if (lastplotstack) then
                  call plotbuffers(.true.) 
               else
                  call plot (np,x,y,1)
               endif
            else
               write(6,329)
            endif

c     Unmark all buffers
         elseif ((ileng.eq.9).and.(ch(1:9).eq.'unmarkall')) then
            do j=1,2*Nstack-1
               if (marked(j).eqv..true.) then
                  marked(j)=.false.
                  write(6,327) j
               endif
            enddo
            if (lastplotstack) then
               call plotbuffers(.true.) 
            else
               call plot (np,x,y,1)
            endif

c     Mark all buffers
         elseif ((ileng.eq.7).and.(ch(1:7).eq.'markall')) then
            do j=1,2*Nstack-1
               if ((nps(j).ne.0) .and.(marked(j).eqv..false.)) then
                  marked(j)=.true.
                  write(6,328) j
               endif
            enddo
            if (lastplotstack) then
               call plotbuffers(.true.) 
            else
               call plot (np,x,y,1)
            endif

c     Overlay marked buffers + current
         elseif (((ileng.eq.5).and.(ch(1:5).eq.'vmark'))) then
c     plot all marked plus current
            call plot(np,x,y,10)

c     let y y/ymax
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'norm')) then
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call search(Np,X,Y,1,Np,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
               do i=1,nps(Nstack)
                  data(i,Nstack,2)=
     :                 (data(i,Nstack,2))/yy
               enddo
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
               ileng=1
               ch(1:1)=' '
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
            

c     'reg'

C     Linear regressions (baselines, normalizations...)
         elseif ((ileng.eq.3).and.(ch(1:4).eq.'reg')) then
            if (np.ne.0) then
               call un_re_do(0) ! save, in soas_buff
               call plot(np,x,y,1) ! in soas_plot
               call reg(np,x,y,datachanged) ! in soas_log
               if (.not.datachanged) call un_re_do(2) ! unsave, in soas_buff
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
               ileng=1
               ch(1:1)=' '
            else
               write(6,308)     ! load a file first
               ileng=2
            endif
c     draw POInts
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'poi')) then 
            if (np.ne.0) then
               if (showstack) call plot(np,x,y,1)
               call gr_exec1('poi')
            else
               write(6,308)     ! load a file first
            endif
c     'pwd'

C     Display current working directory
         elseif ((ileng.eq.3).and.(ch(1:3).eq.'pwd')) then 
c     modif
            call GetCwdShort(directory,l1,l2) ! in soas_misc
            call gr_exec('dr text 0 1.5 "'//
     :           directory(1:l1)//'" 3 /box 7') 
c            call gr_exec('dr text 0 1.5 "'//getcwd()//'" 3 /box 7') 
c     'save' 

C     Save the current buffer to a file
         elseif ((ch(1:4).eq.'save').and.(ileng.eq.4)) then 
            if (np.ne.0) then 
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
            endif
 202        call prompt('Save one point in ...')
            read(5,*,err=202) idelta
            call save(np,x,y,name,leng,idelta,colformat) ! in soas_file
c     'showstack'

C     Shows the stack
         elseif ((ileng.eq.9).and.(ch(1:9).eq.'showstack')) then
            showstack=.not.showstack
            if (showstack) then
               call plotbuffers(.true.) ! in soas_buff
            else
               call plot(np,x,y,1) ! in soas_plot
            endif
c     'sort'

C     Sort data
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'sort')) then
            if (np.ne.0) then
               call un_re_do(0) ! save old data
               call tri(np,x,y) ! in soas_cut
               write(6,303) 'INFO : data sorted.'
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif

C     Sort data in reverese order
         elseif ((ileng.eq.5).and.(ch(1:5).eq.'rsort')) then
            if (np.ne.0) then
               call un_re_do(0) ! save old data
               call ssort(x,y,np,-2)
               write(6,303) 'INFO : data sorted.'
               if (showstack) then
                  call plotbuffers(.true.) ! in soas_buff
               else
                  call plot(np,x,y,1) ! in soas_plot
               endif
            else
               write(6,308)     ! load a file first
               ileng=2
            endif

c     'sys' ! execute shell command 
C     Runs a shell command.
         elseif ((ch(1:4).eq.'sys ').and.(ileng.gt.4)) then 
            call confirm(ch(5:ileng),ileng-5+1,conf,.true.)
            if (conf) call soas_system(ch(5:ileng)) ! soas_system(ShellCommand) 
c     'caption' ! execute shell command 

C     Set the caption for the plot
         elseif ((ch(1:8).eq.'caption ').and.(ileng.gt.8)) then 
c            call confirm(ch(5:ileng),ileng-5+1,conf,.true.)
c            if (conf) call soas_system(ch(5:ileng)) ! soas_system(ShellCommand) 
            call gr_exec('dr text 0 -1.5 "'//
     :           ch(9:ileng)//'" 5 /box 2')
            call confirm("clear caption",13,conf,.false.)
            if (conf) call gr_exec('cl seg')

c     Set the page size to US letter
         elseif (((ch(1:6).eq.'letter').or.(ch(1:6).eq.'Letter'))
     :           .and.(ileng.eq.6)) then
            call confirm('change the page size (this will clear the plot)'
     :           ,47,conf,.false.)
            if (conf) then  
               call initpageLetter
               call initgreg
               write(6,303) 'INFO : paper format = Letter'
            endif

C     Set the page to A4 
         elseif ((ch(1:2).eq.'A4').and.(ileng.eq.2)) then
            call confirm('change the page size (this will clear the plot)'
     :           ,47,conf,.false.)
            if (conf) then 
               call initpageA4
               call initgreg
               write(6,303) 'INFO : paper format = A4'
            endif
c     'temp'
C     Sets the current temperature.
         elseif ((ileng.eq.4).and.(ch(1:4).eq.'temp')) then 
            write(6,312) ctemp
            read(5,*) ctemp
            write(6,312) ctemp
c     dummy
c     elseif ((ileng.eq.5).and.(ch(1:5).eq.'dummy')) then 
c     continue



c$$$
c$$$c     inhibition CO/ccNiR !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c$$$         elseif((ileng.eq.2).and.(ch(1:2).eq.'CO')) then 
c$$$            if (np.ne.0) then
c$$$c     getting y(0)
c$$$               write(6,303) 'GETTING Y0' 
c$$$               call cursor(np,x,y,name,leng,xt,ye,datachanged) ! in soas_peaks
c$$$               write(6,*) ye
c$$$c     getting x(0)
c$$$               write(6,303) 'GETTING X0' 
c$$$               call cursor(np,x,y,name,leng,xe,yt,datachanged) ! in soas_peaks
c$$$               write(6,*) xe
c$$$c     stack before baseline
c$$$               call un_re_do(0) ! save, in soas_buff
c$$$c     baseline
c$$$               write(6,303) 'GETTING spline baseline, quit using V' 
c$$$               call baseline(np,x,y,areaold,datachanged) ! in soas_base
c$$$               call plot(np,x,y,1)
c$$$c               pause
c$$$c     changing name
c$$$c     aaa
c$$$c     aaa.sc.dat
c$$$               filename(Nstack)(ilengname(Nstack)+1:ilengname(Nstack)+7)
c$$$     :              ='.sc.dat'
c$$$               ilengname(Nstack)=ilengname(Nstack)+7
c$$$c     saving
c$$$               call un_re_do(0) ! save, in soas_buff
c$$$               call save(np,x,y,name,leng,1,colformat) ! in soas_file
c$$$c     stack before fit
c$$$               call un_re_do(0) ! save, in soas_buff
c$$$c     fit2expd52
c$$$               write(6,303) 'FITTING' 
c$$$               ch(1:6)='expd52'
c$$$               call fit2_y(np,x,y,name,leng,ch(1:6),6,datachanged) !
c$$$               if (.not.datachanged) call un_re_do(2)
c$$$               call plot(np,x,y,1)
c$$$
c$$$            else
c$$$               write(6,308)     ! load a file first
c$$$               ileng=2
c$$$            endif



c     that's it
         elseif (.not.(((ileng.eq.1).and.(ch(1:1).eq.'m')).or.
     :           ((ileng.eq.0)))) then
            write(6,309) ch(1:ileng) ! unknown command
            write(6,380) ch(1:ileng), ileng
            ch(1:50)='                      '//
     :           '                            ' !
         endif                  ! not sure about that. 
c     flush graphic window 
         call gr_out            ! GTVL\flush  
         if (s_stack) call save_stack 
      enddo                     ! choice.ne.q 
c     exiting 
      end 
c     end OF MAIN
