c     http://www.netlib.org/sfmm/seval.f, modified to evaluate derivative too
c     http://www.netlib.org/sfmm/spline.f

C     Vincent Fourmond: my understanding is that this code is in
C     the public domain.

      subroutine spline_netlib(n, x, y, b, c, d)
      integer n
      real x(n), y(n), b(n), c(n), d(n)
c     the coefficients b(i), c(i), and d(i), i=1,2,...,n are computed
c     for a cubic interpolating spline
c     s(x) = y(i) + b(i)*(x-x(i)) + c(i)*(x-x(i))**2 + d(i)*(x-x(i))**3
c     for  x(i) .le. x .le. x(i+1)
c     input..
c     n = the number of data points or knots (n.ge.2)
c     x = the abscissas of the knots in strictly increasing order
c     y = the ordinates of the knots
c     output..
c     b, c, d  = arrays of spline coefficients as defined above.
c     using  p  to denote differentiation,
c     y(i) = s(x(i))
c     b(i) = sp(x(i))
c     c(i) = spp(x(i))/2
c     d(i) = sppp(x(i))/6  (derivative from the right)
c     the accompanying function subprogram  seval  can be used
c     to evaluate the spline.
      integer nm1, ib, i
      real t
      nm1 = n-1
      if ( n .lt. 2 ) return
      if ( n .lt. 3 ) go to 50
c     set up tridiagonal system
c     b = diagonal, d = offdiagonal, c = right hand side.
      d(1) = x(2) - x(1)
      c(2) = (y(2) - y(1))/d(1)
      do 10 i = 2, nm1
         d(i) = x(i+1) - x(i)
         b(i) = 2.*(d(i-1) + d(i))
         c(i+1) = (y(i+1) - y(i))/d(i)
         c(i) = c(i+1) - c(i)
 10   continue
c     end conditions.  third derivatives at  x(1)  and  x(n)
c     obtained from divided differences
      b(1) = -d(1)
      b(n) = -d(n-1)
      c(1) = 0.
      c(n) = 0.
      if ( n .eq. 3 ) go to 15
      c(1) = c(3)/(x(4)-x(2)) - c(2)/(x(3)-x(1))
      c(n) = c(n-1)/(x(n)-x(n-2)) - c(n-2)/(x(n-1)-x(n-3))
      c(1) = c(1)*d(1)**2/(x(4)-x(1))
      c(n) = -c(n)*d(n-1)**2/(x(n)-x(n-3))
c     forward elimination
 15   do 20 i = 2, n
         t = d(i-1)/b(i-1)
         b(i) = b(i) - t*d(i-1)
         c(i) = c(i) - t*c(i-1)
 20   continue
c     back substitution
      c(n) = c(n)/b(n)
      do 30 ib = 1, nm1
         i = n-ib
         c(i) = (c(i) - d(i)*c(i+1))/b(i)
 30   continue
c     c(i) is now the sigma(i) of the text
c     compute polynomial coefficients
      b(n) = (y(n) - y(nm1))/d(nm1) + d(nm1)*(c(nm1) + 2.*c(n))
      do 40 i = 1, nm1
         b(i) = (y(i+1) - y(i))/d(i) - d(i)*(c(i+1) + 2.*c(i))
         d(i) = (c(i+1) - c(i))/d(i)
         c(i) = 3.*c(i)
 40   continue
      c(n) = 3.*c(n)
      d(n) = d(n-1)
      return
 50   b(1) = (y(2)-y(1))/(x(2)-x(1))
      c(1) = 0.
      d(1) = 0.
      b(2) = b(1)
      c(2) = 0.
      d(2) = 0.
      return
      end

      real function seval_netlib(n, u, x, y, b, c, d, order) 
c     order=0 or 1 for function or its derivative.
      integer n, order
      real  u, x(n), y(n), b(n), c(n), d(n)
c     this subroutine evaluates the cubic spline function
c     seval = y(i) + b(i)*(u-x(i)) + c(i)*(u-x(i))**2 + d(i)*(u-x(i))**3
c     where  x(i) .lt. u .lt. x(i+1), using horner's rule
c     if  u .lt. x(1) then  i = 1  is used.
c     if  u .ge. x(n) then  i = n  is used.
c     input..
c     n = the number of data points
c     u = the abscissa at which the spline is to be evaluated
c     x,y = the arrays of data abscissas and ordinates
c     b,c,d = arrays of spline coefficients computed by spline
c     if  u  is not in the same interval as the previous call, then a
c     binary search is performed to determine the proper interval.
      integer i, j, k
      real dx
      data i/1/
      if ( i .ge. n ) i = 1
      if ( u .lt. x(i) ) go to 10
      if ( u .le. x(i+1) ) go to 30
c  binary search
 10   i = 1
      j = n+1
 20   k = (i+j)/2
      if ( u .lt. x(k) ) j = k
      if ( u .ge. x(k) ) i = k
      if ( j .gt. i+1 ) go to 20
c  evaluate spline
 30   dx = u - x(i)
      if (order.eq.0) then
         seval_netlib = y(i) + dx*(b(i) + dx*(c(i) + dx*d(i)))
      elseif(order.eq.1) then ! d/dx
         seval_netlib = b(i) + 2*dx*c(i) + 3*dx**2*d(i)
      else
         write(6,*) 'bad order in spline_netlib'
         stop
      endif
      return
      end


c     http://www.netlib.org/slatec/src/ssort.f
c     instead of NR's subroutine sort2(N,X,Y)
*DECK SSORT
      SUBROUTINE SSORT (X, Y, N, KFLAG) 
C***  BEGIN PROLOGUE  SSORT
C***  PURPOSE  Sort an array and optionally make the same interchanges in
C     an auxiliary array.  The array may be sorted in increasing
C     or decreasing order.  A slightly modified QUICKSORT
C     algorithm is used.
C***  LIBRARY   SLATEC
C***  CATEGORY  N6A2B
C***  TYPE      SINGLE PRECISION (SSORT-S, DSORT-D, ISORT-I)
C***  KEYWORDS  SINGLETON QUICKSORT, SORT, SORTING
C***  AUTHOR  Jones, R. E., (SNLA)
C     Wisniewski, J. A., (SNLA)
C***  DESCRIPTION
C     
C     SSORT sorts array X and optionally makes the same interchanges in
C     array Y.  The array X may be sorted in increasing order or
C     decreasing order.  A slightly modified quicksort algorithm is used.
C     
C     Description of Parameters
C      X - array of values to be sorted   (usually abscissas)
C      Y - array to be (optionally) carried along
C      N - number of values in array X to be sorted
C      KFLAG - control parameter
C            =  2  means sort X in increasing order and carry Y along.
C            =  1  means sort X in increasing order (ignoring Y)
C            = -1  means sort X in decreasing order (ignoring Y)
C            = -2  means sort X in decreasing order and carry Y along.
C
C***REFERENCES  R. C. Singleton, Algorithm 347, An efficient algorithm
C                 for sorting with minimal storage, Communications of
C                 the ACM, 12, 3 (1969), pp. 185-187.
C***ROUTINES CALLED  XERMSG
C***REVISION HISTORY  (YYMMDD)
C   761101  DATE WRITTEN
C   761118  Modified to use the Singleton quicksort algorithm.  (JAW)
C   890531  Changed all specific intrinsics to generic.  (WRB)
C   890831  Modified array declarations.  (WRB)
C   891009  Removed unreferenced statement labels.  (WRB)
C   891024  Changed category.  (WRB)
C   891024  REVISION DATE from Version 3.2
C   891214  Prologue converted to Version 4.0 format.  (BAB)
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
C   901012  Declared all variables; changed X,Y to SX,SY. (M. McClain)
C   920501  Reformatted the REFERENCES section.  (DWL, WRB)
C   920519  Clarified error messages.  (DWL)
C   920801  Declarations section rebuilt and code restructured to use
C           IF-THEN-ELSE-ENDIF.  (RWC, WRB)
C***END PROLOGUE  SSORT
C     .. Scalar Arguments ..
      INTEGER KFLAG, N
C     .. Array Arguments ..
      REAL X(*), Y(*)
C     .. Local Scalars ..
      REAL R, T, TT, TTY, TY
      INTEGER I, IJ, J, K, KK, L, M, NN
C     .. Local Arrays ..
      INTEGER IL(21), IU(21)
C     .. External Subroutines ..
c      EXTERNAL XERMSG
C     .. Intrinsic Functions ..
      INTRINSIC ABS, INT
C***FIRST EXECUTABLE STATEMENT  SSORT
      NN = N
      IF (NN .LT. 1) THEN
c         CALL XERMSG ('SLATEC', 'SSORT',
c     +      'The number of values to be sorted is not positive.', 1, 1)
         write(6,*) 'SLATEC SSORT The number of values to be sorted is not positive.'!
         RETURN
      ENDIF
      KK = ABS(KFLAG)
      IF (KK.NE.1 .AND. KK.NE.2) THEN
c     CALL XERMSG ('SLATEC', 'SSORT',
c     +      'The sort control parameter, K, is not 2, 1, -1, or -2.', 2,
c     +      1)
         write(6,*) 'SLATEC SSORT wrong control parameter' !
         RETURN
      ENDIF
C     Alter array X to get decreasing order if needed
      IF (KFLAG .LE. -1) THEN
         DO 10 I=1,NN
            X(I) = -X(I)
 10      CONTINUE
      ENDIF
      IF (KK .EQ. 2) GO TO 100
C     Sort X only
      M = 1
      I = 1
      J = NN
      R = 0.375E0
 20   IF (I .EQ. J) GO TO 60
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
 30   K = I
C     Select a central element of the array and save it in location T
      IJ = I + INT((J-I)*R)
      T = X(IJ)
C     If first element of array is greater than T, interchange with T
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
      ENDIF
      L = J
C     If last element of array is less than than T, interchange with T
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
C        If first element of array is greater than T, interchange with T
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
         ENDIF
      ENDIF
C     Find an element in the second half of the array which is smaller
C     than T
 40   L = L-1
      IF (X(L) .GT. T) GO TO 40
C     Find an element in the first half of the array which is greater
C     than T
 50   K = K+1
      IF (X(K) .LT. T) GO TO 50
C     Interchange these elements
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         GO TO 40
      ENDIF
C     Save upper and lower subscripts of the array yet to be sorted
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 70
C     Begin again on another portion of the unsorted array
 60   M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
 70   IF (J-I .GE. 1) GO TO 30
      IF (I .EQ. 1) GO TO 20
      I = I-1
 80   I = I+1
      IF (I .EQ. J) GO TO 60
      T = X(I+1)
      IF (X(I) .LE. T) GO TO 80
      K = I
 90   X(K+1) = X(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 90
      X(K+1) = T
      GO TO 80
C     Sort X and carry Y along
 100  M = 1
      I = 1
      J = NN
      R = 0.375E0
 110  IF (I .EQ. J) GO TO 150
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
 120  K = I
C     Select a central element of the array and save it in location T
      IJ = I + INT((J-I)*R)
      T = X(IJ)
      TY = Y(IJ)
C     If first element of array is greater than T, interchange with T
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
         Y(IJ) = Y(I)
         Y(I) = TY
         TY = Y(IJ)
      ENDIF
      L = J
C     If last element of array is less than T, interchange with T
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
         Y(IJ) = Y(J)
         Y(J) = TY
         TY = Y(IJ)
C        If first element of array is greater than T, interchange with T
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
            Y(IJ) = Y(I)
            Y(I) = TY
            TY = Y(IJ)
         ENDIF
      ENDIF
C     Find an element in the second half of the array which is smaller
C     than T
 130  L = L-1
      IF (X(L) .GT. T) GO TO 130
C     Find an element in the first half of the array which is greater
C     than T
 140  K = K+1
      IF (X(K) .LT. T) GO TO 140
C     Interchange these elements
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         TTY = Y(L)
         Y(L) = Y(K)
         Y(K) = TTY
         GO TO 130
      ENDIF
C     Save upper and lower subscripts of the array yet to be sorted
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 160
C     Begin again on another portion of the unsorted array
 150  M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
 160  IF (J-I .GE. 1) GO TO 120
      IF (I .EQ. 1) GO TO 110
      I = I-1
 170  I = I+1
      IF (I .EQ. J) GO TO 150
      T = X(I+1)
      TY = Y(I+1)
      IF (X(I) .LE. T) GO TO 170
      K = I
 180  X(K+1) = X(K)
      Y(K+1) = Y(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 180
      X(K+1) = T
      Y(K+1) = TY
      GO TO 170
C     Clean up
 190  IF (KFLAG .LE. -1) THEN
         DO 200 I=1,NN
            X(I) = -X(I)
 200     CONTINUE
      ENDIF
      RETURN
      END

C     NR below

      SUBROUTINE locate(xx,n,x,j)
c     given xx(1:n), monotonic, and x, returns j / x between xx(j) xx(j+1)
      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).gt.xx(1)).eqv.(x.gt.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1)) then
         j=1
      elseif (x.eq.xx(n)) then
         j=n
      else
         j=jl
      endif
      if (j.eq.0) j=1 !!!!
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software #200.
      
C     MISCELLANEOUS

      SUBROUTINE moment(data,n,ave,adev,sdev,var,skew,curt)
      INTEGER n
      REAL adev,ave,curt,sdev,skew,var,data(n)
      INTEGER j
      REAL p,s,ep
      if(n.le.1) then
         write(6,*) 'ERROR n must be at least 2 in moment'!
         return
      endif
      s=0.
      do 11 j=1,n
        s=s+data(j)
11    continue
      ave=s/n
      adev=0.
      var=0.
      skew=0.
      curt=0.
      ep=0.
      do 12 j=1,n
        s=data(j)-ave
        ep=ep+s
        adev=adev+abs(s)
        p=s*s
        var=var+p
        p=p*s
        skew=skew+p
        p=p*s
        curt=curt+p
12    continue
      adev=adev/n
      var=(var-ep**2/n)/(n-1)
      sdev=sqrt(var)
      if(var.ne.0.)then
        skew=skew/(n*sdev**3)
        curt=curt/(n*var**2)-3.
      else
CM    pause 'no skew or kurtosis when zero variance in moment'
         skew=0.
         curt=0.
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software #200.


      subroutine aver(data,n,ave,var)
      integer n
      real ave,var,data(*)
      integer j
      real s,ep
      ave=0.
      do j=1,n
         ave=ave+data(j)
      enddo
      ave=ave/n
      var=0.
      ep=0.
      do j=1,n
         s=data(j)-ave
         ep=ep+s
         var=var+s*s
      enddo
      var=(var-ep**2/n)/(n-1)
      return
      end

      subroutine ab(y,np,a,b)
      real y(2**14)
      integer np
      real sxy,sx,sy,sxx,a,b
      sx=0.
      sy=0.
      sxx=0.
      sxy=0.
      do i=1,np
         sx=sx+real(i)
         sy=sy+y(i)
         sxx=sxx+real(i)*real(i) 
         sxy=sxy+real(i)*y(i) 
      enddo
      if ((sxx*real(np)-sx*sx).ne.0) then
         b=(sxy*real(np)-sx*sy)/(sxx*real(np)-sx*sx)
         a=(sxx*sy-sx*sxy)/(sxx*real(np)-sx*sx)      
      else
         a=0.
         b=0.
      endif
      return
      end

c     not from book.
      real function avex(x,y,n1,n2)
      real x(*),y(*)
      sx=0.
      n=n2-n1+1
      do i=n1,n2
         sx=sx+x(i)
      enddo
      avex=sx/real(n)
      return
      end

      real Function reglin2(x,y,n1,n2)
      real x(*),y(*)
      integer n,n1,n2,i
      real*8 sxy,sx,sy,sxx
      sx=0.
      sy=0.
      sxx=0.
      sxy=0.
      n=n2-n1+1
      do i=n1,n2
         sx=sx+x(i)
         sy=sy+y(i)
         sxx=sxx+x(i)*x(i) 
         sxy=sxy+x(i)*y(i) 
      enddo
      if ((sxx*real(n)-sx*sx).ne.0) then
         reglin2=(sxy*real(n)-sx*sy)/(sxx*real(n)-sx*sx)
      else
         reglin2=1.
      endif
      return
      end
      
      real Function ord2(x,y,n1,n2)
      real x(*),y(*)
      integer n,i,n1,n2
      real*8 sxy,sx,sy,sxx
      sx=0.
      sy=0.
      sxx=0.
      sxy=0.
      n=n2-n1+1
      do i=n1,n2
         sx=sx+x(i)
         sy=sy+y(i)
         sxx=sxx+x(i)*x(i)
         sxy=sxy+x(i)*y(i)
      enddo
      ord2=(sxx*sy-sx*sxy)/(sxx*real(n)-sx*sx)
      return
      end

