/* 
   soas_strings.c: handling various strings
   Copyright (C) 2007, 2008 Vincent Fourmond

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   
   This file handles various strings which are much more easy
   to manipulate/query using C structures, such as home directories,
   hardcoded data path (cf generated datadir.h) and others.
*/

#include <cfortran/cfortran.h>
#include <stdlib.h>
#include <stdio.h>

#include <datadir.h>

#include <svn_version.h>

/* Hard-coded buffer size. */
#ifndef BUFFER_SIZE
#define BUFFER_SIZE 100
#endif

#define REVISION "$Revision: 136 $"

const char * data_dir();
FCALLSCFUN0(STRING, data_dir,DATADIR,datadir);

/* Returns the data directory. */
const char * data_dir()
{
  const char * d;
  if(d = getenv("SOAS")) 
    return d;
  return DATADIR;
}



const char * data_file();
FCALLSCFUN1(STRING,data_file,DATAFILE,datafile,STRING);

static char buffer[BUFFER_SIZE];
const char * data_file(const char *file)
{
  snprintf(buffer, BUFFER_SIZE, "%s/%s", data_dir(), file);
  return buffer;
}


/* Same series of functions for the $HOME directory */

const char * home_dir();
FCALLSCFUN0(STRING, home_dir,HOMEDIR,homedir);

const char * home_dir()
{
  return getenv("HOME");
}


const char * home_file();
FCALLSCFUN1(STRING,home_file,HOMEFILE,homefile,STRING);

const char * home_file(const char *file)
{
  snprintf(buffer, BUFFER_SIZE, "%s/%s", home_dir(), file);
  return buffer;
}


const char * expand_filename();
FCALLSCFUN1(STRING,expand_filename,EXPANDFILE,expandfile,STRING);

/* A function to expand ~ from home directories */
const char * expand_filename(const char * file)
{
  if(file[0] == '~')
    {
      snprintf(buffer, BUFFER_SIZE, "%s/%s", home_dir(), file + 1);
      return buffer;
    }
  else 
    return file;
}


const char * soas_banner = "SOAS (version " PACKAGE_VERSION
  " -- SVN revision " SVN_REVISION  
  "). Sorry for the name, that's Kerensa's idea." ;

const char * banner();
FCALLSCFUN0(STRING, banner, BANNER, banner);

const char * banner()
{
  return soas_banner;
}

const char * browser()
{
  return SOAS_BROWSER;
}

FCALLSCFUN0(STRING, browser, BROWSER, browser);

/* This functions prints out the configuration with which
   SOAS was compiled .
*/
void display_configuration() 
{
  printf("Soas (version " PACKAGE_VERSION 
	 ") was compiled with the following options:\n");
#ifdef HAVE_LIBREADLINE
  printf("  * using readline for commands edition/completion\n");
#else 
  printf("  * no commands edition (nor completion)\n");
#endif
#ifdef HAVE_READLINE_HISTORY_H
  printf("  * commands history\n");
#endif
  printf("  * using '" SOAS_BROWSER "' as online help browser\n");
#ifdef USING_SCALC
  printf("  * using scalc for arbitrary fits\n");
#else
  printf("  * no arbitrary fits\n");
#endif
}

FCALLSCSUB0(display_configuration,DISPLAYCONFIGURATION,displayconfiguration);
