C    soas_sodrfits: drivers and functions for sodr fits.
C    Copyright (C) 1999-2008  Christophe Leger
C                  2007, 2008 Vincent Fourmond
C
C    This program is free software; you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation; either version 2 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License along
C    with this program; if not, write to the Free Software Foundation, Inc.,
C    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


C     drivers and functions for sodr fits.
C     documentation: http://www.netlib.org/odrpack/guide.ps 
      
C     if you want to add a function called `NAME' (case sensitive) 
C     follow the examples commented with c$$$ 
      subroutine fit_y(N,X,Y,name,leng,fitfcn,lfitfcn,datachanged) ! 

      parameter (MAXN=2**14)
      integer npin,lfitfcn
      character*128 name,oname   ! o for output (fitres)
      integer leng,oleng        ! o for output
      character*128 fitfcn,fcnname
      character*200 expression,promptfor  ! for arbitrary fits.
      logical datachanged,conf,forceraw,expert,success

C     routines for interactions with scalc
      logical scalcparse        ! parses an expression
      integer scalcparamnumber  ! the number of parameters in the expression
      character*128 scalcparametername ! the parameters names

C  ODRPACK ARGUMENT DEFINITIONS
C      ==> FCN      NAME OF THE USER SUPPLIED FUNCTION SUBROUTINE
C      ==> N        NUMBER OF OBSERVATIONS 
C      ==> M        COLUMNS OF DATA IN THE EXPLANATORY VARIABLE
C      ==> NP       NUMBER OF PARAMETERS
C      ==> NQ       NUMBER OF RESPONSES PER OBSERVATION
C     <==> BETA     FUNCTION PARAMETERS
C      ==> Y        RESPONSE VARIABLE
C      ==> LDY      LEADING DIMENSION OF ARRAY Y
C      ==> X        EXPLANATORY VARIABLE
C      ==> LDX      LEADING DIMENSION OF ARRAY X
C      ==> WE       "EPSILON" WEIGHTS
C      ==> LDWE     LEADING DIMENSION OF ARRAY WE
C      ==> LD2WE    SECOND DIMENSION OF ARRAY WE
C      ==> WD       "DELTA" WEIGHTS
C      ==> LDWD     LEADING DIMENSION OF ARRAY WD
C      ==> LD2WD    SECOND DIMENSION OF ARRAY WD
C      ==> IFIXB    INDICATORS FOR "FIXING" PARAMETERS (BETA)
C      ==> IFIXX    INDICATORS FOR "FIXING" EXPLANATORY VARIABLE (X)
C      ==> LDIFX    LEADING DIMENSION OF ARRAY IFIXX
C      ==> JOB      TASK TO BE PERFORMED 
C      ==> NDIGIT   GOOD DIGITS IN SUBROUTINE FUNCTION RESULTS
C      ==> TAUFAC   TRUST REGION INITIALIZATION FACTOR
C      ==> SSTOL    SUM OF SQUARES CONVERGENCE CRITERION
C      ==> PARTOL   PARAMETER CONVERGENCE CRITERION
C      ==> MAXIT    MAXIMUM NUMBER OF ITERATIONS
C      ==> IPRINT   PRINT CONTROL 
C      ==> LUNERR   LOGICAL UNIT FOR ERROR REPORTS 
C      ==> LUNRPT   LOGICAL UNIT FOR COMPUTATION REPORTS 
C      ==> STPB     STEP SIZES FOR FINITE DIFFERENCE DERIVATIVES WRT BETA
C      ==> STPD     STEP SIZES FOR FINITE DIFFERENCE DERIVATIVES WRT DELTA
C      ==> LDSTPD   LEADING DIMENSION OF ARRAY STPD
C      ==> SCLB     SCALE VALUES FOR PARAMETERS BETA
C      ==> SCLD     SCALE VALUES FOR ERRORS DELTA IN EXPLANATORY VARIABLE 
C      ==> LDSCLD   LEADING DIMENSION OF ARRAY SCLD
C     <==> WORK     REAL             WORK VECTOR
C      ==> LWORK    DIMENSION OF VECTOR WORK
C     <==  IWORK    INTEGER WORK VECTOR
C      ==> LIWORK   DIMENSION OF VECTOR IWORK
C     <==  INFO     STOPPING CONDITION 
 
C  PARAMETERS SPECIFYING MAXIMUM PROBLEM SIZES HANDLED BY THIS DRIVER
C     MAXN          MAXIMUM NUMBER OF OBSERVATIONS 
C     MAXM          MAXIMUM NUMBER OF COLUMNS IN EXPLANATORY VARIABLE
C     MAXNP         MAXIMUM NUMBER OF FUNCTION PARAMETERS
C     MAXNQ         MAXIMUM NUMBER OF RESPONSES PER OBSERVATION 

C  PARAMETER DECLARATIONS AND SPECIFICATIONS
      INTEGER    LDIFX,LDSCLD,LDSTPD,LDWD,LDWE,LDX,LDY,LD2WD,LD2WE,
     +           LIWORK,LWORK,MAXM,MAXNP,MAXNQ   !,MAXN
c$$$  If you want to adjust more than 28 parameters (!), change MAXNP here
      PARAMETER (MAXM=1,MAXNP=28,MAXNQ=1, !,MAXN already set to 2**14
     +           LDY=MAXN,LDX=MAXN,
     +           LDWE=1,LD2WE=1,LDWD=1,LD2WD=1,
     +           LDIFX=MAXN,LDSTPD=1,LDSCLD=1,
     +           LWORK=18 + 11*MAXNP + MAXNP**2 + MAXM + MAXM**2 + 
     +                 4*MAXN*MAXNQ + 6*MAXN*MAXM + 2*MAXN*MAXNQ*MAXNP +  
     +                 2*MAXN*MAXNQ*MAXM + MAXNQ**2 + 
     +                 5*MAXNQ + MAXNQ*(MAXNP+MAXM) + LDWE*LD2WE*MAXNQ,
     +           LIWORK=20+MAXNP+MAXNQ*(MAXNP+MAXM))

C  VARIABLE DECLARATIONS 
      INTEGER          I,INFO,IPRINT,J,JOB,L,LUNERR,LUNRPT,M,MAXIT,N,
     +                 NDIGIT,NP,NQ
      INTEGER          IFIXB(MAXNP),IFIXX(LDIFX,MAXM),IWORK(LIWORK)
      REAL             PARTOL,SSTOL,TAUFAC
      REAL             BETA(MAXNP),SCLB(MAXNP),SCLD(LDSCLD,MAXM),
     +                 STPB(MAXNP),STPD(LDSTPD,MAXM),
     +                 WD(LDWD,LD2WD,MAXM),WE(LDWE,LD2WE,MAXNQ),
     +                 WORK(LWORK),X(LDX,MAXM),Y(LDY,MAXNQ)
C  [christophe:] FOR CALLS TO WINF, SEE PAGE 83 ORDPACK_GUIDE.PS
      LOGICAL          ISODR 
      INTEGER          DELTAI,EPSI,XPLUSI,FNI,SDI,VCVI, 
     +                 RVARI,WSSI,WSSDEI,WSSEPI,RCONDI,ETAI,
     +                 OLMAVI,TAUI,ALPHAI,ACTRSI,PNORMI,RNORSI,PRERSI,
     +                 PARTLI,SSTOLI,TAUFCI,EPSMAI,
     +                 BETA0I,BETACI,BETASI,BETANI,SI,SSI,SSFI,QRAUXI,UI,!
     +                 FSI,FJACBI,WE1I,DIFFI,
     +                 DELTSI,DELTNI,TI,TTI,OMEGAI,FJACDI,
     +                 WRK1I,WRK2I,WRK3I,WRK4I,WRK5I,WRK6I,WRK7I,
     +                 LWKMN
c     to put the covariance matrix, but weird...
      REAL             VCV(MAXNP,MAXNP)
      real             variance(maxnp)
      REAL             VCV2(MAXNP,MAXNP)
c     to overlay initial condition and best fit
      REAL             Y2(LDY,MAXNQ)
c     to allow direct calls to fcn
      REAL             FJACB,FJACD ! (dummy calls to fcn)
      INTEGER          ISTOP
c     allowed functions:
      external         fcn_straight,fcn_mm,fcn_mml,
     :     fcn_expd,            !b1*exp(-X/b2)+b3
     :     fcn_film_expd,       ! same as expd, but with film loss
     :     fcn_expd1,           !1+b1*(exp(-X/b2)-1)
     :     fcn_expd2,           !b1*exp(-X/b2)+b3*exp(-X/b4)+b5
     :     fcn_expd3,           !(b1)*exp(-X/b3)+b2*exp(-X/b4)+1-b1-b2
     :     fcn_expd4,           !(b1)*exp(-X/b2)+b3*exp(-X/b4)-b1-b3
     :     fcn_nernst,          !b1+b2/(1.+exp(b3*f*(x-b4)))
     :     fcn_sigm0,           !b1/(1+b2*exp(x/b3))
     :     fcn_sigm,            !b1/(1+b2*exp(x/b3)), or b1/(1+b2) for t<0
     :     fcn_sigmrh5,          !
     :     fcn_sigmco,           
     :     fcn_sigm2,           !1/(1+b1*exp(-x/b2))+b3/(1+b1**(-1)*exp(x/b2))
     :     fcn_lo20,fcn_lo21,fcn_lo22,fcn_lr20,fcn_lr21,fcn_lr22,
     :     fcn_sr21l,fcn_sr21n,fcn_sr22n,fcn_so21l,fcn_so21n,
     :     fcn_sv21n, ! vague rversible distribue no ilim
     :     fcn_sv21l, ! vague rversible distribue no ilim
     :     fcn_sr11l,fcn_sr11n,fcn_so11l,fcn_so11n,fcn_so11nf,
     :     fcn_nca,fcn_ncl,fcn_nci,fcn_sw,
     :     fcn_napab,
     :     fcn_wr21,fcn_wo21,fcn_wo22,
     :     fcn_wra,fcn_woa,fcn_wc,
     :     fcn_so22n,fcn_so22nc,!,fcn_so22nc2 pas la peine
     :     fcn_soa1nc,fcn_soa2nc,fcn_soa3nc,fcn_soa4nc,
     :     fcn_scalc_arb,       ! arbitrary functions...
     :     fcn_ncc2 ! non cat coope 2
c      soaXnc: termes jusqu' X/2

      character*1 ch
      integer nc                ! number of redox centers, non cat data
      integer sns               ! SUM OF NS when deconv. non cat data
      integer ISHAPE            ! used for fitting complex waves
      real temp,fara            ! temperature (from common), faraday constant
      character*128 parametername(MAXNP) ! names of the parameters
      real xx,yy                ! dummy
      real x_start, x_end       ! to save the X starting end ending points
      real tmp1, tmp2           ! temporary variables
      integer tmpi              ! temporary integer
      common /temperature/ temp
      common /ishape_wc/ ISHAPE
      common /job/ JOB,IPRINT   ! for changing parameters
      common /expert/ expert
C     If the common variable autoout is true, then a summary of the
C     parameters from each fit is written to the output file.
      logical autoout, askout
      common /autooutput/ autoout, askout
      fara=96500./8.3144/temp

C  SPECIFY DEFAULT VALUES FOR SODRC ARGUMENTS
      WE(1,1,1)  = -1.0E0
      WD(1,1,1)  = -1.0E0
c     IFIXB(1)   = -1
      IFIXX(1,1) = -1
      JOB        = -1
      NDIGIT     = -1
      TAUFAC     = -1.0E0
      SSTOL      = -1.0E0
      PARTOL     = -1.0E0
      MAXIT      = -1
c     IPRINT     = -1
      LUNERR     = -1
      LUNRPT     = -1
      STPB(1)    = -1.0E0
      STPD(1,1)  = -1.0E0
      SCLB(1)    = -1.0E0
      SCLD(1,1)  = -1.0E0
c
      M          = 1
      NQ         = 1
      IPRINT     = 0            ! new default = no info, can be modif by user
      x_start = x(1,1)
      x_end   = x(n,1)

C     INITIAL PARAMETERS, dependent on fcn
      
      do i=1,MAXNP
         ifixb(i)=1             ! default for now = all parameters are free
      enddo
      if (fitfcn(1:lfitfcn).eq.'?') then
         write(6,303) 'INFO : available functions are' !
         write(6,303) '1) Catalytic data' !
         write(6,303) '  lo20,lo21,lo22,lr20,lr21,lr22' !
         write(6,303) '       wo21,et wo22, wr21' !
         write(6,303) '  sr21l,sr21n,sr22n,so21l,so21n,so22n,so22nc(2)' !
         write(6,303) '  sr11l,sr11n,so11l,so11n,so11nf' !
         write(6,303) '  woa, wra, wc' !
         write(6,303) ' Meaning of the names' !
         write(6,303) '  [1st letter] l=logt, w=wave, s=slopey wave' !
         write(6,303) '  [2] o=oxidative, r=reductive, c=complex'//
     :        ' (switchy/boosty)' !
         write(6,303) '  [3] 1 electron or 2 electrons, or a'//
     :        ' for fitting napp' !
         write(6,303) '  [4] nbr of k0s, 0=reversible, 1 or 2=quasi' !
         write(6,303) '  [5] if [1=s], l (limiting current reached)'//
     :        ' or n (NOT)'     ! 
         write(6,303) ' Equivalence with former names:' ! 
         write(6,303) '  for(lo20), foq1(lo21), foq2(lo22)' !
         write(6,303) '  frr(lr20), frq1(lr21), frq2(lr22)' !
         write(6,303) '  qo2nb(so21l),qo2nbnl[l](so21n),qr2nb(sr21l),'// 
     :        'qr2nbnl(sr21n)'  !
         write(6,303) '2) Non catalytic data' !  
         write(6,303) ' 2-1) CV. i redox centers with IDENTICAL'//
     :        ' coverage'       ! 
         write(6,303) '  ncai, i=1-9 (2e peaks are fit adjusting'// 
     :        ' nApp)'          ! 
         write(6,303) '  ncli, i=1-9 (2e peaks are fit to Laviron'''//
     :        ' eq, adjusting DeltaE)' ! 
         write(6,303) ' 2-2) CV. i redox centers with INDEPENDENT'//
     :        ' coverages'      !
         write(6,303) '  ncIi, i=1-9 (2e peaks are fit adjusting nApp)' ! 
         write(6,303) ' 2-3) SWV. i redox centers with IDENTICAL'//
     :        ' coverages'      !
         write(6,303) '  swi, i=1-9 (one-electron centers only)' !
         write(6,303) ' Equivalence with former names:' !
         write(6,303) '  F(nca1), F2(ncai), F3(ncIi)' !
         write(6,303) '3) Miscellaneous' !
         write(6,303) '  straight, mm (michaelis-menten)' !
         write(6,303) '  nernst (sigmoid)' !
         write(6,303) '  expd (expon decay),expd2 (expon decay 2 times)' !
         write(6,303) 'To fit with function x, type fit_x.' !
         return


C**** Miscellaneous

C     Linear regression
      elseif (fitfcn(1:lfitfcn).eq.'straight') then
         NP=2
         parametername(1)='a'
         parametername(2)='b'
         beta(1)=reglin2(x,y,1,n)
         beta(2)=ord2(x,y,1,n)
         JOB=30                 ! 3-: analytical derivatives not checked, 
                                ! -0: orthogonal distance regression 
C     For some reasons, this does not seem to work. Falling back to
C     standard stuff. (with derivatives, it seems)
         JOB=32
c$$$      elseif (fitfcn(1:lfitfcn).eq.'NAME') then  
c$$$         NP=? ! nbr of adjustable parameters 
c$$$         parametername(i)='name parameter i', i=1 to NP
c$$$         beta(i)= ! guessed value of para, , i=1 to NP
c$$$         ifixb(k)=0 ! the parameter k will be fixed if =0
c$$$                    ! by default, they are all free
c$$$         JOB=2 ! normal least squares

C     Homogeneous kinetics with a redox equilibrium
      elseif (fitfcn(1:lfitfcn).eq.'dt1e') then
         NP=5
         parametername(1)='Vm' 
         parametername(2)='Km'
         parametername(3)='Red' ! the initial concentration of reductor
         parametername(4)='K'   ! the equilibrium constant
         parametername(5)='Med' ! the total concentration of mediator
         beta(1) = 1
         beta(2) = 0.1
         beta(3) = 1.5
         beta(4) = 10           ! if 1, fit won't work properly, most likely
         beta(5) = 68           ! corresponding to 5mM MV
         ifixb(5)=0
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'nernst') then
         NP=4
         parametername(1)='a1' ! a1+a2/(1+e(f a3 (e-a4)))
         parametername(2)='a2'
         parametername(3)='n'
         parametername(4)='E0'
         beta(1)=y(n,1)
         beta(2)=y(1,1)-beta(1)
         beta(3)=1.
         ifixb(3)=0
         beta(4)=(x(1,1)+x(n,1))/2.
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'sigm') then
         NP=4
         parametername(1)='ilim' ! a1/(1+a2*exp(t/a3))
         parametername(2)='km/c'
         parametername(3)='tau'
         parametername(4)='t0'
         beta(1)=y(1,1)
         beta(2)=.1
         beta(3)=30.
         beta(4)=30.
         ifixb(4)=0.
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'ktot') then
         NP=3
         parametername(1)='e0' 
         parametername(2)='e1'
         parametername(3)='k1'
         if(y(1,1).le.y(n,1)) then
            beta(1)=x(1,1)
            beta(2)=x(n,1)
         else
            beta(1)=x(n,1)
            beta(2)=x(1,1)
         endif
         beta(3)=sqrt(y(1,1) * y(n,1))
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'ampl') then
         NP=3
         parametername(1)='e0' 
         parametername(2)='a'   ! the active portion
         parametername(3)='delta'
         beta(1) = (x(1,1) + x(n,1))/2
         beta(2) = 0.1
         if(y(n/2,1).le.0) then
            beta(3) = -0.06     ! 60 mV steps by default, going down...
         else
            beta(3) = +0.06     ! ... or up
         endif
         ifixb(3) = 0           ! does not vary !!
         JOB=2                  ! normal least squares

C     Like expd but with film loss detection
      elseif (fitfcn(1:lfitfcn).eq.'film_expD') then
         NP=4
         parametername(1)='a'   ! a*exp(-x/tau)+c'
         parametername(2)='tau'
         parametername(3)='c'
         parametername(4)='kfilm' ! this parameter quantifies film loss
         if (x(1,1).ne.0) then
            xx=X(1,1)           ! translate X axis
            do i=1,N
               X(I,1)=X(I,1)-xx
            enddo
            write(6,303) 'INFO : X axis shifted up'
         endif

C     To get a decent idea of the amount of film loss,
C     We approximate the last bit by a line and take it as
C     a tangent to the exponential (so k = a/b)
         tmpi = n * 0.8
         tmp1 = reglin2(x,y,tmpi, n)
         tmp2 = ord2(x,y,tmpi,n)

 2345    format("a = ", G12.3, " b = ", G12.3)
         write(6, 2345) tmp1, tmp2

         beta(4) = - (tmp1 / tmp2)
         beta(1)=Y(1,1) - Y(N,1) * exp(beta(4) * x(n,1))
         beta(2)=X(N,1)/3       ! better, isn't it ?
         beta(3)=Y(N,1) * exp(beta(4) * x(n,1))

C     This fit is unfortunately not fantastic. Exponential fits seem to
C     have problems in general.

         JOB=2                  ! normal least squares
         
C     Fit for NapAB, using model from Bertrand et al, J. Phys. Chem. B,
C     111, 10300, figure 7 (with an extra parameter and no explicit
C     dependence on substrate concentration)
      elseif (fitfcn(1:lfitfcn).eq.'napab_no_s') then
         NP=6
         parametername(1)='Eir' 
         parametername(2)='E''oi'   
         parametername(3)='E''ir'   
         parametername(4)='ki/k2' 
         parametername(5)='kr/k2'   
         parametername(6)='gamma*k2'   
         
         beta(1) = -.5
         beta(2) = -.2
         beta(3) = -.2
         beta(4) = 2
         beta(5) = 1
         beta(6) = min(y(1,1),y(N,1))
         do i = 1, N
            if (y(i,1) .le. beta(6)) then
               beta(6) = y(i,1)
            endif
         enddo

         JOB=2                  ! normal least squares

C     Fit for a double-layer 'capacitive current'
      elseif (fitfcn(1:lfitfcn).eq.'double_layer') then
         NP=6
         parametername(1)='i0'  ! average current = C * V ?
         parametername(2)='a'   ! linear slope = R ??
         parametername(3)='ampl'! amplitude of the initial exponential phase
         parametername(4)='v0'  ! length of the initial exponential phase
         parametername(5)='a2'  ! amplitude of the final exponential phase
         parametername(6)='v1'  ! length of the final exponential phase

         beta(1) = ord2(x,y,1,n)
         beta(2) = reglin2(x,y,1,n)
         beta(3) = y(1,1) - beta(1)
         beta(4) = abs(x(1,1) - x(N,1))/20
         beta(5) = (y(N,1) - beta(1))/1000
         beta(6) = abs(x(1,1) - x(N,1))/3
         JOB=2                  ! normal least squares

C     Fit for a double-layer 'capacitive current'
      elseif (fitfcn(1:lfitfcn).eq.'double_layer_power') then
         ! from soas_double_layer.f
         call setup_double_layer_power(np, parametername)
         call estimate_double_layer_power_parameters(beta, x, y, n)
         JOB=2                  ! normal least squares
C     Fit for the kinetic constant of activation/inactivation
C     as a function of potential for the model of NapAB derived
C     with an inactive branch at the Mo IV + NO side.
      elseif (fitfcn(1:lfitfcn).eq.'napab_activation') then
         NP=8
         parametername(1)='Eir' 
         parametername(2)='E''oi'   
         parametername(3)='E''ir'   
         parametername(4)='ki/k2' 
         parametername(5)='kr/k2'   
         parametername(6)='kd'  ! Deactivation
         parametername(7)='ka'  ! Activation
         parametername(8)='Ein' ! Potential of the (V/IV) couple in the
                                ! inactive branch
         
         beta(1) = -.4
         beta(2) = 1
         beta(3) = -.2
         beta(4) = 2
         beta(5) = 1
         beta(6) = 0.1
         beta(7) = 0.1
         beta(8) = -.4

C     By default, the first 5 parameters are entered manually
C     and therefore shouldn't vary.
         do i = 1,5
            ifixb(i) = 0
         enddo

         JOB=2                  ! normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'sigmrh5') then
         NP=4
         parametername(1)='ilim' ! a1/(1+a2*exp(t/a3))
         parametername(2)='km'
         parametername(3)='tau'
         parametername(4)='t0'
         beta(1)=y(1,1)
         beta(2)=.8
         beta(3)=30.
         beta(4)=30.
         ifixb(4)=0.
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'sigmco') then
         NP=8
         parametername(1)='imax' ! a1/(1+a2*exp(t/a3))
         parametername(2)='km/s'
         parametername(3)='i0/kic'
         parametername(4)='i0/kiu'
         parametername(5)='t0'
         parametername(6)='t1'
         parametername(7)='taus'
         parametername(8)='taui'
         ifixb(4)=0
         ifixb(5)=0
         ifixb(6)=0
         beta(1)=y(1,1)
         beta(2)=.01
         beta(3)=30.
         beta(4)=0.
         beta(7)=20.
         beta(8)=30

         beta(5)=630 !(x(1,1)+x(n,1))/3.
         beta(6)=663 !(x(1,1)+x(n,1))/3.
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'sigm0') then
         NP=3
         parametername(1)='a1' ! a1/(1+a2*exp(t/a3))
         parametername(2)='a2'
         parametername(3)='tau'
         beta(1)=y(1,1)
         beta(2)=.1
         beta(3)=30.
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'sigm2') then
         NP=3
         parametername(1)='i/ki' ! 1/(1+a1*exp(-t/a2))+a3/(1+a1**(-1)*exp(t/a2))
         parametername(2)='tau'
         parametername(3)='res'
         beta(1)=.5
         beta(2)=20.
         beta(3)=.1
         JOB=2                  !normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'mm') then ! michaelis menten
         NP=2
         parametername(1)='kcat'
         parametername(2)='Km'
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(1)=yy             ! either min or max (in case ilim<0)
         beta(2)=(x(1,1)+x(N,1))/2.  
         JOB=2                  ! normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'mml') then ! michaelis menten
         NP=2
         parametername(1)='kcat'
         parametername(2)='Km'
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(1)=yy             ! either min or max (in case ilim<0)
         beta(2)=(10**x(1,1)+10**x(N,1))/2.  
         JOB=2                  ! normal least squares
C     Exponential decay
      elseif (fitfcn(1:lfitfcn).eq.'expd') then ! exponential decay
         NP=3
         parametername(1)='a'   ! a*exp(-x/tau)+c'
         parametername(2)='tau'
         parametername(3)='c'
         beta(1)=Y(1,1)-Y(N,1)            
         beta(2)=X(N,1)
         beta(3)=Y(N,1)
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
C     Exponential decay (shifting X values)
      elseif (fitfcn(1:lfitfcn).eq.'expD') then ! exponential decay, with
                                ! slight modifications: we are shifting
         NP=3
         parametername(1)='a'   ! a*exp(-x/tau)+c'
         parametername(2)='tau'
         parametername(3)='c'
         if (x(1,1).ne.0) then
            xx=X(1,1)           ! translate X axis
            do i=1,N
               X(I,1)=X(I,1)-xx
            enddo
            write(6,303) 'INFO : X axis shifted up'
         endif
         beta(1)=Y(1,1)-Y(N,1)            
         beta(2)=X(N,1)/3       ! better, isn't it ?
         beta(3)=Y(N,1)
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
      elseif (fitfcn(1:lfitfcn).eq.'expd1') then ! exponential decay
         NP=2
         parametername(1)='c'   ! (1-c)*exp(-x/tau)+c'
         parametername(2)='tau'
         beta(1)=Y(1,1)-Y(N,1)            
         beta(2)=X(N,1)
         beta(3)=Y(N,1)
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
      elseif (fitfcn(1:lfitfcn).eq.'arb') then   ! arbitrary function
C     We first print the available constants:
         call scalcinit()
 2098    format("The following constants are available")
         write(6,2098)
         call scalcprintvars()
         expression = promptfor('f(x) = ','e')
         if (scalcparse(expression)) then
            NP = scalcparamnumber()
C205        format("There are ",I3," parameters")
C           write(6,205) NP
            do i=1,NP
               parametername(i) = scalcparametername(i)
C206           format("Parameter ",I3," is ",A)
C              write(6,206) i, parametername(i)
               beta(i) = 1      ! Nothing better, really
            enddo
            JOB=2               ! normal least squares
            call plot(N,X,Y,1)  ! replot
         else                   ! In case something went wrong.
            return 
         endif
C     Two exponentials
      elseif (fitfcn(1:lfitfcn).eq.'expd2') then ! exponential decay
         NP=5
         parametername(1)='a1'   ! a1*exp(-x/tau1)+a2*exp(-x/tau2)+c'
         parametername(2)='tau1'
         parametername(3)='a2'   ! a1*exp(-x/tau1)+a2*exp(-x/tau2)+c'
         parametername(4)='tau2'
         parametername(5)='c'
         xx=X(1,1)              ! translate X axis
         do i=1,N
            X(I,1)=X(I,1)-xx
         enddo
         beta(1)=Y(1,1)             
         beta(2)=X(N,1)
         beta(3)=Y(1,1)/2.
         beta(4)=X(N,1)/2.
         beta(5)=Y(N,1)
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
C     A variant of expd2 that can be especially helpful for fitting
C     activation/decay curves.
      elseif (fitfcn(1:lfitfcn).eq.'expD2') then ! exponential decay
         NP=5
         parametername(1)='a1'   ! a1*exp(-x/tau1)+a2*exp(-x/tau2)+c'
         parametername(2)='tau1'
         parametername(3)='a2'   ! a1*exp(-x/tau1)+a2*exp(-x/tau2)+c'
         parametername(4)='tau2'
         parametername(5)='c'
         xx=X(1,1)              ! translate X axis
         do i=1,N
            X(I,1)=X(I,1)-xx
         enddo
         beta(1)=Y(1,1)             
         beta(2)=X(N,1)/3
         beta(3)=-Y(1,1)/2.
         beta(4)=X(N,1)/2.
         beta(5)=Y(N,1)
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
      elseif (fitfcn(1:lfitfcn).eq.'expd3') then ! exponential decay
         NP=4
         parametername(1)='a1'   ! (a1)*exp(-x/t1)+a2*exp(-x/t2)+(1-a1-a2)'
         parametername(2)='a2'
         parametername(3)='t1'   
         parametername(4)='t2'
c         xx=X(1,1)              ! translate X axis
c         do i=1,N
c            X(I,1)=X(I,1)-xx
c         enddo
         beta(1)=.5         
         beta(2)=.5
         beta(3)=5
         beta(4)=20
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
      elseif (fitfcn(1:lfitfcn).eq.'expd4') then ! exponential decay
         NP=4
         parametername(1)='a1'   ! a1*exp(-x/a2)+a3*exp(-x/a4)-a1-a3
         parametername(2)='tau1'
         parametername(3)='a2'   
         parametername(4)='tau2'
         beta(1)= -Y(N,1)/2
         beta(3)= -Y(N,1)/2
         beta(2)=50
         beta(4)=1000
         JOB=2                  ! normal least squares
         call plot(N,X,Y,1)     ! replot
C**** CATALYTIC
      elseif (fitfcn(1:lfitfcn).eq.'napab') then ! checked
         NP=6
         parametername(1)='E1'
         parametername(2)='E2'
         parametername(3)='E3'
         parametername(4)='km/s'
         parametername(5)='k5/k4'
         parametername(6)='naAGk2'
         beta(1)=-.5
         beta(2)=-.2
         beta(3)=-.2
         beta(4)=1
         beta(5)=2.
         beta(6)=min(y(1,1),y(N,1))
         JOB=2                  ! normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'lo20') then ! checked
         NP=2
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         JOB=2                  ! normal least squares
      elseif (fitfcn(1:lfitfcn).eq.'lo22') then ! checked
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0(O/I)'
         parametername(4)='k2/k0(I/R)'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         beta(4)=.01
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'lo21') then ! checked
         NP=3
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'wo21') then ! checked
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0'
         parametername(4)='ilim' 
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(4)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'wo22') then ! checked
         NP=5
         parametername(1)='EOI'
         parametername(2)='EIR'
         parametername(3)='l(kk0)IR'
         parametername(4)='l(kk0)OI'
         parametername(5)='ilim' 
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2
         beta(4)=-2
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(5)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'wr21') then ! checked
         NP=4!  corriger
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0'
         parametername(4)='ilim' 
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(4)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'lr20') then ! checked
         NP=2
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'lr22') then ! checked
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0(O/I)'
         parametername(4)='k2/k0(I/R)'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         beta(4)=.01
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'lr21') then ! 6
         NP=3
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='k2/k0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=.01
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sr21l') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='log(betad0)'
         parametername(5)='ilim'
c$$$  (x(1,1)+x(N,1))/2. is the mean potential value of the data set
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            ! .1
         beta(4)=0.5            ! 5.
c$$$  the following call returns in yy the extremum (max or min) Y value
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(5)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sr11l') then 
         NP=5
         parametername(1)='unused'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='log(betad0)'
         parametername(5)='ilim'
         beta(1)=0.
         ifixb(1)=0
         beta(2)=(x(1,1)+x(N,1))/2.  
         beta(3)=-1             !.1
         beta(4)=1.             !5.
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(5)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sr21n') then 
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='ilim/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !
c$$$  used to guess ilim/betad0 from the slope of the voltammogram
         beta(4)=-2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sr22n') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='log(k2/k0M)'
         parametername(5)='ilim/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !
         beta(4)=-2.            !
c$$$  used to guess ilim/betad0 from the slope of the voltammogram
         beta(5)=-2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sr11n') then 
         NP=4
         parametername(1)='unused'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='ilim/betad0'
         beta(1)=0.
         ifixb(1)=0
         beta(2)=(x(1,1)+x(N,1))/2. 
         beta(3)=-2.            !.1
         beta(4)=-2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so21l') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='log(betad0)'
         parametername(5)='ilim'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !.1
         beta(4)=1.             !5.
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(5)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so11l') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='unused'
         parametername(3)='log(k2/k0M)'
         parametername(4)='log(betad0)'
         parametername(5)='ilim'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=0.
         ifixb(2)=0
         beta(3)=-2.            !.1
         beta(4)=1.             !5.
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(5)=yy
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so21n') then 
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(k2/k0M)'
         parametername(4)='ilim/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !.1
         beta(4)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so22n') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(kk0)3/2'
         parametername(4)='log(kk0)1/2'
         parametername(5)='ilim/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !.1
         beta(4)=-2.            !.1
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sv21n') then 
         NP=5
         parametername(1)='E(1)'
         parametername(2)='E(2)'
         parametername(3)='log(koxk0M)'
         parametername(4)='kred/kox'
         parametername(5)='ilimox/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.
         beta(4)=.5 
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'sv21l') then 
         NP=6
         parametername(1)='E(1)'
         parametername(2)='E(2)'
         parametername(3)='log(koxk0M)'
         parametername(4)='kred/kox'
         parametername(5)='ilimox'
         parametername(6)='betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.
         beta(4)=.5 
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(6)=max(x(1,1),x(N,1))/beta(5)
         beta(5)=max(x(1,1),x(N,1))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so22nc') then 
         NP=6
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(kk0)3/2'
         parametername(4)='log(kk0)1/2'
         parametername(5)='ilim/betad0'
         parametername(6)='const'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !.1
         beta(4)=-2.            !.1
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(6)=(y(1,1)+y(N,1))/20.  
         JOB=10                 ! au lieu de 2, pour comparaison avec soa4nc
c     *************************************************
      elseif (fitfcn(1:lfitfcn).eq.'soa1nc') then 
         NP=3
         parametername(1)='E(O/I)'
         parametername(2)='ilim/betad0'
         parametername(3)='const'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(3)=0.  
         ifixb(3)=0
         JOB=10
      elseif (fitfcn(1:lfitfcn).eq.'soa2nc') then 
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='log(kk0)1/2'
         parametername(3)='ilim/betad0'
         parametername(4)='const'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=-5.            !.1
         beta(3)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(4)=(y(1,1)+y(N,1))/20.  
         JOB=10
      elseif (fitfcn(1:lfitfcn).eq.'soa3nc') then 
         NP=5
         parametername(1)='E(O/I)'
         parametername(2)='log(kk0)1/2'
         parametername(3)='log(kk0)3/2'
         parametername(4)='ilim/betad0'
         parametername(5)='const'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=-2
         beta(3)=-2.            !.1
         beta(4)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(5)=(y(1,1)+y(N,1))/20.  
         JOB=10
      elseif (fitfcn(1:lfitfcn).eq.'soa4nc') then 
         NP=6
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(kk0)1/2'
         parametername(4)='log(kk0)3/2'
         parametername(5)='ilim/betad0'
         parametername(6)='const'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=beta(1)
         beta(3)=-2.            !.1
         beta(4)=-2.            !.1
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(6)=(y(1,1)+y(N,1))/20.  
         JOB=10
      elseif (fitfcn(1:lfitfcn).eq.'so22nc2') then 
         NP=6
         parametername(1)='E(O/I)'
         parametername(2)='E(I/R)'
         parametername(3)='log(kk0)3/2'
         parametername(4)='log(kk0)1/2'
         parametername(5)='ilim/betad0'
         parametername(6)='const'
         beta(2)=(x(1,1)+x(N,1))/3.  
         beta(1)=(x(1,1)+x(N,1))/2.  ! cond init diffrentes, non crois
         beta(3)=-2.            !.1
         beta(4)=-2.            !.1
         beta(5)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         beta(6)=(y(1,1)+y(N,1))/20.  
         JOB=12
      elseif (fitfcn(1:lfitfcn).eq.'so11n') then 
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='unused'
         parametername(3)='log(k2/k0M)'
         parametername(4)='ilim/betad0'
         beta(1)=(x(1,1)+x(N,1))/2.  
         beta(2)=0.
         ifixb(2)=0
         beta(3)=-2.            !.1
         beta(4)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'so11nf') then 
         NP=4
         parametername(1)='E(O/I)'
         parametername(2)='unused'
         parametername(3)='log(k2/k0M)'
         parametername(4)='ilim/betad0'
         beta(1)=-.250!(x(1,1)+x(N,1))/2.  
         beta(2)=0.
         ifixb(2)=0
         beta(3)=-2.            !.1
         beta(4)=2./fara*(max(reglin2(x,y,1,int(N/10)),
     :        reglin2(x,y,int(9*N/10),int(N))))
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'woa') then 
         NP=3
         parametername(1)='ilim'
         parametername(2)='napp'
         parametername(3)='Eapp'
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(1)=yy
         beta(2)=1.5
         beta(3)=(x(1,1)+x(N,1))/2. 
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'wra') then 
         NP=3
         parametername(1)='ilim'
         parametername(2)='napp'
         parametername(3)='Eapp'
         call search(N,X,Y,1,N,0,xx,yy,xx,xx,xx,xx,xx,xx,.false.,.false.) !
         beta(1)=yy
         beta(2)=1.5
         beta(3)=(x(1,1)+x(N,1))/2. 
         JOB=2
      elseif (fitfcn(1:lfitfcn).eq.'wc') then 
         NP=10
         call get_wave_parameters(NP,beta,IFIXB,ISHAPE,parametername)
         JOB=2

c     non cat coope
      elseif (fitfcn(1:lfitfcn).eq.'ncc2') then
         parametername(1)='F^2NuAGamma/RT'
         call  search(N,X,Y,1,N,0,xx,yy1,
     :        xx,xx,xx,yy,xx,xx,.false.,.false.)
         beta(1)=yy/0.025*yy1/(abs(yy1))/2
         NP=1+2
         parametername(2)='E1'
         parametername(3)='E2'
         beta(2)=(x(1,1)+x(N,1))/2.+.05
         beta(3)=(x(1,1)+x(N,1))/2.-.05
         JOB=2
C**** NON CATALYTIC
      elseif (fitfcn(1:lfitfcn-1).eq.'nca') then
         parametername(1)='F^2NuAGamma/RT'
         call  search(N,X,Y,1,N,0,xx,yy1,
     :        xx,xx,xx,yy,xx,xx,.false.,.false.)
c     area in yy=beta(1)*sum_nc(ns)*0.025
         beta(1)=yy/0.025*yy1/(abs(yy1)) ! to be divided by sum_nc(ns)=sns
c     ichlog returns the char*1 fitfcn(lfitfcn:lfitfcn) into integer nc
c     if < 9, then success.
         call ichlog(fitfcn(lfitfcn:lfitfcn),9,nc,success)
         if (.not.success) then
            write(6,304) fitfcn(1:lfitfcn)
            return
         endif            
         NP=1+3*nc
         sns=0
         do i=1,nc
 200        format(I1)
            write(ch,200) i
            parametername(1+3*i-3+1)='n_s('//ch(1:1)//')'
            parametername(1+3*i-3+2)='n_app('//ch(1:1)//')'
            parametername(1+3*i-3+3)='E^0('//ch(1:1)//')'
            call get_init_center_i(i,
     :           beta(1+3*i-3+1),beta(1+3*i-3+2),beta(1+3*i-3+3),1)
            sns=sns+int(beta(1+3*i-3+1))
            if (beta(1+3*i-3+1).eq.0.) then
               beta(1+3*i-3+1)=1. ! free
            else
               ifixb(1+3*i-3+1)=0
            endif
            if (beta(1+3*i-3+2).eq.0.) then
               beta(1+3*i-3+2)=1.5 ! free
            else
               ifixb(1+3*i-3+2)=0
            endif
         enddo
         beta(1)=beta(1)/real(sns)
         JOB= 2
c     ncl
      elseif (fitfcn(1:lfitfcn-1).eq.'ncl') then
         parametername(1)='F^2NuAGamma/RT'
         call  search(N,X,Y,1,N,0,xx,yy1,
     :        xx,xx,xx,yy,xx,xx,.false.,.false.)
         beta(1)=yy/0.025*yy1/(abs(yy1)) ! to be divided by sum_nc(ns)=sns
         call ichlog(fitfcn(lfitfcn:lfitfcn),9,nc,success)
         if (.not.success) then
            write(6,304) fitfcn(1:lfitfcn)
            return
         endif            
         NP=1+3*nc
         sns=0
         do i=1,nc
            write(ch,200) i
            call get_init_center_i(i, ! i nsi nappi ei ichoice
     :           beta(1+3*i-3+1),beta(1+3*i-3+2),beta(1+3*i-3+3),2)
            sns=sns+int(beta(1+3*i-3+1))
            parametername(1+3*i-3+1)='n_s('//ch(1:1)//')'
            ifixb(1+3*i-3+1)=0  ! fixed
            if (beta(1+3*i-3+1).eq.1.) then
               parametername(1+3*i-3+2)='unused' !
               ifixb(1+3*i-3+2)=0. ! fixed
            else
               parametername(1+3*i-3+2)='Delta_E('//ch(1:1)//')' !
            endif
            parametername(1+3*i-3+3)='E^0('//ch(1:1)//')'
         enddo
         beta(1)=beta(1)/real(sns)
         JOB=2                  ! numerical derivatives
      elseif (fitfcn(1:lfitfcn-1).eq.'ncI') then
         call ichlog(fitfcn(lfitfcn:lfitfcn),9,nc,success)
         if (.not.success) then
            write(6,304) fitfcn(1:lfitfcn)
            return
         endif            
         NP=3*nc
         call  search(N,X,Y,1,N,0,xx,yy1,
     :        xx,xx,xx,yy,xx,xx,.false.,.false.)
         do i=1,nc
            write(ch,200) i
            parametername(3*i-3+1)='F^2NuAG('//ch(1:1)//')ns('//
     :           ch(1:1)//')/RT' !
            beta(3*i-3+1)=yy/0.025*yy1/(abs(yy1))/real(nc) ! 
            call get_init_center_i(i, ! i dummy nappi ei ichoice
     :           xx,beta(3*i-3+2),beta(3*i-3+3),3)
            if (beta(3*i-3+2).eq.0) beta(3*i-3+2)=1.5
            parametername(3*i-3+2)='napp('//ch(1:1)//')' !
            parametername(3*i-3+3)='E^0('//ch(1:1)//')'
         enddo
         JOB=2                  ! numerical derivatives
c     SWV
      elseif (fitfcn(1:lfitfcn-1).eq.'sw') then
         call ichlog(fitfcn(lfitfcn:lfitfcn),9,nc,success)
         if (.not.success) then
            write(6,304) fitfcn(1:lfitfcn)
            return
         endif            
         NP=4+2*nc ! n/f/b, cov, nu, Estep, (k0,E0) nc times
         call get_init_sw(NP,beta,ifixb,parametername,name,leng)
         call search(N,X,Y,1,N,0,xx,xx,
     :        xx,xx,xx,yy1,xx,xx,.false.,.false.)
         IDEVAL=3
         call fcn_sw(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
         call search(N,X,Y2,1,N,0,xx,xx,
     :        xx,xx,xx,yy2,xx,xx,.false.,.false.)
         beta(2)=beta(2)/yy2*yy1 ! to adjust coverage.
         JOB=2                  
      else
C     UNRECOGNIZED FUNCTION
 304     format('ERROR : Unrecognized function ',A,
     :        '. Type fit_? to get the list') !
         write(6,304) fitfcn(1:lfitfcn)
         return
      endif

      if (.not.expert) write(6,303) 'INFO: don''t forget to set'//
     :     ' the temperature'   !
      call modifyparamodr(BETA,IFIXB,NP,parametername)

C     FUNCTION EVALUATION WITH INITIAL GUESS

      IDEVAL=3
      if (fitfcn(1:lfitfcn).eq.'straight') then
         call fcn_straight(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
c$$$      elseif (fitfcn(1:lfitfcn).eq.'NAME') then
c$$$         call FCN_NAME(N,M,NP,NQ,N,M,NP,BETA,X,
c$$$     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)

C     Nernstian
      elseif (fitfcn(1:lfitfcn).eq.'nernst') then
         call fcn_nernst(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
C     Homogeneous kinetics with remnants of reductant
      elseif (fitfcn(1:lfitfcn).eq.'dt1e') then
         call fcn_dt1e(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'double_layer') then
         call fcn_double_layer(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'double_layer_power') then
         call fcn_double_layer_power(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sigm') then
         call fcn_sigm(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sigmrh5') then
         call fcn_sigmrh5(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sigmco') then
         call fcn_sigmco(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sigm0') then
         call fcn_sigm0(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sigm2') then
         call fcn_sigm2(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'film_expD') then
         call fcn_film_expd(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif ((fitfcn(1:lfitfcn).eq.'expd').or.
     :        (fitfcn(1:lfitfcn).eq.'expD')) then
         call fcn_expd(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'expd1') then
         call fcn_expd1(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif ((fitfcn(1:lfitfcn).eq.'expd2').or.
     :        (fitfcn(1:lfitfcn).eq.'expD2')) then
         call fcn_expd2(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'expd3') then
         call fcn_expd3(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'expd4') then
         call fcn_expd4(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'mm') then
         call fcn_mm(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'mml') then
         call fcn_mml(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lo20') then
         call fcn_lo20(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lo22') then
         call fcn_lo22(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lo21') then
         call fcn_lo21(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lr20') then
         call fcn_lr20(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lr22') then
         call fcn_lr22(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'lr21') then
         call fcn_lr21(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'wo21') then
         call fcn_wo21(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'wo22') then
         call fcn_wo22(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'wr21') then
         call fcn_wr21(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sr21l') then
         call fcn_sr21l(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sr11l') then
         call fcn_sr11l(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sr21n') then
         call fcn_sr21n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sr22n') then
         call fcn_sr22n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sr11n') then
         call fcn_sr11n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so21l') then
         call fcn_so21l(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so11l') then
         call fcn_so11l(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so21n') then
         call fcn_so21n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'napab') then
         call fcn_napab(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so22n') then
         call fcn_so22n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sv21n') then
         call fcn_sv21n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'sv21l') then
         call fcn_sv21l(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so22nc') then
         call fcn_so22nc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'soa1nc') then
         call fcn_soa1nc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'soa2nc') then
         call fcn_soa2nc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'soa3nc') then
         call fcn_soa3nc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'soa4nc') then
         call fcn_soa4nc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
!!!!!!!!!!!!!!!! mind so22nc2 mme fonction que so22nc
      elseif (fitfcn(1:lfitfcn).eq.'so22nc2') then
         call fcn_so22nc(N,M,NP,NQ,N,M,NP,BETA,X, 
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so11n') then
         call fcn_so11n(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'so11nf') then
         call fcn_so11nf(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'woa') then
         call fcn_woa(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'wra') then
         call fcn_wra(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'wc') then
         call fcn_wc(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn-1).eq.'nca') then
         call fcn_nca(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn-1).eq.'ncl') then
         call fcn_ncl(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn-1).eq.'ncI') then
         call fcn_ncI(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn-1).eq.'sw') then
         call fcn_sw(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'ncc2') then
         call fcn_ncc2(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
C     Fit for a mediated reversible inactivation/activation scheme
      elseif (fitfcn(1:lfitfcn).eq.'ktot') then
         call fcn_ktot(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
C     Amplitude of the exponential phase of successive activation steps.
      elseif (fitfcn(1:lfitfcn).eq.'ampl') then
         call fcn_ampl(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'napab_no_s') then
         call fcn_napab_no_s(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      elseif (fitfcn(1:lfitfcn).eq.'napab_activation') then
         call fcn_napab_activation(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
C     Arbitrary function fits (not always available)
      elseif (fitfcn(1:lfitfcn).eq.'arb') then
         call fcn_scalc_arb(N,M,NP,NQ,N,M,NP,BETA,X,
     +        IFIXB,IFIXX,LDIFX,IDEVAL,Y2,FJACB,FJACD,ISTOP)
      else
         write(6,303) 'ERROR : unrecognized function.'//
     :        ' Cannot be evaluated. Oops...' !
         return                 ! instead of stop: there is
                                ! no need to stop the program for that
      endif
      IF (ISTOP.ne.0) then
         write(6,303)'ERROR : function could not be evaluated'
         return
      endif
      call plot(N,X,Y2,0)       ! initial guess
      call gr4_give('X',n,x)
      call gr4_give('Y',n,y2)
c     plot individual peaks, if fitting non catalytic stuff
      if ((fitfcn(1:lfitfcn-1).eq.'nca').and.((NP-1)/3.ge.2)) then
         call plot_all_yis_nc(beta,NP,X,N,1) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'ncl').and.((NP-1)/3.ge.2)) then !
         call plot_all_yis_nc(beta,NP,X,N,2) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'ncI').and.((NP)/3.ge.2)) then !
         call plot_all_yis_nc(beta,NP,X,N,3) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'sw').and.((NP-4)/2.ge.2)) then !
         call plot_all_yis_sw(beta,NP,X,N) !
      endif
      write(6,303) 'INFO : Initial guess is overlaid'

C     MODIFY PARAMETERS?
      
      call modifyparamodr(BETA,IFIXB,NP,parametername)

C     let's go?
      call confirm('fit',3,conf,.true.)
      if (.not.conf) then
         datachanged=.false.
         return
      endif

C     call to SODRC (FIT)
      write(6,303) 'WAIT...'
      call wait
      if (fitfcn(1:lfitfcn).eq.'straight') then
c     straight
         CALL SODRC(fcn_straight,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
c$$$      elseif (fitfcn(1:lfitfcn).eq.'NAME') then
c$$$c     NAME
c$$$         CALL SODRC(FCN_NAME,
c$$$     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
c$$$     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
c$$$     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
c$$$     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'nernst') then
c     nernst
         CALL SODRC(fcn_nernst,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'double_layer') then
         CALL SODRC(fcn_double_layer,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'double_layer_power') then
         CALL SODRC(fcn_double_layer_power,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'dt1e') then
c     nernst
         CALL SODRC(fcn_dt1e,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sigm') then
c     sigm
         CALL SODRC(fcn_sigm,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sigmrh5') then
c     sigmrh5
         CALL SODRC(fcn_sigmrh5,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sigmco') then
c     sigmco
         CALL SODRC(fcn_sigmco,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sigm0') then
c     sigm0
         CALL SODRC(fcn_sigm0,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sigm2') then
c     sigm2
         CALL SODRC(fcn_sigm2,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'mm') then
c     mm
         CALL SODRC(fcn_mm,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'mml') then
c     mml
         CALL SODRC(fcn_mml,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif ((fitfcn(1:lfitfcn).eq.'expd').or.
     :        (fitfcn(1:lfitfcn).eq.'expD')) then
c     expd
         CALL SODRC(fcn_expd,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'film_expD') then
         CALL SODRC(fcn_film_expd,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'expd1') then
c     expd1
         CALL SODRC(fcn_expd1,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif ((fitfcn(1:lfitfcn).eq.'expd2').or.
     :        (fitfcn(1:lfitfcn).eq.'expD2')) then
c     expd2
         CALL SODRC(fcn_expd2,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'expd3') then
c     expd3
         CALL SODRC(fcn_expd3,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'expd4') then
c     expd4
         CALL SODRC(fcn_expd4,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lo20') then
c     for
         CALL SODRC(fcn_lo20,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lo22') then
c     foq2
         CALL SODRC(fcn_lo22,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lo21') then 
c     foq1
         CALL SODRC(fcn_lo21,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lr20') then
c     frr
         CALL SODRC(fcn_lr20,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lr22') then
c     frq2
         CALL SODRC(fcn_lr22,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sr21l') then
c     sr21l
         CALL SODRC(fcn_sr21l,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sr11l') then
c     sr11l
         CALL SODRC(fcn_sr11l,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sr21n') then
c     sr21n
         CALL SODRC(fcn_sr21n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sr22n') then
c     sr22n
         CALL SODRC(fcn_sr22n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sr11n') then
c     sr11n
         CALL SODRC(fcn_sr11n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so21l') then
c     so21l
         CALL SODRC(fcn_so21l,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so11l') then
c     so11l
         CALL SODRC(fcn_so11l,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so21n') then
c     so21n
         CALL SODRC(fcn_so21n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'napab') then
c     napab
         CALL SODRC(fcn_napab,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so22n') then
c     so22n
         CALL SODRC(fcn_so22n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sv21n') then
c     sv21n
         CALL SODRC(fcn_sv21n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'sv21l') then
c     sv21l
         CALL SODRC(fcn_sv21l,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so22nc') then
c     so22nc
         CALL SODRC(fcn_so22nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'soa1nc') then
c     soa1nc
         CALL SODRC(fcn_soa1nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'soa2nc') then
c     soa2nc
         CALL SODRC(fcn_soa2nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'soa3nc') then
c     soa3nc
         CALL SODRC(fcn_soa3nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'soa4nc') then
c     soa4nc
         CALL SODRC(fcn_soa4nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so22nc2') then
c     so22nc2 ! mme fonc que so22n
         CALL SODRC(fcn_so22nc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so11n') then
c     so11n
         CALL SODRC(fcn_so11n,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'so11nf') then
c     so11nf
         CALL SODRC(fcn_so11nf,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'lr21') then
c     lr21
         CALL SODRC(fcn_lr21,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'wo21') then
c     wo21
         CALL SODRC(fcn_wo21,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'wo22') then
c     wo22
         CALL SODRC(fcn_wo22,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'wr21') then
c     wr21
         CALL SODRC(fcn_wr21,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'wra') then
c     wra
         CALL SODRC(fcn_wra,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'woa') then
c     woa
         CALL SODRC(fcn_woa,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'wc') then
c     wc
         CALL SODRC(fcn_wc,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn-1).eq.'nca') then
c     ncai
         CALL SODRC(fcn_nca,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn-1).eq.'ncl') then
c     ncli
         CALL SODRC(fcn_ncl,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn-1).eq.'ncI') then
c     ncIi
         CALL SODRC(fcn_ncI,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn-1).eq.'sw') then
c     sw
         CALL SODRC(fcn_sw,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'ncc2') then
c     ncc2
         CALL SODRC(fcn_ncc2,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'ktot') then
c     ktot
         CALL SODRC(fcn_ktot,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'ampl') then
c     ktot
         CALL SODRC(fcn_ampl,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'napab_no_s') then
         CALL SODRC(fcn_napab_no_s,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'napab_activation') then
         CALL SODRC(fcn_napab_activation,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      elseif (fitfcn(1:lfitfcn).eq.'arb') then
c     Arbitrary functions
         CALL SODRC(fcn_scalc_arb,
     +        N,M,NP,NQ,BETA,Y,LDY,X,LDX,WE,LDWE,LD2WE,WD,LDWD,LD2WD,
     +        IFIXB,IFIXX,LDIFX,JOB,NDIGIT,TAUFAC,SSTOL,PARTOL,MAXIT,
     +        IPRINT,LUNERR,LUNRPT,STPB,STPD,LDSTPD,SCLB,SCLD,
     +        LDSCLD,WORK,LWORK,IWORK,LIWORK,INFO)
      else
 303     format(A)
         write(6,303) 'ERROR : unrecognized function. Cannot fit'
         return
      endif

c     RECOVER INFORMATION (best fit) FROM WORK ARRAY
      IF (MOD(JOB,10).eq.2) then
         ISODR=.false.          ! ordinary least square
      else
         ISODR=.true.
      endif
      CALL SWINF(N,M,NP,NQ,LDWE,LD2WE,ISODR, 
     :     DELTAI,EPSI,XPLUSI,FNI,SDI,VCVI,
     :     RVARI,WSSI,WSSDEI,WSSEPI,RCONDI,ETAI,
     :     OLMAVI,TAUI,ALPHAI,ACTRSI,PNORMI,RNORSI,PRERSI,
     :     PARTLI,SSTOLI,TAUFCI,EPSMAI,
     :     BETA0I,BETACI,BETASI,BETANI,SI,SSI,SSFI,QRAUXI,UI,
     :     FSI,FJACBI,WE1I,DIFFI,
     :     DELTSI,DELTNI,TI,TTI,OMEGAI,FJACDI,
     :     WRK1I,WRK2I,WRK3I,WRK4I,WRK5I,WRK6I,WRK7I,
     :     LWKMN)
c     FINAL ESTIMATE = BEST FIT
      do i=1,N
         y2(i,1)=WORK(FNI-1+I)
      enddo

      write(6,*) 'Sum of squares,', WORK(WSSI)


C     We write a small description of the fit:
 372     format('Fit ', A, ' from X = ', G12.4, ' to ', G12.4)
         write(6, 372) fitfcn(1:lfitfcn), x_start, x_end

C     We get the values of the covariance matrix

      do i=1,np
         do j=1,np
            vcv(i,j)=work(vcvi-1+I+(J-1)*NP)*work(rvari)
         enddo
      enddo

      forceraw=.true.
      call writeparamodr(BETA,NP,parametername,leng,name,6,forceraw)

C     Now printing the covariance matrix...
      call writecovariancematrix(np, parametername, beta, vcv)
      
C     OVERLAY BEST FIT
      call plot(N,X,Y,1)        ! raw data
      call plot(N,X,Y2,0)       ! fit overlaid

c      write(6,*)  (Y(i,1),i=1,N)
c      write(6,*)  (Y2(i,1),i=1,N)

c     plot deconvoluted noncat signals, if relevent
      if ((fitfcn(1:lfitfcn-1).eq.'nca').and.((NP-1)/3.ge.2)) then
         call plot_all_yis_nc(beta,NP,X,N,1) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'ncl').and.((NP-1)/3.ge.2)) then !
         call plot_all_yis_nc(beta,NP,X,N,2) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'ncI').and.((NP)/3.ge.2)) then !
         call plot_all_yis_nc(beta,NP,X,N,3) !
      elseif ((fitfcn(1:lfitfcn-1).eq.'sw').and.((NP-4)/2.ge.2)) then !
         call plot_all_yis_sw(beta,NP,X,N) !
      endif
c     
      call plotdiff(x,y,y2,0.,N,.false.) ! residue

      call initgreg             ! recover box parameters
      call reclim(np,x,y)       ! recover limits

C     We should write the fits results into output file -- along
C     with their respective errors, the resulting sum of squares
C     and many other interesting things.
      if(autoout) then
         call auto_append_best_params(beta, np, variance, WORK(WSSI),
     :        x_start, x_end)
         conf = .false.
      elseif(askout) then
         
         call confirm('write best fits to output file',30,
     :        conf, .true.)
         if(conf) then
            call auto_append_best_params(beta, np, variance, WORK(WSSI),
     :        x_start, x_end)
         endif
         conf = .false.
      else
C     If there is automatic output, it's me ;-) (Vincent), and I'm pretty sure
C     I won't need to print - it also doesn't make so much sense to
C     ask for output
         call confirm('print',5,conf,.false.)
         if (conf) call printscreen
         call confirm('update the buffer with the best fit and add '//
     :        'the best parameters to std output file',82,conf,.false.) !
      endif
      

      
      if (conf) then            ! We replace the buffer by the best fit.
c     deconvolution of non cat signals only
         if (                   
     :        ((fitfcn(1:lfitfcn-1).eq.'nca').and.((NP-1)/3.ge.2))
     :        .or.
     :        ((fitfcn(1:lfitfcn-1).eq.'ncl').and.((NP-1)/3.ge.2))
     :        .or.
     :        ((fitfcn(1:lfitfcn-1).eq.'ncI').and.((NP)/3.ge.2))
     :        .or.
     :        ((fitfcn(1:lfitfcn-1).eq.'sw').and.((NP-4)/2.ge.2))
     :        ) then            ! puts into oname name(1:leng)//'.fit.res' 
            call get_filename(leng,name,oleng,oname)
            if (oleng.ne.0) then
               if ((fitfcn(1:lfitfcn-1).eq.'nca').and.((NP-1)/3.ge.2)) 
     :              then
                  call write_all_yis_nc(beta,NP,X,Y,Y2,N,parametername,
     :                 leng,name,oleng,oname,1) !
               elseif ((fitfcn(1:lfitfcn-1).eq.'ncl')
     :                 .and.((NP-1)/3.ge.2)) then !
                  call write_all_yis_nc(beta,NP,X,Y,Y2,N,parametername,
     :                 leng,name,oleng,oname,2) !
               elseif ((fitfcn(1:lfitfcn-1).eq.'ncI').and.((NP)/3.ge.2)) 
     :                 then     !
                  call write_all_yis_nc(beta,NP,X,Y,Y2,N,parametername,
     :                 leng,name,oleng,oname,3) !
               elseif ((fitfcn(1:lfitfcn-1).eq.'sw')
     :                 .and.((NP-4)/2.ge.2)) then !
                  call write_all_yis_sw(beta,NP,X,Y,Y2,N,parametername,
     :                 leng,name,oleng,oname) !
               endif
               do i=1,N
                  Y(i,1)=Y2(i,1) ! update buffer with best fit
               enddo
               datachanged=.true.
               call append_fit_2_name(name,leng)
               call AppendOutFileBestParam(beta,np,parametername,
     :              leng,name) 
            endif
         else                   ! other than deconvolution
            do i=1,N
               Y(i,1)=Y2(i,1)   ! update buffer with best fit
            enddo
            datachanged=.true.
            call append_fit_2_name(name,leng)
            call AppendOutFileBestParam(beta,np,parametername,leng,name) 
         endif
      else
         datachanged=.false.
      endif
      return
      end

************************************************************************
C  SUBROUTINE ARGUMENTS
C      ==> N        NUMBER OF OBSERVATIONS
C      ==> M        NUMBER OF COLUMNS IN EXPLANATORY VARIABLE
C      ==> NP       NUMBER OF PARAMETERS
C      ==> NQ       NUMBER OF RESPONSES PER OBSERVATION
C      ==> LDN      LEADING DIMENSION DECLARATOR EQUAL OR EXCEEDING N
C      ==> LDM      LEADING DIMENSION DECLARATOR EQUAL OR EXCEEDING M
C      ==> LDNP     LEADING DIMENSION DECLARATOR EQUAL OR EXCEEDING NP
C      ==> BETA     CURRENT VALUES OF PARAMETERS
C      ==> XPLUSD   CURRENT VALUE OF EXPLANATORY VARIABLE, I.E., X + DELTA
C      ==> IFIXB    INDICATORS FOR "FIXING" PARAMETERS (BETA)
C      ==> IFIXX    INDICATORS FOR "FIXING" EXPLANATORY VARIABLE (X)
C      ==> LDIFX    LEADING DIMENSION OF ARRAY IFIXX
C      ==> IDEVAL   INDICATOR FOR SELECTING COMPUTATION TO BE PERFORMED
C     <==  F        PREDICTED FUNCTION VALUES
C     <==  FJACB    JACOBIAN WITH RESPECT TO BETA
C     <==  FJACD    JACOBIAN WITH RESPECT TO ERRORS DELTA
C     <==  ISTOP    STOPPING CONDITION, WHERE
C                     0 MEANS CURRENT BETA AND X+DELTA WERE
C                       ACCEPTABLE AND VALUES WERE COMPUTED SUCCESSFULLY
C                     1 MEANS CURRENT BETA AND X+DELTA ARE
C                       NOT ACCEPTABLE;  ODRPACK SHOULD SELECT VALUES 
C                       CLOSER TO MOST RECENTLY USED VALUES IF POSSIBLE
C                    -1 MEANS CURRENT BETA AND X+DELTA ARE
C                       NOT ACCEPTABLE;  ODRPACK SHOULD STOP

c$$$  you can use the following example
c$$$      SUBROUTINE fcn_NAME(N,M,NP,NQ,
c$$$     +               LDN,LDM,LDNP,
c$$$     +               BETA,XPLUSD,
c$$$     +               IFIXB,IFIXX,LDIFX,
c$$$     +               IDEVAL,F,FJACB,FJACD,
c$$$     +               ISTOP)
c$$$      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
c$$$      REAL             BETA(NP),XPLUSD(LDN,M)
c$$$      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
c$$$      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
c$$$c     test for acceptable values of the parameters
c$$$c     let's suppose beta(1)<0 is not an acceptable value
c$$$      IF (BETA(1).le.0)) then
c$$$c     the program will try again with other value
c$$$         ISTOP=1
c$$$         return 
c$$$      else
c$$$         ISTOP=0
c$$$      endif
c$$$C     COMPUTE FUNCTION
c$$$      IF (MOD(IDEVAL,10).GE.1) THEN
c$$$         DO 110 L = 1,NQ        ! nq=1
c$$$            DO 100 I = 1,N      ! n points
c$$$               F(I,L)=... expression (beta(1--np),XPLUSD(I,1))
c$$$c                y                     parameters    x   
c$$$ 100        CONTINUE
c$$$ 110     CONTINUE
c$$$      END IF
c$$$      RETURN
c$$$      END

      SUBROUTINE fcn_straight(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 110 L = 1,NQ        ! nq=1
            DO 100 I = 1,N      ! n points
               F(I,L)=beta(1)*XPLUSD(I,1)+beta(2)
 100        CONTINUE
 110     CONTINUE
      END IF
C     COMPUTE DERIVATIVES WITH RESPECT TO BETA
      IF (MOD(IDEVAL/10,10).GE.1) THEN
         DO 210 L = 1,NQ
            DO 200 I = 1,N
               FJACB(I,1,L) = XPLUSD(I,1)
               FJACB(I,2,L) = 1.
 200        CONTINUE
 210     CONTINUE
      END IF
C     COMPUTE DERIVATIVES WITH RESPECT TO DELTA
      IF (MOD(IDEVAL/100,10).GE.1) THEN
         DO 310 L = 1,NQ
            DO 300 I = 1,N
               FJACD(I,1,L) = BETA(1)
 300        CONTINUE
 310     CONTINUE
      END IF
      RETURN
      END
c$$$         parametername(1)='a1'   ! (1-a1)*exp(-x/t1)+a2*exp(-t/b2)+(a1-a2)'
c$$$         parametername(2)='a2'
c$$$         parametername(3)='t1'   
c$$$         parametername(4)='t2'

      SUBROUTINE fcn_expd(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      if (beta(2).eq.0) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)*exp(-XPLUSD(I,1)/beta(2))+beta(3)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_film_expd(N,M,NP,NQ,
     +     LDN,LDM,LDNP,
     +     BETA,XPLUSD,
     +     IFIXB,IFIXX,LDIFX,
     +     IDEVAL,F,FJACB,FJACD,
     +     ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      if (beta(2).eq.0) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=(beta(1)*exp(-XPLUSD(I,1)/beta(2)) + beta(3)) *
     :           exp(- beta(4) * xplusd(i,1))
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_double_layer(N,M,NP,NQ,
     +     LDN,LDM,LDNP,
     +     BETA,XPLUSD,
     +     IFIXB,IFIXX,LDIFX,
     +     IDEVAL,F,FJACB,FJACD,
     +     ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION

      IF (MOD(IDEVAL,10).GE.1) THEN
         DO I = 1,N         ! n points
            f(i,1)=beta(1) + beta(2)*xplusd(i,1) +
     :           beta(3)*exp(-abs((xplusd(1,1)-xplusd(i,1)))/beta(4)) +
     :           beta(5)*exp(abs((xplusd(1,1)-xplusd(i,1)))/beta(6))
         enddo
      END IF
      RETURN
      END

      subroutine fcn_scalc_arb(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)

C     For the temperature.
      real temp
      common /temperature/ temp

      external scalccompute, scalcsettemperature

C     We make sure the temperature is set right.
      call scalcsettemperature(temp)

      istop = 0
      if (mod(ideval,10).ge.1) then
         call scalccompute(n, xplusd, f, beta) ! ridiculously simple,
                                               ! isn't it ???
      endif
      return 
      end


      SUBROUTINE fcn_expd1(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      if (beta(2).eq.0) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=(1-beta(1))*(exp(-XPLUSD(I,1)/beta(2)))+beta(1)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_nernst(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)+beta(2)/(1.+
     :           exp(beta(3)*fara*(xplusd(i,1)-beta(4))))
 100     CONTINUE
      END IF
      RETURN
      END

C     Homogeneous kinetics with a one-electron reductor not fully consumed
C     before enzyme addition.
C
C     We fit d[med]/dt as a function of [med]
      SUBROUTINE fcn_dt1e(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)

C     Local variables:
      real rate, fact

      ISTOP=0
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO I = 1,N         ! n points

C     The normal rate: (MM)
            rate = - beta(1)/(1 + beta(2)/xplusd(i,1))
C     The modification due to the equilibrium
            fact = 1 + beta(4) * beta(3)*beta(5)/((beta(5) -
     :           (1 - beta(4)) * xplusd(i,1))**2)
            F(I,1)= rate/fact
         enddo
      END IF
      RETURN
      END

c

C     The mediated reversible reductive reactivation scheme
      SUBROUTINE fcn_ktot(N,M,NP,NQ,
     +     LDN,LDM,LDNP,
     +     BETA,XPLUSD,
     +     IFIXB,IFIXX,LDIFX,
     +     IDEVAL,F,FJACB,FJACD,
     +     ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara_2
      common /temperature/ temp
      ISTOP=0
      fara_2=96500./8.3144/temp/2
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO I = 1,N             ! n points
            F(I,1)= beta(3) * cosh((beta(1) - xplusd(i,1)) * fara_2) /
     :           cosh((beta(2) - xplusd(i,1)) * fara_2)
         enddo
      END IF
      RETURN
      END

C     Fits of the amplitude of the exponential phase for successive
C     steps where a Nernstian activation occur.
      SUBROUTINE fcn_ampl(N,M,NP,NQ,
     +     LDN,LDM,LDNP,
     +     BETA,XPLUSD,
     +     IFIXB,IFIXX,LDIFX,
     +     IDEVAL,F,FJACB,FJACD,
     +     ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara
C     The square of e_0 and \delta variables... (not E_0 and \Delta)
      real e0_2,delta_2
      common /temperature/ temp
      ISTOP=0
      if ((0.gt.beta(2)).or.(beta(2).gt.1)) then
         ISTOP=1
         return
      endif
      fara=96500./8.3144/temp
      delta_2 = exp( - fara * beta(3))
      IF (MOD(IDEVAL,10).GE.1) THEN
         do i = 1,n             ! n points
            e0_2 = exp(fara * (xplusd(i,1) - beta(1)))
            f(I,1)= (1 - beta(2)) * e0_2 * (1 - delta_2) /
     :           ((1 + e0_2 * delta_2) * (1 + beta(2) * e0_2))
         enddo
      END IF
      RETURN
      END

      subroutine fcn_napab_no_s(n,m,np,nq,
     +     ldn,ldm,ldnp,
     +     beta,xplusd,
     +     ifixb,ifixx,ldifx,
     +     ideval,f,fjacb,fjacd,
     +     istop)
      integer          i,ideval,istop,l,ldifx,ldm,ldn,ldnp,m,n,np,nq
      real             beta(np),xplusd(ldn,m)
      integer          ifixb(np),ifixx(ldifx,m)
      real             f(ldn,nq),fjacb(ldn,ldnp,nq),fjacd(ldn,ldm,nq)
      real temp,fara
      common /temperature/ temp
C     Now, some intermediate variables:
      real e1, e2, e3, a, b
      fara=96500./8.3144/temp
      istop = 0
C     Some safeguards
      if( (beta(4).le.0).or.(beta(5).le.0)) then
         istop = 1
         return
      endif

      if (mod(ideval,10).ge.1) then
         do i = 1,n             ! n points
            e1 = exp(fara * (xplusd(i,1) - beta(1)))
            e2 = exp(fara * (xplusd(i,1) - beta(2)))
            e3 = exp(fara * (xplusd(i,1) - beta(3)))

C     \approx Flow from the substrate-less states
            a = (beta(4) * e1 + beta(5))/(1 + e1)
C     \approx Catalytic flow
            b = 1/(1 + e3 + e2*e3)
            
            f(i,1) = a/(a + b) * beta(6)/(1 + e3 + e2*e3)
         enddo
      end if
      return
      end


      subroutine fcn_napab_activation(n,m,np,nq,
     +     ldn,ldm,ldnp,
     +     beta,xplusd,
     +     ifixb,ifixx,ldifx,
     +     ideval,f,fjacb,fjacd,
     +     istop)
      integer          i,ideval,istop,l,ldifx,ldm,ldn,ldnp,m,n,np,nq
      real             beta(np),xplusd(ldn,m)
      integer          ifixb(np),ifixx(ldifx,m)
      real             f(ldn,nq),fjacb(ldn,ldnp,nq),fjacd(ldn,ldm,nq)
      real temp,fara
      common /temperature/ temp
C     Now, some intermediate variables:
      real e1, e2, e3, e4, a, b, kdeff, kaeff
      fara=96500./8.3144/temp
      istop = 0
C     Some safeguards
      if( (beta(4).le.0).or.(beta(5).le.0)) then
         istop = 1
         return
      endif

      if (mod(ideval,10).ge.1) then
         do i = 1,n             ! n points
            e1 = exp(fara * (xplusd(i,1) - beta(1)))
            e2 = exp(fara * (xplusd(i,1) - beta(2)))
            e3 = exp(fara * (xplusd(i,1) - beta(3)))
            e4 = exp(fara * (xplusd(i,1) - beta(8)))

C     \approx Flow from the substrate-less states
            a = (beta(4) * e1 + beta(5))/(1 + e1)
C     \approx Catalytic flow
            b = 1/(1 + e3 + e2*e3)

C     The effective activation and inactivation constants
            kdeff = a/(a + b) * beta(6)/(1 + e3 + e2*e3)
            kaeff = beta(7)/(1 + e4)
            
            f(i,1) = kdeff + kaeff
         enddo
      end if
      return
      end


      SUBROUTINE fcn_sigm2(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
c         parametername(1)='i/ki' ! 1/(1+a1*exp(-t/a2))+a3/(1+a1**(-1)*exp(-t/a2))
c         parametername(2)='tau'
c         parametername(3)='res'
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=1/(1+beta(1)*exp(-xplusd(i,1)/beta(2)))
     :           +beta(3)/(1+beta(1)**(-1)*exp(xplusd(i,1)/beta(2)))
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_sigm(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            if (xplusd(i,1).gt.beta(4)) then
               F(I,1)=beta(1)/(1+beta(2)*
     :              exp((xplusd(i,1)-beta(4))/beta(3)))
            else
               F(I,1)=beta(1)/(1+beta(2))
            endif
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_sigmrh5(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            if (xplusd(i,1).gt.beta(4)) then
               F(I,1)=beta(1)/(1+beta(2)/
     :              (0.1+0.9*exp(-(xplusd(i,1)-beta(4))/beta(3))))
            else
               F(I,1)=beta(1)/(1+beta(2))
            endif
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_sigmco(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0

c         parametername(1)='ilim' ! a1/(1+a2*exp(t/a3))
c         parametername(2)='km/c0'
c         parametername(3)='tau'
c         parametername(4)='t0'
c         parametername(5)='i0/kic'
c         parametername(6)='t1'
cc         parametername(7)='i0/kiu'
c         parametername(7)='tauco'

c         parametername(1)='imax' ! a1/(1+a2*exp(t/a3))
c         parametername(2)='km/s'
c         parametername(3)='i0/kic'
c         parametername(4)='i0/kiu'
c         parametername(5)='t0'
c         parametername(6)='t1'
c         parametername(7)='taus'
c         parametername(8)='taui'
 

      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            if (xplusd(i,1).gt.beta(6)) then
               F(I,1)=beta(1)
     :              /(1+beta(4)*exp((beta(6)-xplusd(i,1))/beta(8)))
     :              /(1+beta(2)*exp((xplusd(i,1)-beta(5))/beta(7))
     :              *(1+beta(3)*exp((beta(6)-xplusd(i,1))/beta(8)))
     :              /(1+beta(4)*exp((beta(6)-xplusd(i,1))/beta(8))))
            elseif (xplusd(i,1).gt.beta(5)) then 
               F(I,1)=beta(1)/(1+
     :              beta(2)*exp((xplusd(i,1)-beta(5))/beta(7)))
            else
               F(I,1)=beta(1)/(1+beta(2))
            endif
 100     CONTINUE
      END IF
      RETURN
      END

c
      SUBROUTINE fcn_sigm0(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)/(1+beta(2)*exp(xplusd(i,1)/beta(3)))
 100     CONTINUE
      END IF
      RETURN
      END
c

      SUBROUTINE fcn_expd2(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      if ((beta(2).eq.0).or.(beta(4).eq.0)) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)*exp(-XPLUSD(I,1)/beta(2))
     :           +beta(3)*exp(-XPLUSD(I,1)/beta(4))
     :           +beta(5)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_expd3(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
c$$$         parametername(1)='a1'   ! (a1)*exp(-x/t1)+a2*exp(-t/b2)+(1-a1-a2)'
c$$$         parametername(2)='a2'
c$$$         parametername(3)='t1'   
c$$$         parametername(4)='t2'
C     COMPUTE FUNCTION
      if ((beta(3).eq.0).or.(beta(4).le.0)) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=(beta(1))*exp(-XPLUSD(I,1)/beta(3))
     :           +beta(2)*exp(-XPLUSD(I,1)/beta(4))
     :           +1-beta(1)-beta(2)
 100     CONTINUE
      END IF
      RETURN
      END
      SUBROUTINE fcn_expd4(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
c     (b1)*exp(-x/b2)+b3*exp(-t/b4)-b1-b3

C     COMPUTE FUNCTION
      if ((beta(2).le.0).or.(beta(4).le.0)) then
         ISTOP=1
         return
      endif
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=(beta(1))*exp(-XPLUSD(I,1)/beta(2))
     :           +beta(3)*exp(-XPLUSD(I,1)/beta(4))
     :           -beta(1)-beta(3)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_mm(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      DO 110 L = 1,NQ           ! nq=1
         DO 100 I = 1,N         ! n points
            F(I,L)=beta(1)/(1+beta(2)/XPLUSD(I,1))
 100     CONTINUE
 110  CONTINUE
      RETURN
      END

      SUBROUTINE fcn_mml(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      ISTOP=0
C     COMPUTE FUNCTION
      DO 110 L = 1,NQ           ! nq=1
         DO 100 I = 1,N         ! n points
            F(I,L)=beta(1)/(1+beta(2)/10**(XPLUSD(I,1)))
 100     CONTINUE
 110  CONTINUE
      RETURN
      END


************************************************************************
*     LOG transforms
************************************************************************

      SUBROUTINE fcn_lo20(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e1^(-1/2)
            e2=exp(fara*(beta(2)-xplusd(I,1))) ! e2^(-1/2)
            F(I,1)=log10(e1*(1.+e2)) ! checked
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_lo22(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
c$$$  this common gives the value of the temperature in temp
      If ((beta(3).lt.0).or.(beta(4).lt.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0(O/I)'
c     parametername(4)='k2/k0(I/R)'
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e1(-1/2)
            e2=exp(fara*(beta(2)-xplusd(I,1))) ! e2(-1/2)
            F(I,1)=log10(       !
     :           e1*(1.+e2)     ! 
     :           +beta(4)*(sqrt(e2)) ! k2/k02
     :           +beta(3)*(sqrt(e1)*(1.+e2)) ! k2/k01
     :           )              ! checked
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_lo21(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      If ((beta(3).lt.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     ISTOP=0
      fara=96500./8.3144/temp
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0'
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            F(I,1)=log10(
     :           e1*(1.+e2)     ! 
     :           +beta(3)*(sqrt(e2)) !
     :           +beta(3)*(sqrt(e1)*(1.+e2)) !
     :           )              ! checked
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_lr20(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) ! e1
            e2=exp(fara*(xplusd(I,1)-beta(2))) ! e2
            F(I,1)=log10(e2*(1.+e1)) ! checked
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_lr22(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      If ((beta(3).le.0).or.(beta(4).le.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     ISTOP=0
      fara=96500./8.3144/temp
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0(O/I)'
c     parametername(4)='k2/k0(I/R)'
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) ! e1
            e2=exp(fara*(xplusd(I,1)-beta(2))) ! e2
            F(I,1)=log10(
     :           e2*(1.+e1)     !
     :           +beta(3)*(sqrt(e1)) ! k2/k01
     :           +beta(4)*(sqrt(e2)*(1.+e1)) ! k2/k02
     :           )       
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_lr21(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      If ((beta(3).le.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     ISTOP=0
      fara=96500./8.3144/temp
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0'
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) !
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            F(I,1)=log10(
     :           e2*(1.+e1)     !
     :           +beta(3)*(sqrt(e1)) ! ok
     :           +beta(3)*(sqrt(e2)*(1.+e1)) 
     :           )       
 100     CONTINUE
      END IF
      RETURN
      END

************************************************************************
*     fit waves 2 electrons
************************************************************************

      SUBROUTINE fcn_wr21(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      If ((beta(3).le.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     ISTOP=0
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0'
c     parametername(4)='ilim' 
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) ! e1
            e2=exp(fara*(xplusd(I,1)-beta(2))) ! e2
            F(I,1)=beta(4)/(1.+
     :           e2*(1.+e1)     !
     :           +beta(3)*(sqrt(e1)) !
     :           +beta(3)*(sqrt(e2)*(1.+e1)) 
     :           )              ! checked
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_wo21(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      If ((beta(3).lt.0)) then
         ISTOP= 1
         return
      else
         ISTOP=0
      endif
c     ISTOP=0
c     parametername(1)='E(O/I)'
c     parametername(2)='E(I/R)'
c     parametername(3)='k2/k0'
c     parametername(4)='ilim' 
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e1
            e2=exp(fara*(beta(2)-xplusd(I,1))) ! e2
            F(I,1)=beta(4)/(1.+
     :           e1*(1.+e2)     ! 
     :           +beta(3)*(sqrt(e2)) !
     :           +beta(3)*(sqrt(e1)*(1.+e2)) !
     :           )              ! checked
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_wo22(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2
      common /temperature/ temp
      ISTOP=0
c     parametername(1)= EOI
c     parametername(2)= EIR
c     parametername(3)='log(k2/k0)' IR
c     parametername(4)='log(k2/k0)' OI
c     parametername(5)='ilim' 
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e1=0/I
            e2=exp(fara*(beta(2)-xplusd(I,1))) ! e2=I/R
            F(I,1)=beta(5)/(1.+
     :           e1*(1.+e2)     ! 
     :           +10**(beta(3))*(sqrt(e2)) !
     :           +10**(beta(4))*(sqrt(e1)*(1.+e2)) !
     :           )              ! checked
 100     CONTINUE
      END IF
      RETURN
      END


************************************************************************
*     fit slopey waves 2 electrons
************************************************************************

      SUBROUTINE fcn_sr21l(N,M,NP,NQ, ! xqr2nb
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) !
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            a=1.0+e2*(1.+e1)    !
            y2=10**beta(3)*((sqrt(e1))+(sqrt(e2)*(1.+e1))) ! 
                                ! beta(3)=log(k2/k0M)
                                ! beta(5)=ilim
            y1=y2*exp(10**beta(4))  ! beta(4)=log(beta d_0)
            F(I,1)=beta(5)/a*(1.+(log((a+y2)/(a+y1))/10**beta(4)))
 100     CONTINUE
      END IF
      RETURN
      END
c     
      SUBROUTINE fcn_sr21n(N,M,NP,NQ, ! xqr2nbnl
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) !
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            a=1.0+e2*(1.+e1)
            y2=10**beta(3)*((sqrt(e1))+(sqrt(e2)*(1.+e1))) 
c     beta(3) = log10(k2/k0M)
c     beta(4) = ilim/beta d_0
            F(I,1)=beta(4)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END
c     
      SUBROUTINE fcn_sr22n(N,M,NP,NQ, 
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1))) !
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            a=1.0+e2*(1.+e1)
            y2=10**beta(3)*(sqrt(e1))
     :           +10**beta(4)*(sqrt(e2)*(1.+e1))
c     beta(3) = log10(k2/k0M)
c     beta(5) = ilim/beta d_0
            F(I,1)=beta(5)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_napab(N,M,NP,NQ, ! xqr2nb
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
c         parametername(1)='E1'
c        parametername(2)='E2'
c        parametername(3)='E3'
c        parametername(4)='km/s'
c         parametername(5)='k5/k4'
c         parametername(6)='naAGk2'
c         beta(1)=-.25  
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1)))
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            e3=exp(fara*(xplusd(I,1)-beta(3)))
            F(I,1)=beta(6)/(1+e2*(1+e3)+beta(4)*(1+e1)/(1+beta(5)*e1))
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_so21l(N,M,NP,NQ, ! xqr2nb
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            a=1.0+e1*(1.+e2)    !
            y2=10**beta(3)*((sqrt(e2))+(sqrt(e1)*(1.+e2))) ! 
c     beta(3)=log10(k2/k0M)
c     beta(5)=ilim
c     beta(4)=log10(beta d_0)
            y1=y2*exp(10**beta(4))  
            F(I,1)=beta(5)/a*(1.+(log((a+y2)/(a+y1))/10**beta(4)))
 100     CONTINUE
      END IF
      RETURN
      END
c     
      SUBROUTINE fcn_so21n(N,M,NP,NQ, ! xqr2nbnl
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            a=1.0+e1*(1.+e2)
            y2=10**beta(3)*((sqrt(e2))+(sqrt(e1)*(1.+e2))) 
c     beta(3) = log10(k2/k0M)
c     beta(4) = ilim/beta d_0
            F(I,1)=beta(4)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END

c     
      SUBROUTINE fcn_so22n(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            a=1.0+e1*(1.+e2)
            y2=10**beta(4)*(sqrt(e2)+sqrt(e1))
     :           +10**beta(3)*(sqrt(e1)*e2)
c     beta(3 et 4) = log10(k2/k0M)
c     beta(5) = ilim/beta d_0
            F(I,1)=beta(5)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END

c     
      SUBROUTINE fcn_sv21n(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
c     (1)='E(1)'
c     (2)='E(2)'
c     (3)='log(koxk0M)'
c     (4)='kred/kox'
c     (5)='ilimox/betad0'

      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1)))
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            aox=1.0+e1**(-1.0)*(1.+e2**(-1.0))
            box=e2**(-.5)+e1**(-.5)*(1+e2**(-1))
            cox=e2**(-.5)*(1+e1**(-.5)*(1+e1**(-.5)))
            ared=1+e2*(1+e1)
            bred=sqrt(e1)+sqrt(e2)*(1+e1)
            cred=sqrt(e1)*(1+sqrt(e2)*(1+sqrt(e2)))

            F(I,1)=beta(5)*(1/aox*log((aox+10**beta(3)*(box+beta(4)*cox))
     :           /(10**beta(3)*(box+beta(4)*cox)))
     :           -beta(4)/ared*log((ared+10**beta(3)*(cred+beta(4)*bred))
     :           /(10**beta(3)*(cred+beta(4)*bred))))


 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_sv21l(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
c     (1)='E(1)'
c     (2)='E(2)'
c     (3)='log(koxk0M)'
c     (4)='kred/kox'
c     (5)='ilimox'
c     (6)=betad0'

      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(xplusd(I,1)-beta(1)))
            e2=exp(fara*(xplusd(I,1)-beta(2)))
            aox=1.0+e1**(-1.0)*(1.+e2**(-1.0))
            box=e2**(-.5)+e1**(-.5)*(1+e2**(-1))
            cox=e2**(-.5)*(1+e1**(-.5)*(1+e1**(-.5)))
            ared=1+e2*(1+e1)
            bred=sqrt(e1)+sqrt(e2)*(1+e1)
            cred=sqrt(e1)*(1+sqrt(e2)*(1+sqrt(e2)))

            F(I,1)=beta(5)/beta(6)/aox
     :           *(beta(6)+log((aox+10**beta(3)*(box+beta(4)*cox))
     :           /((aox+(10**beta(3)*(box+beta(4)*cox))*exp(beta(6))))))
     :           -beta(5)*beta(4)/ared
     :           *(beta(6)+log((ared+10**beta(3)*(cred+beta(4)*bred))
     :           /(ared+(10**beta(3)*(cred+beta(4)*bred))*exp(beta(6)))))


 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_so22nc(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            a=1.0+e1*(1.+e2)
            y2=10**beta(4)*(sqrt(e2)+sqrt(e1))
     :           +10**beta(3)*(sqrt(e1)*e2)
c     beta(3 et 4) = log10(k2/k0M)
c     beta(5) = ilim/beta d_0
c     beta(6) = cst, pas besoin de ligne de base
            F(I,1)=beta(5)/a*log((a+y2)/(y2))+beta(6)
 100     CONTINUE
      END IF
      RETURN
      END



************************************************************************

      SUBROUTINE fcn_soa1nc(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            a=1.0 ! pas de terme nerstien
            y2=(sqrt(e1))
c     beta(2) = ilim/beta d_0
c     beta(3) = cst, pas besoin de ligne de base
            F(I,1)=beta(2)/a*log((a+y2)/(y2))+beta(3)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_soa2nc(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            a=1.0+e1
            y2=10**beta(2)*(sqrt(e1))
c     beta(2) = log10(k2/k0M)
c     beta(3) = ilim/beta d_0
c     beta(4) = cst, pas besoin de ligne de base
            F(I,1)=beta(3)/a*log((a+y2)/(y2))+beta(4)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_soa3nc(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            a=1.0+e1
            y2=10**beta(2)*(e1)+10**beta(3)*(e1)**(3/2)

c     
            stop
c c'est faux!!! devrait tre 10**beta(2)*sqrt(e1)

c     beta(2) = log10(k2/k0M)1/2
c     beta(3) = log10(k2/k0M)3/2
c     beta(4) = ilim/beta d_0
c     beta(5) = cst, pas besoin de ligne de base
            F(I,1)=beta(4)/a*log((a+y2)/(y2))+beta(5)
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_soa4nc(N,M,NP,NQ, !
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) !
            e2=exp(fara*(beta(2)-xplusd(I,1)))
            a=1.0+e1*(1.+e2)
            y2=10**beta(3)*(sqrt(e1))+10**beta(4)*((e1)**(3/2))
c     beta(2) = log10(k2/k0M)1/2
c     beta(3) = log10(k2/k0M)3/2
c     beta(5) = ilim/beta d_0
c     beta(6) = cst, pas besoin de ligne de base
            F(I,1)=beta(5)/a*log((a+y2)/(y2))+beta(6)
 100     CONTINUE
      END IF
      RETURN
      END




************************************************************************
*     fit sloppy waves 1 electron
************************************************************************

      SUBROUTINE fcn_sr11l(N,M,NP,NQ, ! xqr2nb
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e2=exp(fara*(xplusd(I,1)-beta(2))) ! e1=0
            a=1.0+e2            !
            y2=10**beta(3)*(sqrt(e2)) ! 
c     beta(3)=log10(k2/k0M)
c     beta(5)=ilim
c     beta(4)=log10(beta d_0)
            y1=y2*exp(10**beta(4))  
            F(I,1)=beta(5)/a*(1.+(log((a+y2)/(a+y1))/10**beta(4)))
 100     CONTINUE
      END IF
      RETURN
      END
c     
      SUBROUTINE fcn_sr11n(N,M,NP,NQ, ! xqr2nbnl
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e2=exp(fara*(xplusd(I,1)-beta(2))) ! e1=0
            a=1.0+e2
            y2=10**beta(3)*sqrt(e2) 
c     beta(3) = log10(k2/k0M)
c     beta(4) = ilim/beta d_0
            F(I,1)=beta(4)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_so11l(N,M,NP,NQ, ! xqr2nb
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e2=0
            a=1.0+e1            !
            y2=10**beta(3)*(sqrt(e1)) ! 
c     beta(3)=log10(k2/k0M)
c     beta(5)=ilim
c     beta(4)=loag10(beta d_0)
            y1=y2*exp(10**beta(4))  
            F(I,1)=beta(5)/a*(1.+(log((a+y2)/(a+y1))/10**beta(4)))
 100     CONTINUE
      END IF
      RETURN
      END
c     
      SUBROUTINE fcn_so11n(N,M,NP,NQ, ! xqr2nbnl
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e2=0
            a=1.0+e1
            y2=10**beta(3)*(sqrt(e1))
c     beta(3) = log10(k2/k0M)
c     beta(4) = ilim/beta d_0
            F(I,1)=beta(4)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE fcn_so11nf(N,M,NP,NQ, ! pour fb2, e1/2 non flottant
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y2,e1f
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            e1=exp(fara*(beta(1)-xplusd(I,1))) ! e2=0
            e1f=exp(fara*(-.251-xplusd(I,1))) ! pour fcb2
            a=1.0+e1
            y2=10**beta(3)*(sqrt(e1f))
c     beta(3) = log10(k2/k0M)
c     beta(4) = ilim/beta d_0
            F(I,1)=beta(4)/a*log((a+y2)/(y2))
 100     CONTINUE
      END IF
      RETURN
      END


************************************************************************
*     fit waves with apparent number of electrons
************************************************************************

      SUBROUTINE fcn_woa(N,M,NP,NQ, 
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)/(1.+exp(beta(2)*fara*(beta(3)-XPLUSD(I,1))))
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_wra(N,M,NP,NQ, 
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,e1,e2,a,y1,y2
      common /temperature/ temp
      ISTOP=0
      fara=96500./8.3144/temp
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=beta(1)/(1.+exp(beta(2)*fara*(XPLUSD(I,1)-beta(3))))
 100     CONTINUE
      END IF
      RETURN
      END
c
      SUBROUTINE fcn_wc(N,M,NP,NQ, 
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real temp,fara,phiox,phired,sumcato,sumano
      integer ishape
      common /temperature/ temp
      common /ishape_wc/ ISHAPE
c     ISHAPE:
c     12
c     1 =0/1/2 no cat current/ cat current/ w/ switch
c     2 =0/1/2 no ano current/ ano current/ w/ switch
c     beta(1) 2  3     4      5    6     7     8      9     10
c     ilimc nswc Eswc, q cat, napp Ecat, ilima, nswa, Eswa, qann

      ISTOP=0
      fara=96500./8.3144/temp
c      write(6,*) MOD(ISHAPE,1)
      do i=1,N
         IF (MOD(ISHAPE/10,10).GE.1) THEN ! cat current
            IF (MOD(ISHAPE/10,10).GE.2) THEN ! cat sw
               phired=1./(1.+exp(fara*beta(2)*(xplusd(I,1)-beta(3)))) ! catodic switch
               sumcato=beta(1)/(1+exp(fara*beta(5)*(xplusd(I,1)-beta(6))))!
     :              *((1-phired)+phired*(1-beta(4)))
            ELSE                ! normal cathodic wave 
               sumcato=beta(1)/(1+exp(fara*beta(5)*(xplusd(I,1)-beta(6))))!
            ENDIF
         ELSE
            sumcato=0.
         ENDIF
         IF (MOD(ISHAPE,10).GE.1) THEN ! anodic current
            IF (MOD(ISHAPE,10).GE.2) THEN ! anodic sw
               phiox=1.-1./(1.+exp(fara*beta(8)*(xplusd(I,1)-beta(9))))
               sumano=beta(7)/(1+exp(fara*beta(5)*(beta(6)-xplusd(I,1))))!
     :              *((1-beta(10))*phiox+(1-phiox))
            ELSE                ! normal anodic wave
               sumano=beta(7)/(1+exp(fara*beta(5)*(beta(6)-xplusd(I,1))))!
            ENDIF
         ELSE
            sumano=0
         ENDIF
         F(I,1)=sumano+sumcato
      enddo
      RETURN
      END
c     
************************************************************************
*     non catalytic functions
************************************************************************

      SUBROUTINE fcn_nca(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real xmin, xmax
      if (Xplusd(1,1).lt.Xplusd(N,1)) then
         xmin=Xplusd(1,1)
         xmax=Xplusd(N,1)
      else
         xmin=Xplusd(N,1)
         xmax=Xplusd(1,1)
      endif
c     test for unacceptable parameters
      ISTOP=0
      do k= 1,(NP-1)/3,3        ! each center
         if (
c     :        (beta(1+(3*k-3)+1).le.0) ! ns < 0
c     :        .or.
     :        (beta(1+(3*k-3)+2).le.0) ! napp < 0
     :        .or.(beta(1+(3*k-3)+3).lt.xmin) ! out of plot
     :        .or.(beta(1+(3*k-3)+3).gt.xmax)
     :        ) then
            ISTOP=1
            return
         endif
      enddo
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=0.
            do k = 1,(NP-1)/3,1 ! sum over each center
               F(I,1)=F(I,1)+centeri(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1)) ! potential
            enddo
            F(I,1)=F(I,1)*beta(1)
 100     CONTINUE
      END IF
C     COMPUTE DERIVATIVES WITH RESPECT TO BETA
      IF (MOD(IDEVAL/10,10).GE.1) THEN
         DO 200 I = 1,N
            FJACB(I,1,1)=0.
            do k = 1,(NP-1)/3,1 ! three der for each center
c     der WRT beta(1)
               FJACB(I,1,1)=FJACB(I,1,1)+centeri(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1))
c     der WRT nsi = beta(1+(3*k-3)+1)
               FJACB(I,1+(3*k-3)+1,1)=dcenteri_dnsi(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1))
     :              *beta(1)
c     der WRT nappi = beta(1+(3*k-3)+2
               FJACB(I,1+(3*k-3)+2,1)=dcenteri_dnappi(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1))
     :              *beta(1)
c     der WRT ei = beta(1+(3*k-3)+3
               FJACB(I,1+(3*k-3)+3,1)=dcenteri_dei(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1))
     :              *beta(1)
            enddo
 200     CONTINUE
      END IF
C     COMPUTE DERIVATIVES WRT DELTA
      IF (MOD(IDEVAL/100,10).GE.1) THEN
         DO 300 I = 1,N
            FJACD(I,1,1) = 0.
            do k = 1,(NP-1)/3,1 ! sum over each center
               FJACD(I,1,1) = FJACD(I,1,1) +
     :              dcenteri_dx(
     :              beta(1+(3*k-3)+1), ! nsi
     :              beta(1+(3*k-3)+2), ! nappi
     :              beta(1+(3*k-3)+3), ! ei
     :              xplusd(I,1))
            enddo
            FJACD(I,1,1) = FJACD(I,1,1) * beta(1)
 300     CONTINUE
      END IF
      RETURN
      END
c     current for one center
      function centeri(nsi,nappi,ei,x)
      real nsi,nappi,ei,x,temp,fara,centeri
      common /temperature/ temp
      fara=96500/8.3144/temp
      centeri=nsi*nappi*exp(nappi*fara*(x-ei))/
     :     (1+exp(nappi*fara*(x-ei)))**2
      return
      end
c     derivative WRT to nsi
      function dcenteri_dnsi(nsi,nappi,ei,x)
      real nsi,nappi,ei,x,temp,fara,dcenteri_dnsi
      common /temperature/ temp
      fara=96500/8.3144/temp
      dcenteri_dnsi=nappi*exp(nappi*fara*(x-ei))/
     :     (1+exp(nappi*fara*(x-ei)))**2
      return
      end
c     derivative WRT nappi
      function dcenteri_dnappi(nsi,nappi,ei,x)
      real nsi,nappi,ei,x,temp,fara,dcenteri_dnappi
      common /temperature/ temp
      fara=96500/8.3144/temp
c     fortran(diff(nsi*nappi*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x-ei)))**2,nappi));
      dcenteri_dnappi=
     #nsi*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x-ei)))**2+nsi*
     #nappi*fara*(x-ei)*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x-ei))
     #)**2-2*nsi*nappi*exp(nappi*fara*(x-ei))**2/(1+exp(nappi*fara*(x-ei
     #)))**3*fara*(x-ei)
      return
      end
c     derivative WRT ei
      function dcenteri_dei(nsi,nappi,ei,x)
c     fortran(diff(nsi*nappi*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x-ei)))**2,ei));
      real nsi,nappi,ei,x,temp,fara,dcenteri_dei
      common /temperature/ temp
      fara=96500/8.3144/temp
      dcenteri_dei = -nsi*nappi**2*fara*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(!
     #x-ei)))**2+2*nsi*nappi**2*exp(nappi*fara*(x-ei))**2/(1+exp(nappi*f
     #ara*(x-ei)))**3*fara
      return
      end
c     derivative WRT x
      function dcenteri_dx(nsi,nappi,ei,x)
c     fortran(diff(nsi*nappi*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x-ei)))**2,x)   );
      real nsi,nappi,ei,x,temp,fara,dcenteri_dx
      common /temperature/ temp
      fara=96500/8.3144/temp
      dcenteri_dx = nsi*nappi**2*fara*exp(nappi*fara*(x-ei))/(1+exp(nappi*fara*(x!
     #-ei)))**2-2*nsi*nappi**2*exp(nappi*fara*(x-ei))**2/(1+exp(nappi*fa
     #ra*(x-ei)))**3*fara
      return
      end

************************************************************************

      SUBROUTINE fcn_ncl(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real xmin, xmax
      if (Xplusd(1,1).lt.Xplusd(N,1)) then
         xmin=Xplusd(1,1)
         xmax=Xplusd(N,1)
      else
         xmin=Xplusd(N,1)
         xmax=Xplusd(1,1)
      endif

c     test for unacceptable parameters
      ISTOP=0
      do k= 1,(NP-1)/3,3        ! each center
         if ((beta(1+(3*k-3)+3).lt.xmin) ! E0(center k) out of plot
     :        .or.(beta(1+(3*k-3)+3).gt.xmax)
     :        ) then
            ISTOP=1
            return
         endif
      enddo
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=0.
            do k = 1,(NP-1)/3,1 ! sum over each center
               if (beta(1+(3*k-3)+1).eq.1.) then
                  F(I,1)=F(I,1)+centeri(
     :                 1.,      ! nsi
     :                 1.,      ! nappi
     :                 beta(1+(3*k-3)+3), ! ei
     :                 xplusd(I,1)) ! potential
               elseif (beta(1+(3*k-3)+1).eq.2.) then
                  F(I,1)=F(I,1)+centeri2(
     :                 beta(1+(3*k-3)+2), ! E01-E02
     :                 beta(1+(3*k-3)+3), ! ei
     :                 xplusd(I,1)) ! potential
               else
                  ISTOP=-1      ! should stop
                  return
               endif
            enddo
            F(I,1)=F(I,1)*beta(1)
 100     CONTINUE
      ENDIF
      RETURN
      END
c     current for one 2-e center according to laviron
      function centeri2(de,ei,x)
      real de,ei,x,temp,fara,centeri2,xi,ka
      common /temperature/ temp
      fara=96500/8.3144/temp
      xi=exp(2.*fara*(x-ei))
      ka=exp(fara*(de))         ! de < 0 means crossed.
      centeri2=sqrt(ka)
     :     *(sqrt(xi)+4./sqrt(ka)+1./sqrt(xi))
     :     /(sqrt(xi)+sqrt(ka)+1./sqrt(xi))**2
      return
      end
      
      SUBROUTINE fcn_ncI(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real xmin, xmax
      if (Xplusd(1,1).lt.Xplusd(N,1)) then
         xmin=Xplusd(1,1)
         xmax=Xplusd(N,1)
      else
         xmin=Xplusd(N,1)
         xmax=Xplusd(1,1)
      endif
c     test for unacceptable parameters
      ISTOP=0
      do k= 1,(NP)/3,3        ! each center
         if ((beta(3*k-3+2).lt.0) ! napp < 0
     :        .or.(beta(3*k-3+3).lt.xmin) ! out of plot
     :        .or.(beta(3*k-3+3).gt.xmax)
     :        ) then
            ISTOP=1
            return
         endif
      enddo
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=0.
            do k = 1,(NP)/3,1 ! sum over each center
               F(I,1)=F(I,1)+beta(3*k-3+1)*centeri(
     :              1., ! nsi
     :              beta(3*k-3+2), ! nappi
     :              beta(3*k-3+3), ! ei
     :              xplusd(I,1)) ! potential
            enddo
 100     CONTINUE
      END IF
      return
      end

      SUBROUTINE fcn_ncc2(N,M,NP,NQ,
     +     LDN,LDM,LDNP,
     +     BETA,XPLUSD,
     +     IFIXB,IFIXX,LDIFX,
     +     IDEVAL,F,FJACB,FJACD,
     +     ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real xmin, xmax
      if (Xplusd(1,1).lt.Xplusd(N,1)) then
         xmin=Xplusd(1,1)
         xmax=Xplusd(N,1)
      else
         xmin=Xplusd(N,1)
         xmax=Xplusd(1,1)
      endif
c     test for unacceptable parameters
      ISTOP=0
      a=96500/8.3144/278!T=5
      do k= 2,3                 ! each cpot
         if ((beta(k).lt.xmin)  ! out of plot
     :        .or.(beta(k).gt.xmax)
     :        ) then
            ISTOP=1
            return
         endif
      enddo
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         DO 100 I = 1,N         ! n points
            F(I,1)=0.
            e1=exp(a*(xplusd(I,1)-beta(2)))
            e2=exp(a*(xplusd(I,1)-beta(3)))
            F(I,1)=beta(1)*(2*a*e1/(e1+1+1./e2)
     :           -(2*e1+1)*(a*e1-a/e2)/
     :           (e1+1+1/e2)**2
     :           )
 100     CONTINUE
      END IF
      return
      end

************************************************************************
*     SWV
************************************************************************

      SUBROUTINE fcn_sw(N,M,NP,NQ,
     +               LDN,LDM,LDNP,
     +               BETA,XPLUSD,
     +               IFIXB,IFIXX,LDIFX,
     +               IDEVAL,F,FJACB,FJACD,
     +               ISTOP)
      INTEGER          I,IDEVAL,ISTOP,L,LDIFX,LDM,LDN,LDNP,M,N,NP,NQ
      REAL             BETA(NP),XPLUSD(LDN,M)
      INTEGER          IFIXB(NP),IFIXX(LDIFX,M)
      REAL             F(LDN,NQ),FJACB(LDN,LDNP,NQ),FJACD(LDN,LDM,NQ)
      real halfperiod, samplingtime, oxold(9)!, oxnew1(9), oxnew2(9)
      real fara, temp, forwi, backwi!, kox, kred, time, elecpot
      real xmin,xmax
      common /temperature/ temp
      fara=96500/8.3144/temp
c     test for unacceptable parameters      
      ISTOP=0
      if (Xplusd(1,1).lt.Xplusd(N,1)) then
         xmin=Xplusd(1,1)
         xmax=Xplusd(N,1)
      else
         xmin=Xplusd(N,1)
         xmax=Xplusd(1,1)
      endif
      do k= 1,(NP-4)/2,1        ! each center
         if (
     :        (beta(4+2*k-2+2).lt.xmin) ! out of plot
     :        .or.(beta(4+2*k-2+2).gt.xmax)
     :        ) then
            ISTOP=1
            return
         endif
      enddo
      if (beta(2).le.0) then
         ISTOP=1
         return
      endif
C     COMPUTE FUNCTION
      IF (MOD(IDEVAL,10).GE.1) THEN
         halfperiod=.5*beta(3)**(-1)  ! seconds
         if (halfperiod.le.40.e-3) then
            samplingtime=halfperiod/2.
         else
            samplingtime=20.e-3
         endif
         do k = 1,(NP-4)/2,1
c     initial condition at starting potential
            oxold(k)=1.-1./(1.+exp(fara*(Xplusd(1,1)-beta(4+2*(k-1)+2)))) !
         enddo
         F(1,1)=0.
         DO 100 I = 2,N         ! n-1 points
            F(I,1)=0.
            do k = 1,(NP-4)/2,1 ! sum over each center
c$$$      if logk0(2-nc)=-100, then k0(1) is used
c$$$      modify everywhere swvstep is called.
               if (beta(4+2*(k-1)+1).eq.-100.) then
                  call swvstep(oxold(k),beta(4+1),xplusd(I,1),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               else
                  call swvstep(oxold(k),beta(4+2*(k-1)+1),xplusd(I,1),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               endif
c$$$               call swvstep(oxold(k),beta(4+2*(k-1)+1),xplusd(I,1),
c$$$     :              beta(4),beta(4+2*(k-1)+2),beta(2),
c$$$     :              halfperiod,samplingtime,fara,forwi,backwi)
c     depending on (beta(1)) increments n/f/b current for species k in F(I,1)
               if (beta(1).eq.-1.) then ! back
                  F(I,1)=F(I,1)+backwi
               elseif (beta(1).eq.1.) then ! forw
                  F(I,1)=F(I,1)+forwi
               elseif (beta(1).eq.0.) then ! forw-backw
                  F(I,1)=F(I,1)+(forwi-backwi)
               else
                  ISTOP=-1
                  return
               endif
            enddo               ! boucle sur species
 100     CONTINUE               ! boucle sur sweep.
      END IF
      return
      end
c 
c$$$        call swvstep(oxold(k),beta(4+2*(k-1)+1),xplusd(I,1),beta(4),
c$$$     :              beta(4+2*(k-1)+2),beta(2),
c$$$     :              halfperiod,samplingtime,fara,forwi,backwi)
      subroutine swvstep(oxold,logk0,estep,eamp,
     :     e0,coverage,halfperiod,samplingtime,fara,forwi,backwi) !
      real oxold,logk0,estep,eamp,
     :     e0,coverage,halfperiod,samplingtime,fara,forwi,backwi!
      real oxnew1,oxnew2,kox,kred
c     forward step
      elecpot=estep+eamp/2.
c     compute rate constants kf kb for species k at potential estep+EAMP
      kox=10**logk0*exp(fara*.5*(elecpot-e0)) !
      kred=10**logk0*exp(fara*.5*(-elecpot+e0)) !
c     beginning of sampling period
      time=halfperiod-samplingtime
      oxnew1=(kox+((kred+kox)*oxold-kox)
     :     *exp(-(kox+kred)*time))/(kox+kred) !
c     end of sampling period
      time=halfperiod
      oxnew2=(kox+((kred+kox)*oxold-kox)
     :     *exp(-(kox+kred)*time))/(kox+kred) !
c     calculate forward current
      forwi=coverage*(oxnew2-oxnew1)/samplingtime
c     initialise before backward step
      oxold=oxnew2
c     backward step
      elecpot=estep-eamp/2.
c     compute rate constants kf kb for species k at potential estep-EAMP
      kox=10**logk0*exp(fara*.5*(elecpot-e0)) !
      kred=10**logk0*exp(fara*.5*(-elecpot+e0)) !
c     beginning of sampling period
      time=halfperiod-samplingtime
      oxnew1=(kox+((kred+kox)*oxold-kox)
     :     *exp(-(kox+kred)*time))/(kox+kred) !
c     end of sampling period
      time=halfperiod
      oxnew2=(kox+((kred+kox)*oxold-kox)
     :     *exp(-(kox+kred)*time))/(kox+kred) !
c     calculate backward current
      backwi=coverage*(oxnew2-oxnew1)/samplingtime
c     init for next step
      oxold=oxnew2
      return
      end

************************************************************************
*     MISC, common to all fits
************************************************************************

      subroutine get_filename(leng,name,oleng,oname)
      character*128 name,oname,tempname ! o for output (fitres)
      character*1 answer
      integer leng,oleng,templeng,ia ! o for output
      logical ex,conf,ok
 1    format(a)                 !q,a)
      ok=.false.
      do while(.not.ok)
         oleng=leng+8
         oname(1:oleng)=name(1:leng)//'.fit.res'
         call prompt('Filename (['//oname(1:oleng)//'],q to abort) :') !
         read(5,1) tempname
         templeng=lnblnk(tempname)
         if (.not.((ileng.eq.1).and.(tempname(1:1).eq.'q'))) then
            if (templeng.eq.0) then
               continue         ! using default.
            else
               oleng=templeng
               oname(1:oleng)=tempname(1:templeng)
            endif
         else
            oleng=0
            oname=''
         endif
         ok=.true.
         inquire(file=oname(1:oleng),exist=ex) 
         if (ex) then           ! file exists
 305        format('WARNING : file ',A,' already exists.')
            write(6,305) oname(1:oleng)
            call confirm('overwrite',9,conf,.true.) !
            if (.not.conf) then
               call confirm('try a new filename',18,conf,.true.)
               if (conf) then
                  ok=.false.
               else
                  ok=.true.
                  oleng=0
                  oname=''      ! no name given, will abort.
               endif
            endif
         endif
      enddo                     ! if ok=.false. input filename again else exit
      return
      end

************************************************************************

      subroutine append_fit_2_name(name,leng)
      character*128 name
      integer leng
      name(1:leng+4)=name(1:leng)//'.fit'
      leng=leng+4
      return
      end

************************************************************************

      subroutine writeparamodr(beta,np,parametername,leng,
     :     name,lun,forceraw)   !
      integer np,lun
      real beta(*),temp
      character*128 parametername(*)
      integer leng,leng2
      character*128 name,tmpstring
      logical forceraw,ex
      common /temperature/ temp
c     write best parameters
 1    format(G12.4,' ',A)
 2    format(2(G12.4),' ',A)
 3    format(3(G12.4),' ',A)
 4    format(4(G12.4),' ',A)
 5    format(5(G12.4),' ',A)
 6    format(6(G12.4),' ',A)
 7    format(7(G12.4),' ',A)
 8    format(8(G12.4),' ',A)
 9    format(9(G12.4),' ',A)
 10   format(10(G12.4),' ',A)
 11   format(11(G12.4),' ',A)
 12   format(12(G12.4),' ',A)
 13   format(13(G12.4),' ',A)
 21   format(' ',1(A12),' name')
 22   format(' ',2(A12),' name')
 23   format(' ',3(A12),' name')
 24   format(' ',4(A12),' name')
 25   format(' ',5(A12),' name')
 26   format(' ',6(A12),' name')
 27   format(' ',7(A12),' name')
 28   format(' ',8(A12),' name')
 29   format(' ',9(A12),' name')
 30   format(' ',10(A12),' name')
 31   format(' ',11(A12),' name')
 32   format(' ',12(A12),' name')
 33   format(' ',13(A12),' name')

      write(lun,*)
 303  format(A)
      write(lun,303) 'Best parameters are :'
      if (forceraw) then
         if (np.eq.1) then
            Write(lun,21) (parametername(I)(1:12),I=1,NP)
            write(lun,1) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.2) then
            Write(lun,22) (parametername(I)(1:12),I=1,NP)
            write(lun,2) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.3) then
            Write(lun,23) (parametername(I)(1:12),I=1,NP)
            write(lun,3) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.4) then
            Write(lun,24) (parametername(I)(1:12),I=1,NP)
            write(lun,4) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.5) then
            Write(lun,25) (parametername(I)(1:12),I=1,NP)
            write(lun,5) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.6) then
            Write(lun,26) (parametername(I)(1:12),I=1,NP)
            write(lun,6) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.7) then
            Write(lun,27) (parametername(I)(1:12),I=1,NP)
            write(lun,7) (BETA(I),I=1,NP),name(1:leng)
         elseif (np.eq.8) then
            Write(lun,28) (parametername(I)(1:12),I=1,NP)
            write(lun,8) (BETA(I),I=1,NP),name(1:leng)
         else
            do i=1,NP
 400           format(G12.4,' ! ',A)
               write(lun,400,err=999) BETA(I),parametername(I)
            enddo
         endif
      else
         do i=1,NP
            if (parametername(I).ne.'unused') then
               tmpstring = parametername(i)
               leng2=lnblnk(tmpstring)
               write(lun,400,err=999) BETA(I),tmpstring(1:leng2)
            endif
         enddo
      endif
      return
 999  write(6,303) 'ERROR occured writting the parameters!'
      return
      end

************************************************************************
      
C     This subroutine takes care of writing the best fits parameters
C     to the common output file along with their respective
C     errors (??) and the x window of the fit (can always be useful...)
      subroutine auto_append_best_params(beta, np, variance, sq_sum,
     :     x_start, x_end)
      integer np, np2
      real beta(*), variance(*)
      real sq_sum, x_start, x_end
      real out (2 * np)
      integer i, ileng
      logical ex
      character*128 filename
      common /outpeak/ filename, ileng
      do i = 1, np
         out(2*i - 1) = beta(i)
         out(2*i) = variance(i)
      enddo

      inquire(file=filename(1:ileng),exist=ex)
      open(unit=9,file=filename(1:ileng),status='unknown',err=240)
      if (ex) then
         do i=1,10000
            read(9,*,end=210)
         enddo
 210     continue
      endif
      np2 = 2*np
      
      write(9, *) (beta(i), i=1,np), ' ! ', sq_sum, x_start, x_end
      close(9)

 306  format('INFO : parameters automatically saved in ',A)
      write(6,306) filename(1:ileng)
      return
 307  format('ERROR during opening/writing of ',A)
 240  write(6,307) filename(1:ileng)
      end

      subroutine AppendOutFileBestParam(beta,np,parametername,leng,name) !
      integer np
      real beta(*),temp
      character*128 parametername(*)
      integer leng
      character*128 name
      logical failed
      common /temperature/ temp

      character*128 filename
      integer ileng
      common /outpeak/ filename, ileng

      call open_output_file(failed)
C     Now unit 9 is the output file.
      if(.not. failed) then
 304     format('Buffer name is : ',A)
         write(9,304) name(1:leng)
         call writeparamodr(beta,np,parametername,leng,name,9,.false.)
C         endfile 9
         close(9)
 306     format('INFO : parameters saved in ',A)
         write(6,306) filename(1:ileng)
         return
      endif
      end

************************************************************************

      subroutine modifyParamODR(param,ia,npar,sign)
      integer npar,ia(*)       ! ,lhome
      real param(*),temp
      character*128 sign(*)
      character*128 tmp
      character*10 ch
      character*128 editor
      integer leditor,job,iprint
      logical ex,conf
      common /edit/ editor, leditor
      common /temperature/ temp
      common /JOB/ job,iprint
C      common /homeDir/ home,lhome
      character*128 homefile
 3    call confirm('modify initial parameters',25,conf,.false.)
      if (.not.conf) return
      inquire(file=homefile('gregtmp/soas_fit.tmp'),exist=ex) ! 
      if(ex) then
         call soas_system('rm -f '// homefile('gregtmp/soas_fit.tmp')) !
      endif
      open(unit=1,file=homefile('gregtmp/soas_fit.tmp'),
     :     status='unknown')   ! 
      do i=1,npar
 1       format(G12.4,' ',I2,' !',A)
         tmp = sign(i)
         write(1,1) param(i),ia(i), tmp(1:lnblnk(tmp)) !
      enddo
 11   format (G12.4,'  - !Temperature')
      write(1,11) temp
 111  format (I5,'         - !JOB')
      write(1,111) JOB
 112  format (I5,'         - !IPRINT')
      write(1,112) IPRINT
 303  format(A)
      write(1,303) 'Take care modifying this file.'//
     :     ' Do not add nor remove lines ' !
      write(1,303) 'The first col contains guessed values'//
     :     ' to be adjusted.'   !  
      write(1,303) 'The 2nd is 0 if parameter is fixed,' 
      write(1,303) ' - - - - - 1 - - - - - - - - free,' 
      write(1,303) 'Do not modify JOB/IPRINT unless you know'// 
     :     ' what you are doing.' ! 
      write(1,303) 'Using EMACS, exit typing Ctrl-X Ctrl-S'// 
     :     ' and Ctrl-X Ctrl-C' !
      close(1)
      write(6,303) 'Please modify the file containing the'//
     :     ' initial parameters.' !
      if (leditor.eq.0) then
         call soas_system('emacs -name Soas -geometry 80x20 -fn 7x14 '//
     :        homefile('gregtmp/soas_fit.tmp')) !
      else
         call soas_system(editor(1:leditor)//' '//
     :        homefile('gregtmp/soas_fit.tmp')) !
      endif
      open(unit=1,file=homefile('gregtmp/soas_fit.tmp'),
     :     status='unknown')    ! 
      do i=1,npar
         read(1,*,end=10,err=10) param(i),ia(i)
      enddo
      read(1,*,err=10,end=10) temp
      read(1,*,err=10,end=10) JOB
      read(1,*,err=10,end=10) IPRINT
      close(1)
      return
 10   write(6,303) 'ERROR during reading of new parameters,'
      write(6,303) 'I don''t know what''s going to happen !'
      return
      end

      subroutine readParamODR(param,ia,npar,sign)
      character*128 homefile
      integer npar,ia(*)
      real param(*),temp
      character*128 sign(*),sign2(28)
      character*10 ch
      character*128 editor
      integer leditor,job,iprint,ileng
      logical ex,conf
      common /edit/ editor, leditor
      common /temperature/ temp
      common /JOB/ job,iprint
 3    call confirm('read former parameters',22,conf,.false.)
      if (.not.conf) return
      inquire(file=homefile('gregtmp/soas_fit.tmp'),exist=ex) ! 
      if (.not.ex) then 
         write(6,*) "sorry, I cannot find the file"
         return
      else
         open(unit=1,file=homefile('gregtmp/soas_fit.tmp'),
     :        status='unknown') ! should be old
         do i=1,npar
            read(1,*,end=10,err=10) param(i),ia(i),sign2(i)
            ileng = lnblnk(sign2(i))
            write(6,*) param(i),ia(i),sign2(i)(1:ileng)
         enddo
         read(1,*,err=10,end=10) temp
         read(1,*,err=10,end=10) JOB
         read(1,*,err=10) IPRINT
         close(1)
         return
      endif
 303  format(A)
 10   write(6,303) 'ERROR during reading of parameters,'
      write(6,303) 'I don''t know what''s going to happen !'
      return
      end

      subroutine writeParamODRfile(param,ia,npar,sign)
      character*128 homefile
      integer npar,ia(*) !,lhome
      real param(*),temp
      character*128 sign(*)
      character*10 ch
      character*128 editor
      integer leditor,job,iprint
      logical ex,conf
      common /edit/ editor, leditor
      common /temperature/ temp
      common /JOB/ job,iprint
C      common /homeDir/ home,lhome
 3    call confirm('write these parameters for further use'
     :     ,38,conf,.false.)
      if (.not.conf) return
      inquire(file=homefile('gregtmp/soas_fit.tmp'),exist=ex) ! 
      open(unit=1,file=homefile('gregtmp/soas_fit.tmp'),
     :     status='unknown')    ! 
      do i=1,npar
 1       format(G12.4,' ',I2,' !',A)
         write(1,1) param(i),ia(i),sign(i) !
      enddo
 11   format (G12.4,'  - !Temperature')
      write(1,11) temp
 111  format (I5,'         - !JOB')
      write(1,111) JOB
 112  format (I5,'         - !IPRINT')
      write(1,112) IPRINT
 303  format(A)
      write(1,303) 'This file contains adjusted parameters'
      close(1)
      return 
      end



************************************************************************
c     non catalytic related, initialisation routines, plot, write
************************************************************************

      subroutine get_init_center_i(i,ns,napp,ei,ichoice)
c     if ichoice=1, ask for E, ns, napp
c     if ichoice=2, ask for E, ns. fit 2e with Laviron's equation
c     if ichoice=3, ask for napp
      integer ichoice
      real ns, napp, ei
      character*10 code
      real*8 xu,yu
      real*4 x4,y4
 303  format(A)
      write(6,*) 'Center ',i
      code(1:1)='-'
      write(6,303) 'Position'
      do while (code(1:1).ne.'^') ! position
         call gr_curs(xu,yu,x4,y4,code)
      enddo
      ei=xu
      write(6,*) ei
      if (ichoice.eq.1) then
         write(6,303) 'ns (0=free,1,2)'
         code(1:1)='-'
         do while ((code(1:1).ne.'0').and.(code(1:1).ne.'1')
     :        .and.(code(1:1).ne.'2')) ! ns
            call gr_curs(xu,yu,x4,y4,code)
         enddo
      elseif (ichoice.eq.2) then
         write(6,303) 'ns (1,2)'
         code(1:1)='-'
         do while ((code(1:1).ne.'1')
     :        .and.(code(1:1).ne.'2')) ! ns
            call gr_curs(xu,yu,x4,y4,code)
         enddo
      endif
      if ((ichoice.eq.1).or.(ichoice.eq.2)) then
         if (code(1:1).eq.'1') then
            ns=1.
         elseif (code(1:1).eq.'2') then
            ns=2.
         elseif (code(1:1).eq.'0') then
            ns=0.
         endif
         write(6,*) ns
      endif
      if ((ichoice.eq.1).or.(ichoice.eq.3)) then
         write(6,303) 'napp (0=free,1,2)'
         code(1:1)='-'
         do while ((code(1:1).ne.'0').and.(code(1:1).ne.'1')
     :        .and.(code(1:1).ne.'2')) ! NAPP
            call gr_curs(xu,yu,x4,y4,code) 
         enddo
         if (code(1:1).eq.'1') then
            napp=1.
         elseif (code(1:1).eq.'2') then
            napp=2.
         elseif (code(1:1).eq.'0') then
            napp=0.
         endif
         write(6,*) napp
      else                      ! ichoice=2, laviron
C     this will be either unused (IF ns=1) or E01-E02 (IF ns=2)
         if (ns.eq.1.) then
            napp=1
         elseif (ns.eq.2) then
            napp=-.05           ! volts, this is E01-E02
         endif
      endif
      return
      end

      subroutine write_all_yis_nc(beta,NP,X,Y,Y2,N,
     :     parametername,leng,name,oleng,oname,ifunc) !
      INTEGER          NP,leng,oleng,ifunc
c     ifunc=1: napp, ifunc=2: laviron, ifunc=3: independent coverages
      REAL             BETA(*),X(*),y(*),y2(*),yd(9) !!!
      character*128 parametername(*)
      character*128 name, oname
c      open(unit=1,file=name(1:leng)//'.fitres',status='unknown')
      open(unit=1,file=oname(1:oleng),status='unknown')
 303  format(A)
      write(1,303) 'Buffer name is : '//name(1:leng)
      write(1,*)
      call writeparamodr(beta,np,parametername,leng,name,1,.false.)
      write(1,*) 
      write(1,*) 'E            i exp       i fit       i_1 etc...'
      if ((ifunc.eq.1).or.(ifunc.eq.2)) then
         do i=1,N
            do k=1,(NP-1)/3     ! number of centers
               ys=0.
               if (((beta(1+(3*k-3)+1).eq.1).and.
     :              ((ifunc.eq.2))).or.(ifunc.eq.1)) then
                  yd(k)=beta(1)*centeri(
     :                 beta(1+(3*k-3)+1), ! nsi
     :                 beta(1+(3*k-3)+2), ! nappi
     :                 beta(1+(3*k-3)+3), ! ei
     :                 x(I))
               elseif ((beta(1+(3*k-3)+1).eq.2).and.(ifunc.eq.2)) then
                  yd(k)=beta(1)*centeri2(
     :                 beta(1+(3*k-3)+2), ! E01-E02
     :                 beta(1+(3*k-3)+3), ! ei
     :                 x(I))
               else             ! should not be here
                  write(6,303) 'ERROR in write_all_yis_nc'
               endif               
 300           format(12(G12.4))
            enddo
            write(1,300,err=999) X(I),Y(I),Y2(I),(yd(k),k=1,(NP-1)/3)
         enddo
      elseif ((ifunc.eq.3)) then
         do i=1,N
            do k=1,(NP)/3     ! number of centers
               ys=0.
               yd(k)=beta(3*k-3+1)*centeri(
     :              1.,        ! nsi
     :              beta(3*k-3+2), ! nappi
     :              beta(3*k-3+3), ! ei
     :              x(I))
            enddo
            write(1,300,err=999) X(I),Y(I),Y2(I),(yd(k),k=1,(NP)/3)
         enddo
      else                      ! should not be here
         write(6,303) 'ERROR in write_all_yis_nc'
      endif
 302  format('INFO : results written in file ',A)
      Write(6,302) oname(1:oleng)!//'.fitres'
 999  close(1)
      return
      end

      subroutine plot_all_yis_nc(beta,NP,X,N,ifunc) !
      parameter (NMAX=2**14)
      INTEGER          NP,leng,ifunc
c     ifunc=1: napp, ifunc=2: laviron
      REAL             BETA(*),X(*),y2(Nmax)
 303  format(A)
      if ((ifunc.eq.1).or.(ifunc.eq.2)) then
         do k=1,(NP-1)/3
            do i=1,N
               if (((beta(1+(3*k-3)+1).eq.1).and.
     :              ((ifunc.eq.2))).or.(ifunc.eq.1)) then
                  y2(i)=beta(1)*centeri(
     :                 beta(1+(3*k-3)+1), ! nsi
     :                 beta(1+(3*k-3)+2), ! nappi
     :                 beta(1+(3*k-3)+3), ! ei
     :                 x(I))
               elseif ((beta(1+(3*k-3)+1).eq.2).and.(ifunc.eq.2)) then
                  y2(i)=beta(1)*centeri2(
     :                 beta(1+(3*k-3)+2), ! E01-E02
     :                 beta(1+(3*k-3)+3), ! ei
     :                 x(I))
               else             ! should not be here
                  write(6,303) 'ERROR in plot_all_yis_nc'
                  return
               endif               
            enddo
            call plot(N,X,y2,-1)
         enddo
      elseif (ifunc.eq.3) then
         do k=1,(NP)/3
            do i=1,N
               y2(i)=beta(3*k-3+1)*centeri(
     :              1.,         ! nsi
     :              beta(3*k-3+2), ! nappi
     :              beta(3*k-3+3), ! ei
     :              x(I))
            enddo
            call plot(N,X,y2,-1)
         enddo
      else                      ! should not be here
         write(6,303) 'ERROR in plot_all_yis_nc'
         return
      endif               
      return
      end

************************************************************************
c     initialisation routine for fcn_wc
************************************************************************

      subroutine get_wave_parameters(NP,beta,IFIXB,ISHAPE,parametername)
      integer ISHAPE
      character*10 code
      character*128 parametername(*)
      real*8 xu,yu
      real*4 x4,y4,beta(*),IFIXB(*)
 303  format(A)
c     ISHAPE:
c     12
c     1 =0/1/2 no cat current/ cat current/ w/ switch
c     2 =0/1/2 no ano current/ ano current/ w/ switch
c     beta(1) 2  3     4      5    6     7     8      9     10
c     ilimc nswc Eswc, q cat, napp Ecat, ilima, nswa, Eswa, qann
      ISHAPE=0
      write(6,303) 'Click with left button on cathodic lim current'
      write(6,303) 'or right button to set it to zero'
      code(1:1)='0'
      do while((code(1:1).ne.'*').and.(code(1:1).ne.'^')) ! right or left
         call gr_curs(xu,yu,x4,y4,code)
      enddo
      if (code(1:1).eq.'*') then ! no cathodic current
         parametername(1)='unused'
         beta(1)=0
         ifixb(1)=0
         parametername(2)='unused'
         beta(2)=0
         ifixb(2)=0
         parametername(3)='unused'
         beta(3)=0
         ifixb(3)=0
         parametername(4)='unused'
         beta(4)=0
         ifixb(4)=0
c        ISHAPE=ISHAPE+0        ! 00
      else
         parametername(1)='ilim_cat'
         beta(1)=yu             ! ilimc
         ISHAPE=ISHAPE+10       ! 10
         write(6,303) 'Click with left button on cathodic Eswitch'
         write(6,303) 'or right button if there is not cathodic swirch'
         code(1:1)='0'
         do while((code(1:1).ne.'*').and.(code(1:1).ne.'^')) ! right or left
            call gr_curs(xu,yu,x4,y4,code)
         enddo
         if (code(1:1).eq.'*') then
            parametername(2)='unused'
            beta(2)=0.
            ifixb(2)=0
            parametername(3)='unused'
            beta(3)=0.
            ifixb(3)=0
            parametername(4)='unused'
            beta(4)=0.
            ifixb(4)=0
         else
            ISHAPE=ISHAPE+10    ! 20
            parametername(2)='n_sw_cat'
            beta(2)=1.5
            parametername(3)='E_sw_cat'
            beta(3)=xu
            parametername(4)='q_sw_cat'
            beta(4)=0.5
         endif
      endif  
c
      write(6,303) 'Click with left button on mid wave potential'
      code(1:1)='0'
      do while((code(1:1).ne.'^')) ! right or left
         call gr_curs(xu,yu,x4,y4,code)
      enddo
      parametername(6)='Ecat'
      beta(6)=xu
      parametername(5)='napp'
      beta(5)=1.5
c
      write(6,303) 'Click with left button on anodic lim current'
      write(6,303) 'or right button to set it to zero'
      code(1:1)='0'
      do while((code(1:1).ne.'*').and.(code(1:1).ne.'^')) ! right or left
         call gr_curs(xu,yu,x4,y4,code)
      enddo
      if (code(1:1).eq.'*') then ! no anodic current
         parametername(10)='unused'
         beta(10)=0.
         ifixb(10)=0
         parametername(9)='unused'
         beta(9)=0
         ifixb(9)=0
         parametername(8)='unused'
         beta(8)=0
         ifixb(8)=0
         parametername(7)='unused'
         beta(7)=0
         ifixb(7)=0
c        ISHAPE=ISHAPE+0        ! ?0
      else
         parametername(7)='ilim_an'
         beta(7)=yu             ! ilimc
         ISHAPE=ISHAPE+1        ! ?1
         write(6,303) 'Click with left button on anodic Eswitch'
         write(6,303) 'or right button if there is not cathodic swirch'
         code(1:1)='0'
         do while((code(1:1).ne.'*').and.(code(1:1).ne.'^')) ! right or left
            call gr_curs(xu,yu,x4,y4,code)
         enddo
         if (code(1:1).eq.'*') then
            parametername(8)='unused'
            beta(8)=0.
            ifixb(8)=0
            parametername(9)='unused'
            beta(9)=0.
            ifixb(9)=0
            parametername(10)='unused'
            beta(10)=0.
            ifixb(10)=0
         else
            ISHAPE=ISHAPE+2     ! ?2
            parametername(8)='n_sw_an'
            beta(8)=1.5
            parametername(9)='E_sw_an'
            beta(9)=xu
            parametername(10)='q_an'
            beta(10)=.5
         endif
      endif  
      return
      end

************************************************************************
c     SWV related, init, plot, write
************************************************************************

      subroutine get_init_sw(NP,beta,ifixb,parametername,name,leng)
      integer NP,ifixb(*)
      real*4 beta(*)
      character*1 chi
      character*128 parametername(*)
      character*128 name
      character*10 ch
      integer ileng,leng
      character*10 code
      real*8 xu,yu
      real*4 x4,y4
      logical conf

      beta(1)=.99
      if (name(leng-5:leng-4).eq.'.n') then
         beta(1)=0
      elseif  (name(leng-5:leng-4).eq.'.f') then
         beta(1)=1
      elseif  (name(leng-5:leng-4).eq.'.b') then
         beta(1)=-1
      endif
      do while (.not.((beta(1).eq.0).or.(beta(1).eq.-1).or.(beta(1).eq.1)))!
         call prompt('Net, forward or backward current (0/1/-1) ?')
         read(5,*) beta(1)
      enddo
      parametername(1)='nfb 0/1/-1'
      ifixb(1)=0

      beta(2)= 1.e-6            ! should calculate once, then divide to get right area.
      parametername(2)='FAGamma'

      
      beta(3)=-1
      do while (beta(3).le.0)   !
         call prompt('Frequency, Hz ?')
         read(5,*) beta(3)
      enddo
      parametername(3)='Frequency'
      ifixb(3)=0

      beta(4)=0
      do while (beta(4).eq.0)   !
         call prompt('Eamp  (V) ?')
         read(5,*) beta(4)
      enddo
      parametername(4)='Eamp'
      ifixb(4)=0


 200  format(I1)

      ileng=0

      if ((NP-4)/2.gt.1) then
 3       call confirm('all k0 values to be the same',28,conf,.false.)
         if (.not.conf) then    ! different k0 values
            do i=1,(NP-4)/2     ! each center
               write(chi,200) i
               beta(4+2*(i-1)+1)=2. ! k0
               parametername(4+2*(i-1)+1)='log10[k0('//chi(1:1)//')]' !
            enddo
         else                   ! same k0 values
            beta(4+1)=2.
            parametername(4+1)='log10[k0]' !
            do i=2,(NP-4)/2     ! each center from #2
               beta(4+2*(i-1)+1)=-100. ! dummy k0
               parametername(4+2*(i-1)+1)='unused'
               ifixb(4+2*(i-1)+1)=0
            enddo
            
         endif
      else                      ! only one center
         beta(4+1)=2.           ! k0
         parametername(4+1)='log10[k0(1)]' !
      endif         

 303  format(A)
      do i=1,(NP-4)/2
         write(6,*) 'Center ',i
         write(chi,200) i
         code(1:1)='-'
         write(6,303) 'Position'
         do while (code(1:1).ne.'^') ! position
            call gr_curs(xu,yu,x4,y4,code)
         enddo
         beta(4+2*(i-1)+2)=xu
         write(6,*) beta(4+2*(i-1)+2)
         parametername(4+2*(i-1)+2)='E0('//chi(1:1)//')' !
      enddo 
      return
      end

      subroutine plot_all_yis_sw(beta,NP,X,N) 
      parameter (NMAX=2**14)
      INTEGER          NP,leng
      REAL             BETA(*),X(*),y2(Nmax)
      real halfperiod, samplingtime, oxold(9) !, oxnew1(9), oxnew2(9)
      real fara, elecpot, forwi, backwi !, kox, kred, temp, time
      common /temperature/ temp
      fara=96500/8.3144/temp
      halfperiod=.5*beta(3)**(-1) ! seconds
      if (halfperiod.le.40.e-3) then
         samplingtime=halfperiod/2.
      else
         samplingtime=20.e-3
      endif
      do k = 1,(NP-4)/2,1
c     initial condition at starting potential
         oxold(k)=1.-1./(1.+exp(fara*(X(1)-beta(4+2*(k-1)+2)))) !
      enddo

      do k = 1,(NP-4)/2,1       ! sum over each center
         y2(1)=0.
         DO I = 2,N         ! n-1 points
            y2(i)=0
c$$$      if logk0(2-nc)=-100, then k0(1) is used
               if (beta(4+2*(k-1)+1).eq.-100) then
                  call swvstep(oxold(k),beta(4+1),x(I),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               else
                  call swvstep(oxold(k),beta(4+2*(k-1)+1),x(I),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               endif
c$$$            call swvstep(oxold(k),beta(4+2*(k-1)+1),x(I),
c$$$     :           beta(4),beta(4+2*(k-1)+2),beta(2),
c$$$     :           halfperiod,samplingtime,fara,forwi,backwi)
c     depending on (beta(1)) increments n/f/b current for species k in F(I,1)
            if (beta(1).eq.-1.) then ! back
               y2(I)=y2(I)+backwi
            elseif (beta(1).eq.1.) then ! forw
               y2(I)=y2(I)+forwi
            elseif (beta(1).eq.0.) then ! forw-backw
               y2(I)=y2(I)+(forwi-backwi)
            else
               return
            endif
         enddo                  ! boucle sur species
         call plot(N,X,y2,-1)
      enddo
      return
      end

      subroutine write_all_yis_sw(beta,NP,X,Y,Y2,N,parametername,
     :     leng,name,oleng,oname) !
      INTEGER          NP,oleng
      REAL             BETA(*),X(*),y(*),y2(*),yd(9) !!!
      character*128 parametername(*)
      character*128 name,oname
      real halfperiod, samplingtime, oxold(9) !, oxnew1(9), oxnew2(9)
      real fara, forwi, backwi !, kox, kred, temp, time
      common /temperature/ temp
      fara=96500/8.3144/temp

c     open(unit=1,file=name(1:leng)//'.fitres',status='unknown')
      open(unit=1,file=oname(1:oleng),status='unknown')
 303  format(A)
      write(1,303) 'Buffer name is : '//name(1:leng)
      write(1,*)
      call writeparamodr(beta,np,parametername,leng,name,1,.false.)
      write(1,*) 
      write(1,*) 'E            i exp       i fit       i_1 etc...'

      halfperiod=.5*beta(3)**(-1) ! seconds
      if (halfperiod.le.40.e-3) then
         samplingtime=halfperiod/2.
      else
         samplingtime=20.e-3
      endif

      do k = 1,(NP-4)/2,1
         oxold(k)=1.-1./(1.+exp(fara*(X(1)-beta(4+2*(k-1)+2)))) !
         yd(k)=0
      enddo
      write(1,300,err=999) X(1),Y(1),Y2(1),(yd(k),k=1,(NP-4)/2)

      DO I = 2,N                ! n-1 points
         do k = 1,(NP-4)/2,1    ! sum over each center
c$$$      if logk0(2-nc)=-100, then k0(1) is used
               if (beta(4+2*(k-1)+1).eq.-100) then
                  call swvstep(oxold(k),beta(4+1),x(I),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               else
                  call swvstep(oxold(k),beta(4+2*(k-1)+1),x(I),
     :              beta(4),beta(4+2*(k-1)+2),beta(2),
     :              halfperiod,samplingtime,fara,forwi,backwi)
               endif
c$$$            call swvstep(oxold(k),beta(4+2*(k-1)+1),x(I),
c$$$     :           beta(4),beta(4+2*(k-1)+2),beta(2),
c$$$     :           halfperiod,samplingtime,fara,forwi,backwi)
c     depending on (beta(1)) increments n/f/b current for species k in F(I,1)
            if (beta(1).eq.-1.) then ! back
               yd(k)=backwi
            elseif (beta(1).eq.1.) then ! forw
               yd(k)=forwi
            elseif (beta(1).eq.0.) then ! forw-backw
               yd(k)=(forwi-backwi)
            else
               return
            endif
         enddo                  ! boucle sur species

 300     format(12(G12.4))
         write(1,300,err=999) X(I),Y(I),Y2(I),(yd(k),k=1,(NP-4)/2)
      enddo
 302  format('INFO : results written in file ',A)
      Write(6,302) oname(1:oleng)!//'.fitres'
 999  close(1)
      return
      end


      subroutine writecovariancematrix(np, parametername, beta, vcv)
      parameter (maxnp=28) !,MAXN already set to 2**14
      real vcv(maxnp,maxnp)
      integer np
      real             beta(maxnp)
      character*128 parametername(maxnp)

      real variance(maxnp)
      real vcv2(maxnp,maxnp)
      
      
C     Let us try to estimate the 95% confidence interval for
C     the parameters. We assume, as stated in the ODRPACK manual
C     that if we have a reasonable number of degrees of freedom,
C     the student's t function will be, say, 2.5.

 344  format('Beware: The confidence limit, even if it is ',
     :     'likely to be good, might not be just as accurate...')
      write(6,344) 
 345  format(A12,' = ',G12.4, ' +/- ', G12.4, ' or +/-  ', F8.4,'%')
      do i=1,np
         variance(i) = vcv(i,i)**0.5 * 2.5
         write(6,345) parametername(i), beta(i), variance(i),
     :        variance(i) * 100/(beta(i))
      enddo
      
C     Outputting the covariance matrix normalized by the square
C     root of the diagonals.

C     We first actually normalize it:
      do i=1,np
         do j = 1,np
            vcv2(i,j) = vcv(i,j)/(vcv(i,i)**0.5 * vcv(j,j)**0.5)
         enddo
      enddo

 346  format('Covariance matrix:')
      write(6,346)
      do i=1,np
         write(6,*) (vcv2(i,j),j=1,i)
      enddo
      end
